@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -55,22 +55,17 @@ const char
     * const GridBase::META_FILE_MEM_BYTES = "file_mem_bytes",
     * const GridBase::META_FILE_VOXEL_COUNT = "file_voxel_count";
 
-namespace {
-/// @todo Remove (deprecated in favor of META_SAVE_HALF_FLOAT)
-const char *SAVE_FLOAT_AS_HALF = "write as 16-bit float";
-}
-
 
 ////////////////////////////////////////
 
 
 namespace {
 
-typedef std::map<Name, GridBase::GridFactory> GridFactoryMap;
-typedef GridFactoryMap::const_iterator GridFactoryMapCIter;
+using GridFactoryMap = std::map<Name, GridBase::GridFactory>;
+using GridFactoryMapCIter = GridFactoryMap::const_iterator;
 
-typedef tbb::mutex Mutex;
-typedef Mutex::scoped_lock Lock;
+using Mutex = tbb::mutex;
+using Lock = Mutex::scoped_lock;
 
 struct LockedGridRegistry {
     LockedGridRegistry() {}
@@ -88,9 +83,9 @@ getGridRegistry()
 {
     Lock lock(sInitGridRegistryMutex);
 
-    static LockedGridRegistry* registry = NULL;
+    static LockedGridRegistry* registry = nullptr;
 
-    if (registry == NULL) {
+    if (registry == nullptr) {
 
 #ifdef __ICC
 // Disable ICC "assignment to statically allocated variable" warning.
@@ -394,15 +389,10 @@ GridBase::setCreator(const std::string& creator)
 bool
 GridBase::saveFloatAsHalf() const
 {
-    bool saveAsHalf = false;
     if (Metadata::ConstPtr meta = (*this)[META_SAVE_HALF_FLOAT]) {
-        saveAsHalf = meta->asBool();
-    } else if ((*this)[SAVE_FLOAT_AS_HALF]) {
-        // Old behavior: saveAsHalf is true if metadata named
-        // SAVE_FLOAT_AS_HALF exists, regardless of its value.
-        saveAsHalf = true;
+        return meta->asBool();
     }
-    return saveAsHalf;
+    return false;
 }
 
 
@@ -411,9 +401,6 @@ GridBase::setSaveFloatAsHalf(bool saveAsHalf)
 {
     this->removeMeta(META_SAVE_HALF_FLOAT);
     this->insertMeta(META_SAVE_HALF_FLOAT, BoolMetadata(saveAsHalf));
-
-    // Remove the old, deprecated metadata.
-    this->removeMeta(SAVE_FLOAT_AS_HALF);
 }
 
 
@@ -465,12 +452,12 @@ GridBase::getStatsMetadata() const
         META_FILE_BBOX_MAX,
         META_FILE_MEM_BYTES,
         META_FILE_VOXEL_COUNT,
-        NULL
+        nullptr
     };
 
     /// @todo Check that the fields are of the correct type?
     MetaMap::Ptr ret(new MetaMap);
-    for (int i = 0; fields[i] != NULL; ++i) {
+    for (int i = 0; fields[i] != nullptr; ++i) {
         if (Metadata::ConstPtr m = (*this)[fields[i]]) {
             ret->insertMeta(fields[i], *m);
         }
@@ -482,7 +469,7 @@ GridBase::getStatsMetadata() const
 ////////////////////////////////////////
 
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
 void
 GridBase::clipGrid(const BBoxd& worldBBox)
 {
@@ -495,6 +482,6 @@ GridBase::clipGrid(const BBoxd& worldBBox)
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
