@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -38,15 +38,20 @@
 #include <openvdb/util/logging.h>
 
 // Boost.Interprocess uses a header-only portion of Boost.DateTime
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunused-macros"
+#endif
 #define BOOST_DATE_TIME_NO_LIB
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 #include <boost/interprocess/file_mapping.hpp>
 #include <boost/interprocess/mapped_region.hpp>
 #include <boost/iostreams/device/array.hpp>
 #include <boost/iostreams/stream.hpp>
-#include <boost/system/error_code.hpp>
 #include <boost/uuid/uuid_generators.hpp>
 #include <boost/uuid/uuid_io.hpp>
-#include <boost/version.hpp> // for BOOST_VERSION
 
 #include <tbb/atomic.h>
 
@@ -66,13 +71,13 @@ namespace boost { namespace interprocess { namespace detail {} namespace ipcdeta
 #include <cerrno> // for errno
 #include <cstdlib> // for getenv()
 #include <cstring> // for std::memcpy()
+#include <ctime> // for std::time()
 #include <iostream>
 #include <map>
+#include <random>
+#include <set>
 #include <sstream>
-
-#ifndef DWA_BOOST_VERSION
-#define DWA_BOOST_VERSION (10 * BOOST_VERSION)
-#endif
+#include <system_error> // for std::error_code()
 
 
 namespace openvdb {
@@ -136,7 +141,7 @@ StreamState::StreamState(): magicNumber(std::ios_base::xalloc())
         }
     }
 
-    if (existingArray >= 0 && std::cout.pword(existingArray) != NULL) {
+    if (existingArray >= 0 && std::cout.pword(existingArray) != nullptr) {
         // If a lower-numbered entry was found to contain the magic number,
         // a coexisting version of this library must have registered it.
         // In that case, the corresponding pointer should point to an existing
@@ -184,7 +189,7 @@ StreamState::~StreamState()
 {
     // Ensure that this StreamState struct can no longer be accessed.
     std::cout.iword(magicNumber) = 0;
-    std::cout.pword(magicNumber) = NULL;
+    std::cout.pword(magicNumber) = nullptr;
 }
 
 } // unnamed namespace
@@ -195,24 +200,16 @@ StreamState::~StreamState()
 
 struct StreamMetadata::Impl
 {
-    Impl()
-        : mFileVersion(OPENVDB_FILE_VERSION)
-        , mLibraryVersion(OPENVDB_LIBRARY_MAJOR_VERSION, OPENVDB_LIBRARY_MINOR_VERSION)
-        , mCompression(COMPRESS_NONE)
-        , mGridClass(GRID_UNKNOWN)
-        , mBackgroundPtr(NULL)
-        , mHalfFloat(false)
-        , mWriteGridStats(false)
-    {
-    }
-
-    uint32_t mFileVersion;
-    VersionId mLibraryVersion;
-    uint32_t mCompression;
-    uint32_t mGridClass;
-    const void* mBackgroundPtr; ///< @todo use Metadata::Ptr?
-    bool mHalfFloat;
-    bool mWriteGridStats;
+    uint32_t mFileVersion = OPENVDB_FILE_VERSION;
+    VersionId mLibraryVersion = { OPENVDB_LIBRARY_MAJOR_VERSION, OPENVDB_LIBRARY_MINOR_VERSION };
+    uint32_t mCompression = COMPRESS_NONE;
+    uint32_t mGridClass = GRID_UNKNOWN;
+    const void* mBackgroundPtr = nullptr; ///< @todo use Metadata::Ptr?
+    bool mHalfFloat = false;
+    bool mWriteGridStats = false;
+    bool mSeekable = false;
+    bool mCountingPasses = false;
+    uint32_t mPass = 0;
     MetaMap mGridMetadata;
     AuxDataMap mAuxData;
 }; // struct StreamMetadata
@@ -273,6 +270,9 @@ uint32_t        StreamMetadata::gridClass() const       { return mImpl->mGridCla
 const void*     StreamMetadata::backgroundPtr() const   { return mImpl->mBackgroundPtr; }
 bool            StreamMetadata::halfFloat() const       { return mImpl->mHalfFloat; }
 bool            StreamMetadata::writeGridStats() const  { return mImpl->mWriteGridStats; }
+bool            StreamMetadata::seekable() const        { return mImpl->mSeekable; }
+bool            StreamMetadata::countingPasses() const  { return mImpl->mCountingPasses; }
+uint32_t        StreamMetadata::pass() const            { return mImpl->mPass; }
 MetaMap&        StreamMetadata::gridMetadata()          { return mImpl->mGridMetadata; }
 const MetaMap&  StreamMetadata::gridMetadata() const    { return mImpl->mGridMetadata; }
 
@@ -286,18 +286,24 @@ void StreamMetadata::setGridClass(uint32_t c)           { mImpl->mGridClass = c;
 void StreamMetadata::setBackgroundPtr(const void* ptr)  { mImpl->mBackgroundPtr = ptr; }
 void StreamMetadata::setHalfFloat(bool b)               { mImpl->mHalfFloat = b; }
 void StreamMetadata::setWriteGridStats(bool b)          { mImpl->mWriteGridStats = b; }
-
+void StreamMetadata::setSeekable(bool b)                { mImpl->mSeekable = b; }
+void StreamMetadata::setCountingPasses(bool b)          { mImpl->mCountingPasses = b; }
+void StreamMetadata::setPass(uint32_t i)                { mImpl->mPass = i; }
 
 std::string
 StreamMetadata::str() const
 {
     std::ostringstream ostr;
+    ostr << std::boolalpha;
     ostr << "version: " << libraryVersion().first << "." << libraryVersion().second
         << "/" << fileVersion() << "\n";
     ostr << "class: " << GridBase::gridClassToString(static_cast<GridClass>(gridClass())) << "\n";
     ostr << "compression: " << compressionToString(compression()) << "\n";
-    ostr << "half_float: " << (halfFloat() ? "true" : "false") << "\n";
-    ostr << "write_grid_stats_metadata: " << (writeGridStats() ? "true" : "false") << "\n";
+    ostr << "half_float: " << halfFloat() << "\n";
+    ostr << "seekable: " << seekable() << "\n";
+    ostr << "pass: " << pass() << "\n";
+    ostr << "counting_passes: " << countingPasses() << "\n";
+    ostr << "write_grid_stats_metadata: " << writeGridStats() << "\n";
     if (!auxData().empty()) ostr << auxData();
     if (gridMetadata().metaCount() != 0) {
         ostr << "grid_metadata:\n" << gridMetadata().str(/*indent=*/"    ");
@@ -423,7 +429,7 @@ public:
 
         if (void* fh = open_existing_file(filename, boost::interprocess::read_only)) {
             struct { unsigned long lo, hi; } mtime; // Windows FILETIME struct
-            if (GetFileTime(fh, NULL, NULL, &mtime)) {
+            if (GetFileTime(fh, nullptr, nullptr, &mtime)) {
                 result = (Index64(mtime.hi) << 32) | mtime.lo;
             }
             close_file(fh);
@@ -469,7 +475,7 @@ MappedFile::filename() const
 }
 
 
-boost::shared_ptr<std::streambuf>
+SharedPtr<std::streambuf>
 MappedFile::createBuffer() const
 {
     if (!mImpl->mAutoDelete && mImpl->mLastWriteTime > 0) {
@@ -482,9 +488,9 @@ MappedFile::createBuffer() const
         }
     }
 
-    return boost::shared_ptr<std::streambuf>(
-        new boost::iostreams::stream_buffer<boost::iostreams::array_source>(
-            static_cast<const char*>(mImpl->mRegion.get_address()), mImpl->mRegion.get_size()));
+    return SharedPtr<std::streambuf>{
+        new boost::iostreams::stream_buffer<boost::iostreams::array_source>{
+            static_cast<const char*>(mImpl->mRegion.get_address()), mImpl->mRegion.get_size()}};
 }
 
 
@@ -498,7 +504,7 @@ MappedFile::setNotifier(const Notifier& notifier)
 void
 MappedFile::clearNotifier()
 {
-    mImpl->mNotifier.clear();
+    mImpl->mNotifier = nullptr;
 }
 
 
@@ -508,11 +514,7 @@ MappedFile::clearNotifier()
 std::string
 getErrorString(int errorNum)
 {
-#if DWA_BOOST_VERSION >= 1044000
-    return boost::system::error_code(errorNum, boost::system::generic_category()).message();
-#else
-    return boost::system::error_code(errorNum, boost::system::get_generic_category()).message();
-#endif
+    return std::error_code(errorNum, std::generic_category()).message();
 }
 
 
@@ -556,11 +558,7 @@ Archive::copy() const
 std::string
 Archive::getUniqueTag() const
 {
-    /// @todo Once versions of Boost < 1.44.0 are no longer in use,
-    /// this can be replaced with "return boost::uuids::to_string(mUuid);".
-    std::ostringstream ostr;
-    ostr << mUuid;
-    return ostr.str();
+    return boost::uuids::to_string(mUuid);
 }
 
 
@@ -716,7 +714,8 @@ Archive::setGridCompression(std::ostream& os, const GridBase& grid) const
             // ZLIB compression is not used on level sets or fog volumes.
             c = c & ~COMPRESS_ZIP;
             break;
-        default:
+        case GRID_STAGGERED:
+        case GRID_UNKNOWN:
             break;
     }
     io::setDataCompression(os, c);
@@ -855,7 +854,7 @@ StreamMetadata::Ptr
 clearStreamMetadataPtr(std::ios_base& strm)
 {
     StreamMetadata::Ptr result = getStreamMetadataPtr(strm);
-    strm.pword(sStreamState.metadata) = NULL;
+    strm.pword(sStreamState.metadata) = nullptr;
     return result;
 }
 
@@ -965,9 +964,9 @@ Archive::writeHeader(std::ostream& os, bool seekable) const
     //    (Omitted as of version 222)
 
     // 6) Generate a new random 16-byte (128-bit) uuid and write it to the stream.
-    boost::mt19937 ran;
-    ran.seed(static_cast<boost::mt19937::result_type>(time(NULL)));
-    boost::uuids::basic_random_generator<boost::mt19937> gen(&ran);
+    std::mt19937 ran;
+    ran.seed(std::mt19937::result_type(std::random_device()() + std::time(nullptr)));
+    boost::uuids::basic_random_generator<std::mt19937> gen(&ran);
     mUuid = gen(); // mUuid is mutable
     os << mUuid;
 }
@@ -1023,10 +1022,10 @@ Archive::connectInstance(const GridDescriptor& gd, const NamedGridMap& grids) co
 bool
 Archive::isDelayedLoadingEnabled()
 {
-#ifdef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER <= 2
     return false;
 #else
-    return (NULL == std::getenv("OPENVDB_DISABLE_DELAYED_LOAD"));
+    return (nullptr == std::getenv("OPENVDB_DISABLE_DELAYED_LOAD"));
 #endif
 }
 
@@ -1041,7 +1040,7 @@ doReadGrid(GridBase::Ptr grid, const GridDescriptor& gd, std::istream& is, const
 {
     struct Local {
         static void readBuffers(GridBase& g, std::istream& istrm, NoBBox) { g.readBuffers(istrm); }
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
         static void readBuffers(GridBase& g, std::istream& istrm, const CoordBBox& indexBBox) {
             g.readBuffers(istrm, indexBBox);
         }
@@ -1073,7 +1072,7 @@ doReadGrid(GridBase::Ptr grid, const GridDescriptor& gd, std::istream& is, const
     io::setStreamMetadataPtr(is, streamMetadata, /*transfer=*/false);
 
     io::setGridClass(is, GRID_UNKNOWN);
-    io::setGridBackgroundValuePtr(is, NULL);
+    io::setGridBackgroundValuePtr(is, nullptr);
 
     grid->readMeta(is);
 
@@ -1122,7 +1121,7 @@ Archive::readGrid(GridBase::Ptr grid, const GridDescriptor& gd, std::istream& is
     doReadGrid(grid, gd, is, NoBBox());
 }
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
 void
 Archive::readGrid(GridBase::Ptr grid, const GridDescriptor& gd,
     std::istream& is, const BBoxd& worldBBox)
@@ -1176,12 +1175,12 @@ Archive::write(std::ostream& os, const GridCPtrVec& grids, bool seekable,
     }
     os.write(reinterpret_cast<char*>(&gridCount), sizeof(int32_t));
 
-    typedef std::map<const TreeBase*, GridDescriptor> TreeMap;
-    typedef TreeMap::iterator TreeMapIter;
+    using TreeMap = std::map<const TreeBase*, GridDescriptor>;
+    using TreeMapIter = TreeMap::iterator;
     TreeMap treeMap;
 
     // Determine which grid names are unique and which are not.
-    typedef std::map<std::string, int /*count*/> NameHistogram;
+    using NameHistogram = std::map<std::string, int /*count*/>;
     NameHistogram nameCount;
     for (GridCPtrVecCIter i = grids.begin(), e = grids.end(); i != e; ++i) {
         if (const GridBase::ConstPtr& grid = *i) {
@@ -1295,9 +1294,9 @@ Archive::writeGrid(GridDescriptor& gd, GridBase::ConstPtr grid,
         grid->writeMeta(os);
     } else {
         // Compute and add grid statistics metadata.
-        GridBase::Ptr copyOfGrid = grid->copyGrid(); // shallow copy
-        copyOfGrid->addStatsMetadata();
-        copyOfGrid->insertMeta(GridBase::META_FILE_COMPRESSION,
+        const auto copyOfGrid = grid->copyGrid(); // shallow copy
+        ConstPtrCast<GridBase>(copyOfGrid)->addStatsMetadata();
+        ConstPtrCast<GridBase>(copyOfGrid)->insertMeta(GridBase::META_FILE_COMPRESSION,
             StringMetadata(compressionToString(getDataCompression(os))));
         copyOfGrid->writeMeta(os);
     }
@@ -1371,6 +1370,6 @@ Archive::writeGridInstance(GridDescriptor& gd, GridBase::ConstPtr grid,
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
