@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -31,18 +31,18 @@
 #ifndef OPENVDB_IO_ARCHIVE_HAS_BEEN_INCLUDED
 #define OPENVDB_IO_ARCHIVE_HAS_BEEN_INCLUDED
 
+#include <openvdb/version.h>
+#include "Compression.h" // for COMPRESS_ZIP, etc.
+#include <openvdb/Grid.h>
+#include <openvdb/MetaMap.h>
 #include <openvdb/Platform.h>
+#include <openvdb/version.h> // for VersionId
+#include <boost/uuid/uuid.hpp>
+#include <cstdint>
 #include <iosfwd>
 #include <map>
+#include <memory>
 #include <string>
-#include <boost/uuid/uuid.hpp>
-#include <boost/cstdint.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/shared_ptr.hpp>
-#include <openvdb/Grid.h>
-#include <openvdb/metadata/MetaMap.h>
-#include <openvdb/version.h> // for VersionId
-#include "Compression.h" // for COMPRESS_ZIP, etc.
 
 
 class TestFile;
@@ -59,12 +59,14 @@ class GridDescriptor;
 class OPENVDB_API Archive
 {
 public:
-    typedef boost::shared_ptr<Archive> Ptr;
-    typedef boost::shared_ptr<const Archive> ConstPtr;
+    using Ptr = SharedPtr<Archive>;
+    using ConstPtr = SharedPtr<const Archive>;
 
     static const uint32_t DEFAULT_COMPRESSION_FLAGS;
 
     Archive();
+    Archive(const Archive&) = default;
+    Archive& operator=(const Archive&) = default;
     virtual ~Archive();
 
     /// @brief Return a copy of this archive.
@@ -156,7 +158,7 @@ protected:
 
     /// Populate the given grid from the input stream.
     static void readGrid(GridBase::Ptr, const GridDescriptor&, std::istream&);
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
     /// @brief Populate the given grid from the input stream, but only where it
     /// intersects the given world-space bounding box.
     static void readGrid(GridBase::Ptr, const GridDescriptor&, std::istream&, const BBoxd&);
@@ -165,7 +167,7 @@ protected:
     static void readGrid(GridBase::Ptr, const GridDescriptor&, std::istream&, const CoordBBox&);
 #endif
 
-    typedef std::map<Name /*uniqueName*/, GridBase::Ptr> NamedGridMap;
+    using NamedGridMap = std::map<Name /*uniqueName*/, GridBase::Ptr>;
 
     /// @brief If the grid represented by the given grid descriptor
     /// is an instance, connect it with its instance parent.
@@ -223,6 +225,6 @@ private:
 
 #endif // OPENVDB_IO_ARCHIVE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
