@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -27,15 +27,15 @@
 // LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
 //
 ///////////////////////////////////////////////////////////////////////////
-//
-/// @file File.cc
+
+/// @file io/File.cc
 
 #include "File.h"
 
 #include "TempFile.h"
 #include <openvdb/Exceptions.h>
 #include <openvdb/util/logging.h>
-#include <boost/cstdint.hpp>
+#include <cstdint>
 #include <boost/iostreams/copy.hpp>
 #ifndef _MSC_VER
 #include <sys/types.h>
@@ -83,7 +83,7 @@ struct File::Impl
         file.Archive::readGrid(grid, gd, file.inputStream());
     }
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
     static void unarchive(const File& file, GridBase::Ptr& grid,
         const GridDescriptor& gd, const CoordBBox& indexBBox)
     {
@@ -101,7 +101,7 @@ struct File::Impl
     {
         Index64 result = DEFAULT_COPY_MAX_BYTES;
         if (const char* s = std::getenv("OPENVDB_DELAYED_LOAD_COPY_MAX_BYTES")) {
-            char* endptr = NULL;
+            char* endptr = nullptr;
             result = std::strtoul(s, &endptr, /*base=*/10);
         }
         return result;
@@ -113,9 +113,9 @@ struct File::Impl
     // The memory-mapped file
     MappedFile::Ptr mFileMapping;
     // The buffer for the input stream, if it is a memory-mapped file
-    boost::shared_ptr<std::streambuf> mStreamBuf;
+    SharedPtr<std::streambuf> mStreamBuf;
     // The file stream that is open for reading
-    boost::scoped_ptr<std::istream> mInStream;
+    std::unique_ptr<std::istream> mInStream;
     // File-level stream metadata (file format, compression, etc.)
     StreamMetadata::Ptr mStreamMetadata;
     // Flag indicating if we have read in the global information (header,
@@ -175,10 +175,10 @@ File::operator=(const File& other)
 }
 
 
-boost::shared_ptr<Archive>
+SharedPtr<Archive>
 File::copy() const
 {
-    return boost::shared_ptr<Archive>(new File(*this));
+    return SharedPtr<Archive>{new File{*this}};
 }
 
 
@@ -303,8 +303,8 @@ File::open(bool delayLoad, const MappedFile::Notifier& notifier)
     mImpl->mInStream.reset();
 
     // Open the file.
-    boost::scoped_ptr<std::istream> newStream;
-    boost::shared_ptr<std::streambuf> newStreamBuf;
+    std::unique_ptr<std::istream> newStream;
+    SharedPtr<std::streambuf> newStreamBuf;
     MappedFile::Ptr newFileMapping;
     if (!delayLoad || !Archive::isDelayedLoadingEnabled()) {
         newStream.reset(new std::ifstream(
@@ -343,7 +343,7 @@ File::open(bool delayLoad, const MappedFile::Notifier& notifier)
         } catch (std::exception& e) {
             std::ostringstream ostr;
             ostr << "could not open file " << filename();
-            if (e.what() != NULL) ostr << " (" << e.what() << ")";
+            if (e.what() != nullptr) ostr << " (" << e.what() << ")";
             OPENVDB_THROW(IoError, ostr.str());
         }
     }
@@ -372,6 +372,7 @@ File::open(bool delayLoad, const MappedFile::Notifier& notifier)
     // Tag the input stream with the file format and library version numbers
     // and other metadata.
     mImpl->mStreamMetadata.reset(new StreamMetadata);
+    mImpl->mStreamMetadata->setSeekable(true);
     io::setStreamMetadataPtr(inputStream(), mImpl->mStreamMetadata, /*transfer=*/false);
     Archive::setFormatVersion(inputStream());
     Archive::setLibraryVersion(inputStream());
@@ -389,8 +390,8 @@ File::open(bool delayLoad, const MappedFile::Notifier& notifier)
         mImpl->mNamedGrids.clear();
 
         // Stream in the entire contents of the file and append all grids to mGrids.
-        const boost::int32_t gridCount = readGridCount(inputStream());
-        for (boost::int32_t i = 0; i < gridCount; ++i) {
+        const int32_t gridCount = readGridCount(inputStream());
+        for (int32_t i = 0; i < gridCount; ++i) {
             GridDescriptor gd;
             gd.read(inputStream());
 
@@ -531,7 +532,11 @@ File::readAllGridMetadata()
         // have already been streamed in and stored in mGrids.
         for (size_t i = 0, N = mImpl->mGrids->size(); i < N; ++i) {
             // Return copies of the grids, but with empty trees.
+#if OPENVDB_ABI_VERSION_NUMBER <= 3
             ret->push_back((*mImpl->mGrids)[i]->copyGrid(/*treePolicy=*/CP_NEW));
+#else
+            ret->push_back((*mImpl->mGrids)[i]->copyGridWithNewTree());
+#endif
         }
     } else {
         // Read just the metadata and transforms for all grids.
@@ -542,7 +547,11 @@ File::readAllGridMetadata()
             // (As of 0.98.0, at least, it would suffice to just const cast
             // the grid pointers returned by readGridPartial(), but shallow
             // copying the grids helps to ensure future compatibility.)
+#if OPENVDB_ABI_VERSION_NUMBER <= 3
             ret->push_back(grid->copyGrid(/*treePolicy=*/CP_NEW));
+#else
+            ret->push_back(grid->copyGridWithNewTree());
+#endif
         }
     }
     return ret;
@@ -571,55 +580,11 @@ File::readGridMetadata(const Name& name)
         const GridDescriptor& gd = it->second;
         ret = readGridPartial(gd, /*readTopology=*/false);
     }
+#if OPENVDB_ABI_VERSION_NUMBER <= 3
     return ret->copyGrid(/*treePolicy=*/CP_NEW);
-}
-
-
-////////////////////////////////////////
-
-
-GridBase::ConstPtr
-File::readGridPartial(const Name& name)
-{
-    if (!isOpen()) {
-        OPENVDB_THROW(IoError, filename() << " is not open for reading.");
-    }
-
-    GridBase::ConstPtr ret;
-    if (!inputHasGridOffsets()) {
-        // Retrieve the grid from mGrids, which should already contain
-        // the entire contents of the file.
-        if (GridBase::Ptr grid = readGrid(name)) {
-            ret = boost::const_pointer_cast<const GridBase>(grid);
-        }
-    } else {
-        NameMapCIter it = findDescriptor(name);
-        if (it == gridDescriptors().end()) {
-            OPENVDB_THROW(KeyError, filename() << " has no grid named \"" << name << "\"");
-        }
-
-        // Seek to and read in the grid from the file.
-        const GridDescriptor& gd = it->second;
-        ret = readGridPartial(gd, /*readTopology=*/true);
-
-        if (gd.isInstance()) {
-            NameMapCIter parentIt =
-                findDescriptor(GridDescriptor::nameAsString(gd.instanceParentName()));
-            if (parentIt == gridDescriptors().end()) {
-                OPENVDB_THROW(KeyError, "missing instance parent \""
-                    << GridDescriptor::nameAsString(gd.instanceParentName())
-                    << "\" for grid " << GridDescriptor::nameAsString(gd.uniqueName())
-                    << " in file " << filename());
-            }
-            if (GridBase::ConstPtr parent =
-                readGridPartial(parentIt->second, /*readTopology=*/true))
-            {
-                boost::const_pointer_cast<GridBase>(ret)->setTree(
-                    boost::const_pointer_cast<GridBase>(parent)->baseTreePtr());
-            }
-        }
-    }
-    return ret;
+#else
+    return ret->copyGridWithNewTree();
+#endif
 }
 
 
@@ -633,7 +598,7 @@ File::readGrid(const Name& name)
 }
 
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
 GridBase::Ptr
 File::readGrid(const Name& name, const BBoxd& bbox)
 {
@@ -642,7 +607,7 @@ File::readGrid(const Name& name, const BBoxd& bbox)
 #endif
 
 
-#ifdef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER <= 2
 GridBase::Ptr
 File::readGridByName(const Name& name, const BBoxd&)
 #else
@@ -654,7 +619,7 @@ File::readGridByName(const Name& name, const BBoxd& bbox)
         OPENVDB_THROW(IoError, filename() << " is not open for reading.");
     }
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
     const bool clip = bbox.isSorted();
 #endif
 
@@ -663,7 +628,7 @@ File::readGridByName(const Name& name, const BBoxd& bbox)
     // doesn't support random access), retrieve and return it.
     GridBase::Ptr grid = retrieveCachedGrid(name);
     if (grid) {
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
         if (clip) {
             grid = grid->deepCopyGrid();
             grid->clipGrid(bbox);
@@ -679,7 +644,7 @@ File::readGridByName(const Name& name, const BBoxd& bbox)
 
     // Seek to and read in the grid from the file.
     const GridDescriptor& gd = it->second;
-#ifdef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER <= 2
     grid = readGrid(gd);
 #else
     grid = (clip ? readGrid(gd, bbox) : readGrid(gd));
@@ -697,7 +662,7 @@ File::readGridByName(const Name& name, const BBoxd& bbox)
         }
 
         GridBase::Ptr parent;
-#ifdef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER <= 2
         parent = readGrid(parentIt->second);
 #else
         if (clip) {
@@ -751,7 +716,7 @@ File::readGridDescriptors(std::istream& is)
 
     gridDescriptors().clear();
 
-    for (boost::int32_t i = 0, N = readGridCount(is); i < N; ++i) {
+    for (int32_t i = 0, N = readGridCount(is); i < N; ++i) {
         // Read the grid descriptor.
         GridDescriptor gd;
         gd.read(is);
@@ -859,7 +824,7 @@ File::readGrid(const GridDescriptor& gd) const
 }
 
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
 GridBase::Ptr
 File::readGrid(const GridDescriptor& gd, const BBoxd& bbox) const
 {
@@ -923,6 +888,6 @@ File::endName() const
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
