@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -33,9 +33,12 @@
 
 #include "Math.h" // for math::isApproxEqual() and math::Tolerance()
 #include "Vec3.h"
-#include <ostream>
-#include <algorithm> // for min/max
-#include <boost/type_traits/is_integral.hpp>
+#include <algorithm> // for std::min(), std::max()
+#include <cmath> // for std::abs()
+#include <iostream>
+#include <limits>
+#include <type_traits>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -47,124 +50,116 @@ template<typename Vec3T>
 class BBox
 {
 public:
-    typedef Vec3T Vec3Type;
-    typedef Vec3T ValueType;
-    typedef Vec3T VectorType;
-    typedef typename Vec3Type::ValueType ElementType;
+    using Vec3Type = Vec3T;
+    using ValueType = Vec3T;
+    using VectorType = Vec3T;
+    using ElementType = typename Vec3Type::ValueType;
 
-    /// @brief Default constructor creates an invalid BBox
+    /// @brief The default constructor creates an invalid bounding box.
     BBox();
-
-    /// @brief Constructor based on a minimum and maximum point.
+    /// @brief Construct a bounding box that exactly encloses the given
+    /// minimum and maximum points.
     BBox(const Vec3T& xyzMin, const Vec3T& xyzMax);
-
-    /// @brief Constructor based on a minimum and maximum point.
-    /// If sorted is false the points will be sorted by x,y,z component.
+    /// @brief Construct a bounding box that exactly encloses the given
+    /// minimum and maximum points.
+    /// @details If @a sorted is false, sort the points by their
+    /// @e x, @e y and @e z components.
     BBox(const Vec3T& xyzMin, const Vec3T& xyzMax, bool sorted);
-
-    /// @brief Contruct a cubical BBox from a minimum coordinate and a
-    /// single edge length.
-    /// @note inclusive for integral <tt>ElementType</tt>s
+    /// @brief Contruct a cubical bounding box from a minimum coordinate
+    /// and an edge length.
+    /// @note Inclusive for integral <b>ElementType</b>s
     BBox(const Vec3T& xyzMin, const ElementType& length);
 
-    /// @brief Constructor based on a raw array of six points. If
-    /// sorted is false the points will be sorted by x,y,z component.
+    /// @brief Construct a bounding box that exactly encloses two points,
+    /// whose coordinates are given by an array of six values,
+    /// <i>x<sub>1</sub></i>, <i>y<sub>1</sub></i>, <i>z<sub>1</sub></i>,
+    /// <i>x<sub>2</sub></i>, <i>y<sub>2</sub></i> and <i>z<sub>2</sub></i>.
+    /// @details If @a sorted is false, sort the points by their
+    /// @e x, @e y and @e z components.
     explicit BBox(const ElementType* xyz, bool sorted = true);
 
-    /// @brief Copy constructor
-    BBox(const BBox& other);
+    BBox(const BBox&) = default;
+    BBox& operator=(const BBox&) = default;
 
-    /// @brief Sort the min/max by x,y,z component.
+    /// @brief Sort the mininum and maximum points of this bounding box
+    /// by their @e x, @e y and @e z components.
     void sort();
 
-    /// @brief Return a const reference to the minimum point of the BBox
+    /// @brief Return a const reference to the minimum point of this bounding box.
     const Vec3T& min() const { return mMin; }
-
-    /// @brief Return a const reference to the maximum point of the BBox
+    /// @brief Return a const reference to the maximum point of this bounding box.
     const Vec3T& max() const { return mMax; }
-
-    /// @brief Return a non-const reference to the minimum point of the BBox
+    /// @brief Return a non-const reference to the minimum point of this bounding box.
     Vec3T& min() { return mMin; }
-
-    /// @brief Return a non-const reference to the maximum point of the BBox
+    /// @brief Return a non-const reference to the maximum point of this bounding box.
     Vec3T& max() { return mMax; }
 
-    /// @brief Return true if the two BBox'es are identical
+    /// @brief Return @c true if this bounding box is identical to the given bounding box.
     bool operator==(const BBox& rhs) const;
-
-    /// @brief Return true if the two BBox'es are not identical
+    /// @brief Return @c true if this bounding box differs from the given bounding box.
     bool operator!=(const BBox& rhs) const { return !(*this == rhs); }
 
-    /// @brief Return true if the BBox is empty, i.e. has no
-    /// (positive) volume.
+    /// @brief Return @c true if this bounding box is empty, i.e., it has no (positive) volume.
     bool empty() const;
-
-    /// @brief Return true if the BBox has a (positive) volume.
+    /// @brief Return @c true if this bounding box has (positive) volume.
     bool hasVolume() const { return !this->empty(); }
-
-    /// @brief Return true if the BBox is valid, i.e. as a (positive) volume.
+    /// @brief Return @c true if this bounding box has (positive) volume.
     operator bool() const { return !this->empty(); }
 
-    /// @brief Return true if the all components of mMin <= mMax,
-    /// i.e. the volume is not negative.
-    /// @note For floating point values a tolerance is used for this test.
+    /// @brief Return @c true if all components of the minimum point are less than
+    /// or equal to the corresponding components of the maximum point.
+    /// @details This is equivalent to testing whether this bounding box has nonnegative volume.
+    /// @note For floating-point <b>ElementType</b>s a tolerance is used for this test.
     bool isSorted() const;
 
-    /// @brief Return the center point of the BBox
+    /// @brief Return the center point of this bounding box.
     Vec3d getCenter() const;
 
-    /// @brief Returns the extents of the BBox, i.e. the length per axis
-    /// for floating points values or number of grids per axis points
-    /// integral values.
-    /// @note inclusive for integral <tt>ElementType</tt>s
+    /// @brief Return the extents of this bounding box, i.e., the length along each axis.
+    /// @note Inclusive for integral <b>ElementType</b>s
     Vec3T extents() const;
-
-    /// @brief Return the volume spanned by this BBox.
-    ElementType volume() const { Vec3T e = this->extents(); return e[0] * e[1] * e[2]; }
-
-    /// Return the index (0, 1 or 2) of the longest axis.
+    /// @brief Return the index (0, 1 or 2) of the longest axis.
     size_t maxExtent() const { return MaxIndex(mMax - mMin); }
-
-    /// Return the index (0, 1 or 2) of the shortest axis.
+    /// @brief Return the index (0, 1 or 2) of the shortest axis.
     size_t minExtent() const { return MinIndex(mMax - mMin); }
 
-    /// Return @c true if point (x, y, z) is inside this bounding box.
+    /// @brief Return the volume enclosed by this bounding box.
+    ElementType volume() const { Vec3T e = this->extents(); return e[0] * e[1] * e[2]; }
+
+    /// @brief Return @c true if the given point is inside this bounding box.
     bool isInside(const Vec3T& xyz) const;
-
-    /// Return @c true if the given bounding box is inside this bounding box.
+    /// @brief Return @c true if the given bounding box is inside this bounding box.
     bool isInside(const BBox&) const;
-
-    /// Return @c true if the given bounding box overlaps with this bounding box.
+    /// @brief Return @c true if the given bounding box overlaps with this bounding box.
     bool hasOverlap(const BBox&) const;
+    /// @brief Return @c true if the given bounding box overlaps with this bounding box.
+    bool intersects(const BBox& other) const { return hasOverlap(other); }
 
-    /// Pad this bounding box.
+    /// @brief Pad this bounding box.
     void expand(ElementType padding);
-
-    /// Expand this bounding box to enclose point (x, y, z).
+    /// @brief Expand this bounding box to enclose the given point.
     void expand(const Vec3T& xyz);
-
-    /// Union this bounding box with the given bounding box.
+    /// @brief Union this bounding box with the given bounding box.
     void expand(const BBox&);
-    // @brief Union this bbox with the cubical bbox defined from xyzMin and
-    // length
-    /// @note inclusive for integral <tt>ElementType</tt>s
+    /// @brief Union this bounding box with the cubical bounding box with
+    /// minimum point @a xyzMin and the given edge length.
+    /// @note Inclusive for integral <b>ElementType</b>s
     void expand(const Vec3T& xyzMin, const ElementType& length);
 
-    /// Translate this bounding box by \f$(t_x, t_y, t_z)\f$.
+    /// @brief Translate this bounding box by
+    /// (<i>t<sub>x</sub></i>, <i>t<sub>y</sub></i>, <i>t<sub>z</sub></i>).
     void translate(const Vec3T& t);
 
-    /// Apply a map to this bounding box
+    /// @brief Apply a map to this bounding box.
     template<typename MapType>
     BBox applyMap(const MapType& map) const;
-
-     /// Apply the inverse of a map to this bounding box
+     /// @brief Apply the inverse of a map to this bounding box
     template<typename MapType>
     BBox applyInverseMap(const MapType& map) const;
 
-    /// Unserialize this bounding box from the given stream.
+    /// @brief Unserialize this bounding box from the given stream.
     void read(std::istream& is) { mMin.read(is); mMax.read(is); }
-
-    /// Serialize this bounding box to the given stream.
+    /// @brief Serialize this bounding box to the given stream.
     void write(std::ostream& os) const { mMin.write(os); mMax.write(os); }
 
 private:
@@ -204,7 +199,7 @@ BBox<Vec3T>::BBox(const Vec3T& xyzMin, const ElementType& length):
     mMin(xyzMin), mMax(xyzMin)
 {
     // min and max are inclusive for integral ElementType
-    const ElementType size = boost::is_integral<ElementType>::value ? length-1 : length;
+    const ElementType size = std::is_integral<ElementType>::value ? length-1 : length;
     mMax[0] += size;
     mMax[1] += size;
     mMax[2] += size;
@@ -220,14 +215,6 @@ BBox<Vec3T>::BBox(const ElementType* xyz, bool sorted):
 }
 
 
-template<typename Vec3T>
-inline
-BBox<Vec3T>::BBox(const BBox& other):
-    mMin(other.mMin), mMax(other.mMax)
-{
-}
-
-
 ////////////////////////////////////////
 
 
@@ -235,7 +222,7 @@ template<typename Vec3T>
 inline bool
 BBox<Vec3T>::empty() const
 {
-    if (boost::is_integral<ElementType>::value) {
+    if (std::is_integral<ElementType>::value) {
         // min and max are inclusive for integral ElementType
         return (mMin[0] > mMax[0] || mMin[1] > mMax[1] || mMin[2] > mMax[2]);
     }
@@ -247,7 +234,7 @@ template<typename Vec3T>
 inline bool
 BBox<Vec3T>::operator==(const BBox& rhs) const
 {
-    if (boost::is_integral<ElementType>::value) {
+    if (std::is_integral<ElementType>::value) {
         return mMin == rhs.min() && mMax == rhs.max();
     } else {
         return math::isApproxEqual(mMin, rhs.min()) && math::isApproxEqual(mMax, rhs.max());
@@ -271,7 +258,7 @@ template<typename Vec3T>
 inline bool
 BBox<Vec3T>::isSorted() const
 {
-    if (boost::is_integral<ElementType>::value) {
+    if (std::is_integral<ElementType>::value) {
         return (mMin[0] <= mMax[0] && mMin[1] <= mMax[1] && mMin[2] <= mMax[2]);
     } else {
         ElementType t = math::Tolerance<ElementType>::value();
@@ -292,7 +279,7 @@ template<typename Vec3T>
 inline Vec3T
 BBox<Vec3T>::extents() const
 {
-    if (boost::is_integral<ElementType>::value) {
+    if (std::is_integral<ElementType>::value) {
         return (mMax - mMin) + Vec3T(1, 1, 1);
     } else {
         return (mMax - mMin);
@@ -306,7 +293,7 @@ template<typename Vec3T>
 inline bool
 BBox<Vec3T>::isInside(const Vec3T& xyz) const
 {
-    if (boost::is_integral<ElementType>::value) {
+    if (std::is_integral<ElementType>::value) {
         return xyz[0] >= mMin[0] && xyz[0] <= mMax[0] &&
                xyz[1] >= mMin[1] && xyz[1] <= mMax[1] &&
                xyz[2] >= mMin[2] && xyz[2] <= mMax[2];
@@ -323,7 +310,7 @@ template<typename Vec3T>
 inline bool
 BBox<Vec3T>::isInside(const BBox& b) const
 {
-    if (boost::is_integral<ElementType>::value) {
+    if (std::is_integral<ElementType>::value) {
         return b.min()[0] >= mMin[0]  && b.max()[0] <= mMax[0] &&
                b.min()[1] >= mMin[1]  && b.max()[1] <= mMax[1] &&
                b.min()[2] >= mMin[2]  && b.max()[2] <= mMax[2];
@@ -340,7 +327,7 @@ template<typename Vec3T>
 inline bool
 BBox<Vec3T>::hasOverlap(const BBox& b) const
 {
-    if (boost::is_integral<ElementType>::value) {
+    if (std::is_integral<ElementType>::value) {
         return mMax[0] >= b.min()[0] && mMin[0] <= b.max()[0] &&
                mMax[1] >= b.min()[1] && mMin[1] <= b.max()[1] &&
                mMax[2] >= b.min()[2] && mMin[2] <= b.max()[2];
@@ -393,7 +380,7 @@ template<typename Vec3T>
 inline void
 BBox<Vec3T>::expand(const Vec3T& xyzMin, const ElementType& length)
 {
-    const ElementType size = boost::is_integral<ElementType>::value ? length-1 : length;
+    const ElementType size = std::is_integral<ElementType>::value ? length-1 : length;
     for (int i = 0; i < 3; ++i) {
         mMin[i] = std::min(mMin[i], xyzMin[i]);
         mMax[i] = std::max(mMax[i], xyzMin[i] + size);
@@ -414,7 +401,7 @@ template<typename MapType>
 inline BBox<Vec3T>
 BBox<Vec3T>::applyMap(const MapType& map) const
 {
-    typedef Vec3<double> Vec3R;
+    using Vec3R = Vec3<double>;
     BBox<Vec3T> bbox;
     bbox.expand(map.applyMap(Vec3R(mMin[0], mMin[1], mMin[2])));
     bbox.expand(map.applyMap(Vec3R(mMin[0], mMin[1], mMax[2])));
@@ -432,7 +419,7 @@ template<typename MapType>
 inline BBox<Vec3T>
 BBox<Vec3T>::applyInverseMap(const MapType& map) const
 {
-    typedef Vec3<double> Vec3R;
+    using Vec3R = Vec3<double>;
     BBox<Vec3T> bbox;
     bbox.expand(map.applyInverseMap(Vec3R(mMin[0], mMin[1], mMin[2])));
     bbox.expand(map.applyInverseMap(Vec3R(mMin[0], mMin[1], mMax[2])));
@@ -462,6 +449,6 @@ operator<<(std::ostream& os, const BBox<Vec3T>& b)
 
 #endif // OPENVDB_MATH_BBOX_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
