@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -28,10 +28,10 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 
-/// @file   ConjGradient.h
+/// @file    ConjGradient.h
 /// @authors D.J. Hill, Peter Cucka
-/// @brief  Preconditioned conjugate gradient solver (solves @e Ax = @e b using
-///         the conjugate gradient method with one of a selection of preconditioners)
+/// @brief   Preconditioned conjugate gradient solver (solves @e Ax = @e b using
+///          the conjugate gradient method with one of a selection of preconditioners)
 
 #ifndef OPENVDB_MATH_CONJGRADIENT_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_CONJGRADIENT_HAS_BEEN_INCLUDED
@@ -41,8 +41,6 @@
 #include <openvdb/util/logging.h>
 #include <openvdb/util/NullInterrupter.h>
 #include "Math.h" // for Abs(), isZero(), Max(), Sqrt()
-#include <boost/shared_ptr.hpp>
-#include <boost/scoped_array.hpp>
 #include <tbb/parallel_for.h>
 #include <tbb/parallel_reduce.h>
 #include <algorithm> // for std::lower_bound()
@@ -50,6 +48,7 @@
 #include <cmath> // for std::isfinite()
 #include <limits>
 #include <sstream>
+#include <string>
 
 
 namespace openvdb {
@@ -58,9 +57,9 @@ namespace OPENVDB_VERSION_NAME {
 namespace math {
 namespace pcg {
 
-typedef Index32 SizeType;
+using SizeType = Index32;
 
-typedef tbb::blocked_range<SizeType> SizeRange;
+using SizeRange = tbb::blocked_range<SizeType>;
 
 template<typename ValueType> class Vector;
 
@@ -109,10 +108,10 @@ terminationDefaults()
 ///     <dt><i>iterations</i>
 ///     <dd>the maximum number of iterations, with or without convergence
 ///     <dt><i>relativeError</i>
-///     <dd>the relative error ||<i>b</i> &minus; <i>A</i>@f$\hat{x}@f$|| / ||<i>b</i>||
+///     <dd>the relative error ||<i>b</i> &minus; <i>Ax</i>|| / ||<i>b</i>||
 ///         that denotes convergence
 ///     <dt><i>absoluteError</i>
-///     <dd>the absolute error ||<i>b</i> &minus; <i>A</i>@f$\hat{x}@f$|| that denotes convergence
+///     <dd>the absolute error ||<i>b</i> &minus; <i>Ax</i>|| that denotes convergence
 ///
 /// @throw ArithmeticError if either @a x or @a b is not of the appropriate size.
 template<typename PositiveDefMatrix>
@@ -138,10 +137,10 @@ solve(
 ///     <dt><i>iterations</i>
 ///     <dd>the maximum number of iterations, with or without convergence
 ///     <dt><i>relativeError</i>
-///     <dd>the relative error ||<i>b</i> &minus; <i>A</i>@f$\hat{x}@f$|| / ||<i>b</i>||
+///     <dd>the relative error ||<i>b</i> &minus; <i>Ax</i>|| / ||<i>b</i>||
 ///         that denotes convergence
 ///     <dt><i>absoluteError</i>
-///     <dd>the absolute error ||<i>b</i> &minus; <i>A</i>@f$\hat{x}@f$|| that denotes convergence
+///     <dd>the absolute error ||<i>b</i> &minus; <i>Ax</i>|| that denotes convergence
 /// @param interrupter  an object adhering to the util::NullInterrupter interface
 ///     with which computation can be interrupted
 ///
@@ -166,17 +165,17 @@ template<typename T>
 class Vector
 {
 public:
-    typedef T ValueType;
-    typedef boost::shared_ptr<Vector> Ptr;
+    using ValueType = T;
+    using Ptr = SharedPtr<Vector>;
 
     /// Construct an empty vector.
-    Vector(): mData(NULL), mSize(0) {}
+    Vector(): mData(nullptr), mSize(0) {}
     /// Construct a vector of @a n elements, with uninitialized values.
     Vector(SizeType n): mData(new T[n]), mSize(n) {}
     /// Construct a vector of @a n elements and initialize each element to the given value.
     Vector(SizeType n, const ValueType& val): mData(new T[n]), mSize(n) { this->fill(val); }
 
-    ~Vector() { mSize = 0; delete[] mData; mData = NULL; }
+    ~Vector() { mSize = 0; delete[] mData; mData = nullptr; }
 
     /// Deep copy the given vector.
     Vector(const Vector&);
@@ -252,9 +251,8 @@ private:
     SizeType mSize;
 };
 
-typedef Vector<float> VectorS;
-typedef Vector<double> VectorD;
-//typedef Vector<double> LinearVector;
+using VectorS = Vector<float>;
+using VectorD = Vector<double>;
 
 
 ////////////////////////////////////////
@@ -266,9 +264,9 @@ template<typename ValueType_, SizeType STENCIL_SIZE>
 class SparseStencilMatrix
 {
 public:
-    typedef ValueType_ ValueType;
-    typedef Vector<ValueType> VectorType;
-    typedef boost::shared_ptr<SparseStencilMatrix> Ptr;
+    using ValueType = ValueType_;
+    using VectorType = Vector<ValueType>;
+    using Ptr = SharedPtr<SparseStencilMatrix>;
 
     class ConstValueIter;
     class ConstRow;
@@ -356,7 +354,7 @@ private:
     class RowBase
     {
     public:
-        typedef DataType_ DataType;
+        using DataType = DataType_;
 
         static SizeType capacity() { return STENCIL_SIZE; }
 
@@ -405,7 +403,7 @@ private:
         DataType mData;
     };
 
-    typedef RowBase<ConstRowData> ConstRowBase;
+    using ConstRowBase = RowBase<ConstRowData>;
 
 public:
     /// Iterator over the stored values in a row of this matrix
@@ -479,9 +477,9 @@ private:
     template<typename OtherValueType> struct EqOp;
 
     const SizeType                  mNumRows;
-    boost::scoped_array<ValueType>  mValueArray;
-    boost::scoped_array<SizeType>   mColumnIdxArray;
-    boost::scoped_array<SizeType>   mRowSizeArray;
+    std::unique_ptr<ValueType[]>    mValueArray;
+    std::unique_ptr<SizeType[]>     mColumnIdxArray;
+    std::unique_ptr<SizeType[]>     mRowSizeArray;
 }; // class SparseStencilMatrix
 
 
@@ -493,11 +491,11 @@ template<typename T>
 class Preconditioner
 {
 public:
-    typedef T ValueType;
-    typedef boost::shared_ptr<Preconditioner> Ptr;
+    using ValueType = T;
+    using Ptr = SharedPtr<Preconditioner>;
 
     template<SizeType STENCIL_SIZE> Preconditioner(const SparseStencilMatrix<T, STENCIL_SIZE>&) {}
-    virtual ~Preconditioner() {}
+    virtual ~Preconditioner() = default;
 
     virtual bool isValid() const { return true; }
 
@@ -659,36 +657,32 @@ Vector<T>::scale(const Scalar& s)
 template<typename T>
 struct Vector<T>::DeterministicDotProductOp
 {
-    DeterministicDotProductOp(const T* a_, const T* b_, 
-                              const SizeType binCount_, const SizeType arraySize_, T* reducetmp_): 
+    DeterministicDotProductOp(const T* a_, const T* b_,
+            const SizeType binCount_, const SizeType arraySize_, T* reducetmp_):
         a(a_), b(b_), binCount(binCount_), arraySize(arraySize_), reducetmp(reducetmp_) {}
-    
+
     void operator()(const SizeRange& range) const
     {
-        
-        const SizeType binSize = arraySize / binCount; 
-        
+        const SizeType binSize = arraySize / binCount;
+
         // Iterate over bins (array segments)
         for (SizeType n = range.begin(), N = range.end(); n < N; ++n) {
             const SizeType begin = n * binSize;
-            const SizeType end   = (n == binCount-1) ? arraySize : begin + binSize;
+            const SizeType end = (n == binCount-1) ? arraySize : begin + binSize;
 
-            // Compute the partial sum for this array segment 
+            // Compute the partial sum for this array segment
             T sum = zeroVal<T>();
-            for (SizeType i = begin; i < end; ++i) {
-                
-                sum += a[i] * b[i];
-            }
+            for (SizeType i = begin; i < end; ++i) { sum += a[i] * b[i]; }
             // Store the partial sum
             reducetmp[n] = sum;
         }
     }
 
-    
+
     const T* a;
     const T* b;
     const SizeType binCount;
-    const SizeType arraySize; 
+    const SizeType arraySize;
     T* reducetmp;
 };
 
@@ -715,21 +709,21 @@ Vector<T>::dot(const Vector<T>& other) const
 
     } else {
 
-        // Compute the dot product by segmenting the arrays into 
-        // a predetermined number of sub arrays in parallel and 
+        // Compute the dot product by segmenting the arrays into
+        // a predetermined number of sub arrays in parallel and
         // accumulate the finial result in series.
-        
+
         const SizeType binCount = 100;
         T partialSums[100];
-        
-        tbb::parallel_for(SizeRange(0, binCount), 
-                          DeterministicDotProductOp(aData, bData, binCount, arraySize, partialSums));
+
+        tbb::parallel_for(SizeRange(0, binCount),
+            DeterministicDotProductOp(aData, bData, binCount, arraySize, partialSums));
 
         for (SizeType n = 0; n < binCount; ++n) {
             result += partialSums[n];
         }
     }
-    
+
     return result;
 }
 
@@ -747,8 +741,6 @@ struct Vector<T>::InfNormOp
         return maxValue;
     }
 
-    static T join(T max1, T max2) { return Max(max1, max2); }
-
     const T* data;
 };
 
@@ -759,7 +751,7 @@ Vector<T>::infNorm() const
 {
     // Parallelize over the elements of this vector.
     T result = tbb::parallel_reduce(SizeRange(0, this->size()), /*seed=*/zeroVal<T>(),
-        InfNormOp(this->data()), InfNormOp::join);
+        InfNormOp(this->data()), /*join=*/[](T max1, T max2) { return Max(max1, max2); });
     return result;
 }
 
@@ -779,8 +771,6 @@ struct Vector<T>::IsFiniteOp
         return finite;
     }
 
-    static bool join(bool finite1, bool finite2) { return (finite1 && finite2); }
-
     const T* data;
 };
 
@@ -791,7 +781,8 @@ Vector<T>::isFinite() const
 {
     // Parallelize over the elements of this vector.
     bool finite = tbb::parallel_reduce(SizeRange(0, this->size()), /*seed=*/true,
-        IsFiniteOp(this->data()), IsFiniteOp::join);
+        IsFiniteOp(this->data()),
+        /*join=*/[](bool finite1, bool finite2) { return (finite1 && finite2); });
     return finite;
 }
 
@@ -812,8 +803,6 @@ struct Vector<T>::EqOp
         return equal;
     }
 
-    static bool join(bool eq1, bool eq2) { return (eq1 && eq2); }
-
     const T* a;
     const OtherValueType* b;
     const T eps;
@@ -827,7 +816,8 @@ Vector<T>::eq(const Vector<OtherValueType>& other, ValueType eps) const
 {
     if (this->size() != other.size()) return false;
     bool equal = tbb::parallel_reduce(SizeRange(0, this->size()), /*seed=*/true,
-        EqOp<OtherValueType>(this->data(), other.data(), eps), EqOp<OtherValueType>::join);
+        EqOp<OtherValueType>(this->data(), other.data(), eps),
+        /*join=*/[](bool eq1, bool eq2) { return (eq1 && eq2); });
     return equal;
 }
 
@@ -1036,8 +1026,6 @@ struct SparseStencilMatrix<ValueType, STENCIL_SIZE>::EqOp
         return equal;
     }
 
-    static bool join(bool eq1, bool eq2) { return (eq1 && eq2); }
-
     const SparseStencilMatrix* a;
     const SparseStencilMatrix<OtherValueType, STENCIL_SIZE>* b;
     const ValueType eps;
@@ -1052,7 +1040,8 @@ SparseStencilMatrix<ValueType, STENCIL_SIZE>::eq(
 {
     if (this->numRows() != other.numRows()) return false;
     bool equal = tbb::parallel_reduce(SizeRange(0, this->numRows()), /*seed=*/true,
-        EqOp<OtherValueType>(*this, other, eps), EqOp<OtherValueType>::join);
+        EqOp<OtherValueType>(*this, other, eps),
+        /*join=*/[](bool eq1, bool eq2) { return (eq1 && eq2); });
     return equal;
 }
 
@@ -1075,8 +1064,6 @@ struct SparseStencilMatrix<ValueType, STENCIL_SIZE>::IsFiniteOp
         return finite;
     }
 
-    static bool join(bool finite1, bool finite2) { return (finite1 && finite2); }
-
     const SparseStencilMatrix* mat;
 };
 
@@ -1087,7 +1074,7 @@ SparseStencilMatrix<ValueType, STENCIL_SIZE>::isFinite() const
 {
     // Parallelize over the rows of this matrix.
     bool finite = tbb::parallel_reduce(SizeRange(0, this->numRows()), /*seed=*/true,
-        IsFiniteOp(*this), IsFiniteOp::join);
+        IsFiniteOp(*this), /*join=*/[](bool finite1, bool finite2) { return (finite1&&finite2); });
     return finite;
 }
 
@@ -1324,10 +1311,10 @@ private:
     struct ApplyOp;
 
 public:
-    typedef typename MatrixType::ValueType ValueType;
-    typedef Preconditioner<ValueType> BaseType;
-    typedef Vector<ValueType> VectorType;
-    typedef boost::shared_ptr<JacobiPreconditioner> Ptr;
+    using ValueType = typename MatrixType::ValueType;
+    using BaseType = Preconditioner<ValueType>;
+    using VectorType = Vector<ValueType>;
+    using Ptr = SharedPtr<JacobiPreconditioner>;
 
     JacobiPreconditioner(const MatrixType& A): BaseType(A), mDiag(A.numRows())
     {
@@ -1335,9 +1322,9 @@ public:
         tbb::parallel_for(SizeRange(0, A.numRows()), InitOp(A, mDiag.data()));
     }
 
-    virtual ~JacobiPreconditioner() {}
+    ~JacobiPreconditioner() override = default;
 
-    virtual void apply(const Vector<ValueType>& r, Vector<ValueType>& z)
+    void apply(const Vector<ValueType>& r, Vector<ValueType>& z) override
     {
         const SizeType size = mDiag.size();
 
@@ -1390,13 +1377,13 @@ private:
     struct TransposeOp;
 
 public:
-    typedef typename MatrixType::ValueType ValueType;
-    typedef Preconditioner<ValueType> BaseType;
-    typedef Vector<ValueType> VectorType;
-    typedef boost::shared_ptr<IncompleteCholeskyPreconditioner> Ptr;
-    typedef SparseStencilMatrix<ValueType, 4>    TriangularMatrix;
-    typedef typename TriangularMatrix::ConstRow  TriangleConstRow;
-    typedef typename TriangularMatrix::RowEditor TriangleRowEditor;
+    using ValueType = typename MatrixType::ValueType;
+    using BaseType = Preconditioner<ValueType>;
+    using VectorType = Vector<ValueType>;
+    using Ptr = SharedPtr<IncompleteCholeskyPreconditioner>;
+    using TriangularMatrix = SparseStencilMatrix<ValueType, 4>;
+    using TriangleConstRow = typename TriangularMatrix::ConstRow;
+    using TriangleRowEditor = typename TriangularMatrix::RowEditor;
 
     IncompleteCholeskyPreconditioner(const MatrixType& matrix)
         : BaseType(matrix)
@@ -1491,11 +1478,11 @@ public:
             TransposeOp(matrix, mLowerTriangular, mUpperTriangular));
     }
 
-    virtual ~IncompleteCholeskyPreconditioner() {}
+    ~IncompleteCholeskyPreconditioner() override = default;
 
-    virtual bool isValid() const { return mPassedCompatibilityCondition; }
+    bool isValid() const override { return mPassedCompatibilityCondition; }
 
-    virtual void apply(const Vector<ValueType>& rVec, Vector<ValueType>& zVec)
+    void apply(const Vector<ValueType>& rVec, Vector<ValueType>& zVec) override
     {
         if (!mPassedCompatibilityCondition) {
             OPENVDB_THROW(ArithmeticError, "invalid Cholesky decomposition");
@@ -1671,8 +1658,8 @@ solve(
     Interrupter& interrupter,
     const State& termination)
 {
-    typedef typename PositiveDefMatrix::ValueType ValueType;
-    typedef Vector<ValueType> VectorType;
+    using ValueType = typename PositiveDefMatrix::ValueType;
+    using VectorType = Vector<ValueType>;
 
     State result;
     result.success = false;
@@ -1701,7 +1688,7 @@ solve(
 
     // Compute norm of B (the source)
     const ValueType tmp = bVec.infNorm();
-    const ValueType infNormOfB = isZero(tmp) ? 1.f : tmp;
+    const ValueType infNormOfB = isZero(tmp) ? ValueType(1) : tmp;
 
     // Compute rVec: residual = b - Ax.
     VectorType rVec(size); // vector of residuals
@@ -1812,7 +1799,7 @@ solve(
 
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
