@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -27,8 +27,8 @@
 // LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
 //
 ///////////////////////////////////////////////////////////////////////////
-//
-/// @file FiniteDifference.h
+
+/// @file math/FiniteDifference.h
 
 #ifndef OPENVDB_MATH_FINITEDIFFERENCE_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_FINITEDIFFERENCE_HAS_BEEN_INCLUDED
@@ -37,7 +37,7 @@
 #include "Math.h"
 #include "Coord.h"
 #include "Vec3.h"
-
+#include <string>
 #include <boost/algorithm/string/case_conv.hpp>
 #include <boost/algorithm/string/trim.hpp>
 
@@ -329,30 +329,31 @@ temporalIntegrationSchemeToMenuName(TemporalIntegrationScheme tis)
 template<typename ValueType>
 inline ValueType
 WENO5(const ValueType& v1, const ValueType& v2, const ValueType& v3,
-    const ValueType& v4, const ValueType& v5, float scale2 = 0.01)
+    const ValueType& v4, const ValueType& v5, float scale2 = 0.01f)
 {
-    static const double C=13.0/12.0;
-    // Weno is formulated for non-dimensional equations, here the optional scale2
+    const double C = 13.0 / 12.0;
+    // WENO is formulated for non-dimensional equations, here the optional scale2
     // is a reference value (squared) for the function being interpolated.  For
     // example if 'v' is of order 1000, then scale2 = 10^6 is ok.  But in practice
     // leave scale2 = 1.
-    const double eps=1e-6*scale2;
+    const double eps = 1.0e-6 * static_cast<double>(scale2);
     // {\tilde \omega_k} = \gamma_k / ( \beta_k + \epsilon)^2 in Shu's ICASE report)
     const double A1=0.1/math::Pow2(C*math::Pow2(v1-2*v2+v3)+0.25*math::Pow2(v1-4*v2+3.0*v3)+eps),
                  A2=0.6/math::Pow2(C*math::Pow2(v2-2*v3+v4)+0.25*math::Pow2(v2-v4)+eps),
                  A3=0.3/math::Pow2(C*math::Pow2(v3-2*v4+v5)+0.25*math::Pow2(3.0*v3-4*v4+v5)+eps);
 
-    return ValueType(A1*(2.0*v1 - 7.0*v2 + 11.0*v3) +
-                     A2*(5.0*v3 -     v2 +  2.0*v4) +
-                     A3*(2.0*v3 + 5.0*v4 -      v5))/(6.0*(A1+A2+A3));
+    return static_cast<ValueType>(static_cast<ValueType>(
+        A1*(2.0*v1 - 7.0*v2 + 11.0*v3) +
+        A2*(5.0*v3 -     v2 +  2.0*v4) +
+        A3*(2.0*v3 + 5.0*v4 -      v5))/(6.0*(A1+A2+A3)));
 }
 
 
 template <typename Real>
-inline Real GudonovsNormSqrd(bool isOutside,
-                     Real dP_xm, Real dP_xp,
-                     Real dP_ym, Real dP_yp,
-                     Real dP_zm, Real dP_zp)
+inline Real GodunovsNormSqrd(bool isOutside,
+                             Real dP_xm, Real dP_xp,
+                             Real dP_ym, Real dP_yp,
+                             Real dP_zm, Real dP_zp)
 {
     using math::Max;
     using math::Min;
@@ -375,9 +376,9 @@ inline Real GudonovsNormSqrd(bool isOutside,
 
 template<typename Real>
 inline Real
-GudonovsNormSqrd(bool isOutside, const Vec3<Real>& gradient_m, const Vec3<Real>& gradient_p)
+GodunovsNormSqrd(bool isOutside, const Vec3<Real>& gradient_m, const Vec3<Real>& gradient_p)
 {
-    return GudonovsNormSqrd<Real>(isOutside,
+    return GodunovsNormSqrd<Real>(isOutside,
                                   gradient_m[0], gradient_p[0],
                                   gradient_m[1], gradient_p[1],
                                   gradient_m[2], gradient_p[2]);
@@ -402,14 +403,14 @@ WENO5<simd::Float4>(const simd::Float4& v1, const simd::Float4& v2, const simd::
                     const simd::Float4& v4, const simd::Float4& v5, float scale2)
 {
     using math::Pow2;
-    typedef simd::Float4 F4;
+    using F4 = simd::Float4;
     const F4
-        C(13.0 / 12.0),
-        eps(1e-6 * scale2),
+        C(13.f / 12.f),
+        eps(1.0e-6f * scale2),
         two(2.0), three(3.0), four(4.0), five(5.0), fourth(0.25),
-        A1 = F4(0.1) / Pow2(C*Pow2(v1-two*v2+v3) + fourth*Pow2(v1-four*v2+three*v3) + eps),
-        A2 = F4(0.6) / Pow2(C*Pow2(v2-two*v3+v4) + fourth*Pow2(v2-v4) + eps),
-        A3 = F4(0.3) / Pow2(C*Pow2(v3-two*v4+v5) + fourth*Pow2(three*v3-four*v4+v5) + eps);
+        A1 = F4(0.1f) / Pow2(C*Pow2(v1-two*v2+v3) + fourth*Pow2(v1-four*v2+three*v3) + eps),
+        A2 = F4(0.6f) / Pow2(C*Pow2(v2-two*v3+v4) + fourth*Pow2(v2-v4) + eps),
+        A3 = F4(0.3f) / Pow2(C*Pow2(v3-two*v4+v5) + fourth*Pow2(three*v3-four*v4+v5) + eps);
     return (A1 * (two * v1 - F4(7.0) * v2 + F4(11.0) * v3) +
             A2 * (five * v3 - v2 + two * v4) +
             A3 * (two * v3 + five * v4 - v5)) / (F4(6.0) * (A1 + A2 + A3));
@@ -427,7 +428,7 @@ simdSum(const simd::Float4& v)
 }
 
 inline float
-GudonovsNormSqrd(bool isOutside, const simd::Float4& dP_m, const simd::Float4& dP_p)
+GodunovsNormSqrd(bool isOutside, const simd::Float4& dP_m, const simd::Float4& dP_p)
 {
     const simd::Float4 zero(0.0);
     simd::Float4 v = isOutside
@@ -437,6 +438,7 @@ GudonovsNormSqrd(bool isOutside, const simd::Float4& dP_m, const simd::Float4& d
 }
 #endif
 
+
 template<DScheme DiffScheme>
 struct D1
 {
@@ -845,12 +847,11 @@ struct D1<FD_3RD>
 {
 
     // the difference opperator
-    template <typename ValueType>
+    template<typename ValueType>
     static ValueType difference(const ValueType& xp3, const ValueType& xp2,
         const ValueType& xp1, const ValueType& xp0)
     {
-        return ValueType(1./3.)*xp3 - ValueType(1.5)*xp2
-            + ValueType(3.)*xp1 - ValueType(11./6.)*xp0;
+        return static_cast<ValueType>(xp3/3.0 - 1.5*xp2 + 3.0*xp1 - 11.0*xp0/6.0);
     }
 
 
@@ -1102,7 +1103,7 @@ struct D1<BD_3RD>
 template<>
 struct D1<FD_WENO5>
 {
-    // the difference opperator
+    // the difference operator
     template <typename ValueType>
     static ValueType difference(const ValueType& xp3, const ValueType& xp2,
                                 const ValueType& xp1, const ValueType& xp0,
@@ -1116,7 +1117,7 @@ struct D1<FD_WENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inX(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType  ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(3,0,0));
         V[1] = grid.getValue(ijk.offsetBy(2,0,0));
@@ -1131,7 +1132,7 @@ struct D1<FD_WENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inY(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(0,3,0));
         V[1] = grid.getValue(ijk.offsetBy(0,2,0));
@@ -1146,7 +1147,7 @@ struct D1<FD_WENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inZ(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(0,0,3));
         V[1] = grid.getValue(ijk.offsetBy(0,0,2));
@@ -1163,36 +1164,38 @@ struct D1<FD_WENO5>
     static typename Stencil::ValueType inX(const Stencil& S)
     {
 
-        return difference( S.template getValue< 3, 0, 0>(),
-                           S.template getValue< 2, 0, 0>(),
-                           S.template getValue< 1, 0, 0>(),
-                           S.template getValue< 0, 0, 0>(),
-                           S.template getValue<-1, 0, 0>(),
-                           S.template getValue<-2, 0, 0>() );
+        return static_cast<typename Stencil::ValueType>(difference(
+            S.template getValue< 3, 0, 0>(),
+            S.template getValue< 2, 0, 0>(),
+            S.template getValue< 1, 0, 0>(),
+            S.template getValue< 0, 0, 0>(),
+            S.template getValue<-1, 0, 0>(),
+            S.template getValue<-2, 0, 0>() ));
 
     }
 
     template<typename Stencil>
     static typename Stencil::ValueType inY(const Stencil& S)
     {
-        return difference( S.template getValue< 0, 3, 0>(),
-                           S.template getValue< 0, 2, 0>(),
-                           S.template getValue< 0, 1, 0>(),
-                           S.template getValue< 0, 0, 0>(),
-                           S.template getValue< 0,-1, 0>(),
-                           S.template getValue< 0,-2, 0>() );
+        return static_cast<typename Stencil::ValueType>(difference(
+            S.template getValue< 0, 3, 0>(),
+            S.template getValue< 0, 2, 0>(),
+            S.template getValue< 0, 1, 0>(),
+            S.template getValue< 0, 0, 0>(),
+            S.template getValue< 0,-1, 0>(),
+            S.template getValue< 0,-2, 0>() ));
     }
 
     template<typename Stencil>
     static typename Stencil::ValueType inZ(const Stencil& S)
     {
-
-        return difference( S.template getValue< 0, 0, 3>(),
-                           S.template getValue< 0, 0, 2>(),
-                           S.template getValue< 0, 0, 1>(),
-                           S.template getValue< 0, 0, 0>(),
-                           S.template getValue< 0, 0,-1>(),
-                           S.template getValue< 0, 0,-2>() );
+        return static_cast<typename Stencil::ValueType>(difference(
+            S.template getValue< 0, 0, 3>(),
+            S.template getValue< 0, 0, 2>(),
+            S.template getValue< 0, 0, 1>(),
+            S.template getValue< 0, 0, 0>(),
+            S.template getValue< 0, 0,-1>(),
+            S.template getValue< 0, 0,-2>() ));
     }
 };
 
@@ -1212,7 +1215,7 @@ struct D1<FD_HJWENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inX(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(3,0,0));
         V[1] = grid.getValue(ijk.offsetBy(2,0,0));
@@ -1228,7 +1231,7 @@ struct D1<FD_HJWENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inY(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(0,3,0));
         V[1] = grid.getValue(ijk.offsetBy(0,2,0));
@@ -1243,7 +1246,7 @@ struct D1<FD_HJWENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inZ(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(0,0,3));
         V[1] = grid.getValue(ijk.offsetBy(0,0,2));
@@ -1310,7 +1313,7 @@ struct D1<BD_WENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inX(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(-3,0,0));
         V[1] = grid.getValue(ijk.offsetBy(-2,0,0));
@@ -1325,7 +1328,7 @@ struct D1<BD_WENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inY(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(0,-3,0));
         V[1] = grid.getValue(ijk.offsetBy(0,-2,0));
@@ -1340,7 +1343,7 @@ struct D1<BD_WENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inZ(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(0,0,-3));
         V[1] = grid.getValue(ijk.offsetBy(0,0,-2));
@@ -1356,7 +1359,7 @@ struct D1<BD_WENO5>
     template<typename Stencil>
     static typename Stencil::ValueType inX(const Stencil& S)
     {
-        typedef typename Stencil::ValueType ValueType;
+        using ValueType = typename Stencil::ValueType;
         ValueType V[6];
         V[0] = S.template getValue<-3, 0, 0>();
         V[1] = S.template getValue<-2, 0, 0>();
@@ -1371,7 +1374,7 @@ struct D1<BD_WENO5>
     template<typename Stencil>
     static typename Stencil::ValueType inY(const Stencil& S)
     {
-        typedef typename Stencil::ValueType ValueType;
+        using ValueType = typename Stencil::ValueType;
         ValueType V[6];
         V[0] = S.template getValue< 0,-3, 0>();
         V[1] = S.template getValue< 0,-2, 0>();
@@ -1386,7 +1389,7 @@ struct D1<BD_WENO5>
     template<typename Stencil>
     static typename Stencil::ValueType inZ(const Stencil& S)
     {
-        typedef typename Stencil::ValueType ValueType;
+        using ValueType = typename Stencil::ValueType;
         ValueType V[6];
         V[0] = S.template getValue< 0, 0,-3>();
         V[1] = S.template getValue< 0, 0,-2>();
@@ -1414,7 +1417,7 @@ struct D1<BD_HJWENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inX(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(-3,0,0));
         V[1] = grid.getValue(ijk.offsetBy(-2,0,0));
@@ -1429,7 +1432,7 @@ struct D1<BD_HJWENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inY(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(0,-3,0));
         V[1] = grid.getValue(ijk.offsetBy(0,-2,0));
@@ -1444,7 +1447,7 @@ struct D1<BD_HJWENO5>
     template<typename Accessor>
     static typename Accessor::ValueType inZ(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType ValueType;
+        using ValueType = typename Accessor::ValueType;
         ValueType V[6];
         V[0] = grid.getValue(ijk.offsetBy(0,0,-3));
         V[1] = grid.getValue(ijk.offsetBy(0,0,-2));
@@ -1460,7 +1463,7 @@ struct D1<BD_HJWENO5>
     template<typename Stencil>
     static typename Stencil::ValueType inX(const Stencil& S)
     {
-        typedef typename Stencil::ValueType ValueType;
+        using ValueType = typename Stencil::ValueType;
         ValueType V[6];
         V[0] = S.template getValue<-3, 0, 0>();
         V[1] = S.template getValue<-2, 0, 0>();
@@ -1475,7 +1478,7 @@ struct D1<BD_HJWENO5>
     template<typename Stencil>
     static typename Stencil::ValueType inY(const Stencil& S)
     {
-        typedef typename Stencil::ValueType ValueType;
+        using ValueType = typename Stencil::ValueType;
         ValueType V[6];
         V[0] = S.template getValue< 0,-3, 0>();
         V[1] = S.template getValue< 0,-2, 0>();
@@ -1490,7 +1493,7 @@ struct D1<BD_HJWENO5>
     template<typename Stencil>
     static typename Stencil::ValueType inZ(const Stencil& S)
     {
-        typedef typename Stencil::ValueType ValueType;
+        using ValueType = typename Stencil::ValueType;
         ValueType V[6];
         V[0] = S.template getValue< 0, 0,-3>();
         V[1] = S.template getValue< 0, 0,-2>();
@@ -1658,7 +1661,7 @@ struct D1Vec<CD_2ND>
 
 template<>
 struct D1Vec<CD_4TH> {
-    // typedef typename Accessor::ValueType::value_type  value_type;
+    // using value_type = typename Accessor::ValueType::value_type;
 
 
     // random access version
@@ -1719,7 +1722,7 @@ struct D1Vec<CD_4TH> {
 template<>
 struct D1Vec<CD_6TH>
 {
-    //typedef typename Accessor::ValueType::value_type::value_type  ValueType;
+    //using ValueType = typename Accessor::ValueType::value_type::value_type;
 
     // random access version
     template<typename Accessor>
@@ -2001,7 +2004,7 @@ struct D2<CD_FOURTH>
     template<typename Accessor>
     static typename Accessor::ValueType inXandY(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType  ValueType;
+        using ValueType = typename Accessor::ValueType;
         typename Accessor::ValueType tmp1 =
             D1<CD_4TH>::inX(grid, ijk.offsetBy(0, 1, 0)) -
             D1<CD_4TH>::inX(grid, ijk.offsetBy(0,-1, 0));
@@ -2014,7 +2017,7 @@ struct D2<CD_FOURTH>
     template<typename Accessor>
     static typename Accessor::ValueType inXandZ(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType  ValueType;
+        using ValueType = typename Accessor::ValueType;
         typename Accessor::ValueType tmp1 =
             D1<CD_4TH>::inX(grid, ijk.offsetBy(0, 0, 1)) -
             D1<CD_4TH>::inX(grid, ijk.offsetBy(0, 0,-1));
@@ -2027,7 +2030,7 @@ struct D2<CD_FOURTH>
     template<typename Accessor>
     static typename Accessor::ValueType inYandZ(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType  ValueType;
+        using ValueType = typename Accessor::ValueType;
         typename Accessor::ValueType tmp1 =
             D1<CD_4TH>::inY(grid, ijk.offsetBy(0, 0, 1)) -
             D1<CD_4TH>::inY(grid, ijk.offsetBy(0, 0,-1));
@@ -2195,46 +2198,49 @@ struct D2<CD_SIXTH>
     template<typename Accessor>
     static typename Accessor::ValueType inXandY(const Accessor& grid, const Coord& ijk)
     {
-        typename Accessor::ValueType tmp1 =
+        using ValueT = typename Accessor::ValueType;
+        ValueT tmp1 =
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 1, 0)) -
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0,-1, 0));
-        typename Accessor::ValueType tmp2 =
+        ValueT tmp2 =
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 2, 0)) -
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0,-2, 0));
-        typename Accessor::ValueType tmp3 =
+        ValueT tmp3 =
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 3, 0)) -
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0,-3, 0));
-        return 0.75*tmp1 - 0.15*tmp2 + 1./60*tmp3;
+        return ValueT(0.75*tmp1 - 0.15*tmp2 + 1./60*tmp3);
     }
 
     template<typename Accessor>
     static typename Accessor::ValueType inXandZ(const Accessor& grid, const Coord& ijk)
     {
-        typename Accessor::ValueType tmp1 =
+        using ValueT = typename Accessor::ValueType;
+        ValueT tmp1 =
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 0, 1)) -
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 0,-1));
-        typename Accessor::ValueType tmp2 =
+        ValueT tmp2 =
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 0, 2)) -
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 0,-2));
-        typename Accessor::ValueType tmp3 =
+        ValueT tmp3 =
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 0, 3)) -
             D1<CD_6TH>::inX(grid, ijk.offsetBy(0, 0,-3));
-        return 0.75*tmp1 - 0.15*tmp2 + 1./60*tmp3;
+        return ValueT(0.75*tmp1 - 0.15*tmp2 + 1./60*tmp3);
     }
 
     template<typename Accessor>
     static typename Accessor::ValueType inYandZ(const Accessor& grid, const Coord& ijk)
     {
-        typename Accessor::ValueType tmp1 =
+        using ValueT = typename Accessor::ValueType;
+        ValueT tmp1 =
             D1<CD_6TH>::inY(grid, ijk.offsetBy(0, 0, 1)) -
             D1<CD_6TH>::inY(grid, ijk.offsetBy(0, 0,-1));
-        typename Accessor::ValueType tmp2 =
+        ValueT tmp2 =
             D1<CD_6TH>::inY(grid, ijk.offsetBy(0, 0, 2)) -
             D1<CD_6TH>::inY(grid, ijk.offsetBy(0, 0,-2));
-        typename Accessor::ValueType tmp3 =
+        ValueT tmp3 =
             D1<CD_6TH>::inY(grid, ijk.offsetBy(0, 0, 3)) -
             D1<CD_6TH>::inY(grid, ijk.offsetBy(0, 0,-3));
-        return 0.75*tmp1 - 0.15*tmp2 + 1./60*tmp3;
+        return ValueT(0.75*tmp1 - 0.15*tmp2 + 1./60*tmp3);
     }
 
 
@@ -2344,6 +2350,6 @@ struct D2<CD_SIXTH>
 
 #endif // OPENVDB_MATH_FINITEDIFFERENCE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
