@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -34,9 +34,9 @@
 #ifndef OPENVDB_MATH_TUPLE_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_TUPLE_HAS_BEEN_INCLUDED
 
-#include <sstream>
-#include <boost/type_traits/is_integral.hpp>
 #include "Math.h"
+#include <cmath>
+#include <sstream>
 
 
 namespace openvdb {
@@ -44,6 +44,10 @@ OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace math {
 
+/// @brief Dummy class for tag dispatch of conversion constructors
+struct Conversion {};
+
+
 /// @class Tuple "Tuple.h"
 /// A base class for homogenous tuple types
 template<int SIZE, typename T>
@@ -54,23 +58,36 @@ public:
 
     static const int size = SIZE;
 
-    /// Default ctor.  Does nothing.  Required because declaring a copy (or
-    /// other) constructor means the default constructor gets left out.
+    /// @brief Default ctor.  Does nothing.
+    /// @details This is required because declaring a copy (or other) constructor
+    /// prevents the compiler from synthesizing a default constructor.
     Tuple() {}
 
     /// Copy constructor.  Used when the class signature matches exactly.
-    inline Tuple(Tuple const &src) {
+    Tuple(Tuple const& src) {
         for (int i = 0; i < SIZE; ++i) {
             mm[i] = src.mm[i];
         }
     }
 
-    /// Conversion constructor.  Tuples with different value types and
-    /// different sizes can be interconverted using this member.  Converting
-    /// from a larger tuple results in truncation; converting from a smaller
-    /// tuple results in the extra data members being zeroed out.  This
-    /// function assumes that the integer 0 is convertible to the tuple's
-    /// value type.
+    /// @brief Assignment operator
+    /// @details This is required because declaring a copy (or other) constructor
+    /// prevents the compiler from synthesizing a default assignment operator.
+    Tuple& operator=(Tuple const& src) {
+        if (&src != this) {
+            for (int i = 0; i < SIZE; ++i) {
+                mm[i] = src.mm[i];
+            }
+        }
+        return *this;
+    }
+
+    /// @brief Conversion constructor.
+    /// @details Tuples with different value types and different sizes can be
+    /// interconverted using this member.  Converting from a larger tuple
+    /// results in truncation; converting from a smaller tuple results in
+    /// the extra data members being zeroed out.  This function assumes that
+    /// the integer 0 is convertible to the tuple's value type.
     template <int src_size, typename src_valtype>
     explicit Tuple(Tuple<src_size, src_valtype> const &src) {
         enum { COPY_END = (SIZE < src_size ? SIZE : src_size) };
@@ -120,8 +137,7 @@ public:
     //@}  Compatibility
 
     /// @return string representation of Classname
-    std::string
-    str() const {
+    std::string str() const {
         std::ostringstream buffer;
 
         buffer << "[";
@@ -129,7 +145,7 @@ public:
         // For each column
         for (unsigned j(0); j < SIZE; j++) {
             if (j) buffer << ", ";
-            buffer << mm[j];
+            buffer << PrintCast(mm[j]);
         }
 
         buffer << "]";
@@ -144,6 +160,38 @@ public:
         is.read(reinterpret_cast<char*>(&mm), sizeof(T)*SIZE);
     }
 
+    /// True if a Nan is present in this tuple
+    bool isNan() const { 
+        for (int i = 0; i < SIZE; ++i) {
+            if (std::isnan(mm[i])) return true;
+        }
+        return false;
+    }
+
+    /// True if an Inf is present in this tuple
+    bool isInfinite() const {
+        for (int i = 0; i < SIZE; ++i) {
+            if (std::isinf(mm[i])) return true;
+        }
+        return false;
+    }
+
+    /// True if no Nan or Inf values are present
+    bool isFinite() const {
+        for (int i = 0; i < SIZE; ++i) {
+            if (!std::isfinite(mm[i])) return false;
+        }
+        return true;
+    }
+
+    /// True if all elements are exactly zero
+    bool isZero() const {
+        for (int i = 0; i < SIZE; ++i) {
+            if (!isZero(mm[i])) return false;
+        }
+        return true;
+    }
+
 protected:
     T mm[SIZE];
 };
@@ -189,6 +237,21 @@ Abs(const Tuple<SIZE, T>& t)
     return result;
 }
 
+/// Return @c true if a Nan is present in the tuple.
+template<int SIZE, typename T>
+inline bool isNan(const Tuple<SIZE, T>& t) { return t.isNan(); }
+
+/// Return @c true if an Inf is present in the tuple.
+template<int SIZE, typename T>
+inline bool isInfinite(const Tuple<SIZE, T>& t) { return t.isInfinite(); }
+
+/// Return @c true if no Nan or Inf values are present.
+template<int SIZE, typename T>
+inline bool isFinite(const Tuple<SIZE, T>& t) { return t.isFinite(); }
+
+/// Return @c true if all elements are exactly equal to zero.
+template<int SIZE, typename T>
+inline bool isZero(const Tuple<SIZE, T>& t) { return t.isZero(); }
 
 ////////////////////////////////////////
 
@@ -207,6 +270,6 @@ std::ostream& operator<<(std::ostream& ostr, const Tuple<SIZE, T>& classname)
 
 #endif // OPENVDB_MATH_TUPLE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
