@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -29,12 +29,12 @@
 ///////////////////////////////////////////////////////////////////////////
 
 #include "openvdb.h"
-#include <openvdb/tools/PointIndexGrid.h>
+//#ifdef OPENVDB_ENABLE_POINTS
+#include "points/PointDataGrid.h"
+//#endif
+#include "tools/PointIndexGrid.h"
+#include "util/logging.h"
 #include <tbb/mutex.h>
-#ifdef OPENVDB_USE_LOG4CPLUS
-#include <log4cplus/configurator.h>
-#include <log4cplus/logger.h>
-#endif
 #ifdef OPENVDB_USE_BLOSC
 #include <blosc.h>
 #endif
@@ -58,17 +58,7 @@ initialize()
     Lock lock(sInitMutex);
     if (sIsInitialized) return;
 
-#ifdef OPENVDB_USE_LOG4CPLUS
-    {
-        // Configure log4cplus if it was not already configured (at least,
-        // if there are no appenders associated with the root logger).
-        log4cplus::Logger rootLogger = log4cplus::Logger::getRoot();
-        if (rootLogger.getAllAppenders().empty()) {
-            log4cplus::BasicConfigurator::doConfigure();
-            rootLogger.setLogLevel(log4cplus::WARN_LOG_LEVEL); // was DEBUG_LOG_LEVEL
-        }
-    }
-#endif
+    logging::initialize();
 
     // Register metadata.
     Metadata::clearRegistry();
@@ -116,6 +106,11 @@ initialize()
     Metadata::registerType(typeNameAsString<PointIndex64>(), Int64Metadata::createMetadata);
     tools::PointIndexGrid::registerGrid();
 
+    // Register types associated with point data grids.
+//#ifdef OPENVDB_ENABLE_POINTS
+    points::internal::initialize();
+//#endif
+
 #ifdef OPENVDB_USE_BLOSC
     blosc_init();
     if (blosc_set_compressor("lz4") < 0) {
@@ -159,6 +154,10 @@ __pragma(warning(default:1711))
     GridBase::clearRegistry();
     math::MapRegistry::clear();
 
+//#ifdef OPENVDB_ENABLE_POINTS
+    points::internal::uninitialize();
+//#endif
+
 #ifdef OPENVDB_USE_BLOSC
     // We don't want to destroy Blosc, because it might have been
     // initialized by some other library.
@@ -169,6 +168,6 @@ __pragma(warning(default:1711))
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
