@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -30,7 +30,7 @@
 //
 /// @author Ken Museth
 ///
-/// @file Filter.h
+/// @file tools/Filter.h
 ///
 /// @brief Filtering of VDB volumes. Note that only the values in the
 /// grid are changed, not its topology! All operations can optionally
@@ -40,9 +40,6 @@
 #define OPENVDB_TOOLS_FILTER_HAS_BEEN_INCLUDED
 
 #include <tbb/parallel_for.h>
-#include <boost/bind.hpp>
-#include <boost/function.hpp>
-#include <boost/type_traits/is_floating_point.hpp>
 #include <openvdb/Types.h>
 #include <openvdb/math/Math.h>
 #include <openvdb/math/Stencils.h>
@@ -51,6 +48,10 @@
 #include <openvdb/util/NullInterrupter.h>
 #include <openvdb/Grid.h>
 #include "Interpolation.h"
+#include <algorithm> // for std::max()
+#include <functional>
+#include <type_traits>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -66,25 +67,26 @@ template<typename GridT,
 class Filter
 {
 public:
-    typedef GridT                                GridType;
-    typedef MaskT                                MaskType;
-    typedef typename GridType::TreeType          TreeType;
-    typedef typename TreeType::LeafNodeType      LeafType;
-    typedef typename GridType::ValueType         ValueType;
-    typedef typename MaskType::ValueType         AlphaType;
-    typedef typename tree::LeafManager<TreeType> LeafManagerType;
-    typedef typename LeafManagerType::LeafRange  RangeType;
-    typedef typename LeafManagerType::BufferType BufferType;
-    BOOST_STATIC_ASSERT(boost::is_floating_point<AlphaType>::value);
+    using GridType = GridT;
+    using MaskType = MaskT;
+    using TreeType = typename GridType::TreeType;
+    using LeafType = typename TreeType::LeafNodeType;
+    using ValueType = typename GridType::ValueType;
+    using AlphaType = typename MaskType::ValueType;
+    using LeafManagerType = typename tree::LeafManager<TreeType>;
+    using RangeType = typename LeafManagerType::LeafRange;
+    using BufferType = typename LeafManagerType::BufferType;
+    static_assert(std::is_floating_point<AlphaType>::value,
+        "openvdb::tools::Filter requires a mask grid with floating-point values");
 
     /// Constructor
     /// @param grid Grid to be filtered.
     /// @param interrupt Optional interrupter.
-    Filter(GridT& grid, InterruptT* interrupt = NULL)
+    Filter(GridT& grid, InterruptT* interrupt = nullptr)
         : mGrid(&grid)
         , mTask(0)
         , mInterrupter(interrupt)
-        , mMask(NULL)
+        , mMask(nullptr)
         , mGrainSize(1)
         , mMinMask(0)
         , mMaxMask(1)
@@ -143,7 +145,7 @@ public:
     /// @param width The width of the mean-value filter is 2*width+1 voxels.
     /// @param iterations Number of times the mean-value filter is applied.
     /// @param mask Optional alpha mask.
-    void mean(int width = 1, int iterations = 1, const MaskType* mask = NULL);
+    void mean(int width = 1, int iterations = 1, const MaskType* mask = nullptr);
 
     /// @brief One iteration of a fast separable Gaussian filter.
     ///
@@ -152,7 +154,7 @@ public:
     /// @param width The width of the mean-value filter is 2*width+1 voxels.
     /// @param iterations Number of times the mean-value filter is applied.
     /// @param mask Optional alpha mask.
-    void gaussian(int width = 1, int iterations = 1, const MaskType* mask = NULL);
+    void gaussian(int width = 1, int iterations = 1, const MaskType* mask = nullptr);
 
     /// @brief One iteration of a median-value filter
     ///
@@ -160,12 +162,12 @@ public:
     /// @param width The width of the mean-value filter is 2*width+1 voxels.
     /// @param iterations Number of times the mean-value filter is applied.
     /// @param mask Optional alpha mask.
-    void median(int width = 1, int iterations = 1, const MaskType* mask = NULL);
+    void median(int width = 1, int iterations = 1, const MaskType* mask = nullptr);
 
     /// Offsets (i.e. adds) a constant value to all active voxels.
     /// @param offset Offset in the same units as the grid.
     /// @param mask Optional alpha mask.
-    void offset(ValueType offset, const MaskType* mask = NULL);
+    void offset(ValueType offset, const MaskType* mask = nullptr);
 
     /// @brief Used internally by tbb::parallel_for()
     /// @param range Range of LeafNodes over which to multi-thread.
@@ -178,12 +180,12 @@ public:
     }
 
 private:
-    typedef typename TreeType::LeafNodeType                  LeafT;
-    typedef typename LeafT::ValueOnIter                      VoxelIterT;
-    typedef typename LeafT::ValueOnCIter                     VoxelCIterT;
-    typedef typename tree::LeafManager<TreeType>::BufferType BufferT;
-    typedef typename RangeType::Iterator                     LeafIterT;
-    typedef tools::AlphaMask<GridT, MaskT>                   AlphaMaskT;
+    using LeafT = typename TreeType::LeafNodeType;
+    using VoxelIterT = typename LeafT::ValueOnIter;
+    using VoxelCIterT = typename LeafT::ValueOnCIter;
+    using BufferT = typename tree::LeafManager<TreeType>::BufferType;
+    using LeafIterT = typename RangeType::Iterator;
+    using AlphaMaskT = tools::AlphaMask<GridT, MaskT>;
 
     void cook(LeafManagerType& leafs);
 
@@ -208,7 +210,7 @@ private:
     bool wasInterrupted();
 
     GridType*        mGrid;
-    typename boost::function<void (Filter*, const RangeType&)> mTask;
+    typename std::function<void (Filter*, const RangeType&)> mTask;
     InterruptT*      mInterrupter;
     const MaskType*  mMask;
     int              mGrainSize;
@@ -256,13 +258,13 @@ Filter<GridT, MaskT, InterruptT>::mean(int width, int iterations, const MaskType
     LeafManagerType leafs(mGrid->tree(), 1, mGrainSize==0);
 
     for (int i=0; i<iterations && !this->wasInterrupted(); ++i) {
-        mTask = boost::bind(&Filter::doBoxX, _1, _2, w);
+        mTask = std::bind(&Filter::doBoxX, std::placeholders::_1, std::placeholders::_2, w);
         this->cook(leafs);
 
-        mTask = boost::bind(&Filter::doBoxY, _1, _2, w);
+        mTask = std::bind(&Filter::doBoxY, std::placeholders::_1, std::placeholders::_2, w);
         this->cook(leafs);
 
-        mTask = boost::bind(&Filter::doBoxZ, _1, _2, w);
+        mTask = std::bind(&Filter::doBoxZ, std::placeholders::_1, std::placeholders::_2, w);
         this->cook(leafs);
     }
 
@@ -284,13 +286,13 @@ Filter<GridT, MaskT, InterruptT>::gaussian(int width, int iterations, const Mask
 
     for (int i=0; i<iterations; ++i) {
         for (int n=0; n<4 && !this->wasInterrupted(); ++n) {
-            mTask = boost::bind(&Filter::doBoxX, _1, _2, w);
+            mTask = std::bind(&Filter::doBoxX, std::placeholders::_1, std::placeholders::_2, w);
             this->cook(leafs);
 
-            mTask = boost::bind(&Filter::doBoxY, _1, _2, w);
+            mTask = std::bind(&Filter::doBoxY, std::placeholders::_1, std::placeholders::_2, w);
             this->cook(leafs);
 
-            mTask = boost::bind(&Filter::doBoxZ, _1, _2, w);
+            mTask = std::bind(&Filter::doBoxZ, std::placeholders::_1, std::placeholders::_2, w);
             this->cook(leafs);
         }
     }
@@ -309,7 +311,8 @@ Filter<GridT, MaskT, InterruptT>::median(int width, int iterations, const MaskTy
 
     LeafManagerType leafs(mGrid->tree(), 1, mGrainSize==0);
 
-    mTask = boost::bind(&Filter::doMedian, _1, _2, std::max(1, width));
+    mTask = std::bind(&Filter::doMedian,
+        std::placeholders::_1, std::placeholders::_2, std::max(1, width));
     for (int i=0; i<iterations && !this->wasInterrupted(); ++i) this->cook(leafs);
 
     if (mInterrupter) mInterrupter->end();
@@ -326,7 +329,7 @@ Filter<GridT, MaskT, InterruptT>::offset(ValueType value, const MaskType* mask)
 
     LeafManagerType leafs(mGrid->tree(), 0, mGrainSize==0);
 
-    mTask = boost::bind(&Filter::doOffset, _1, _2, value);
+    mTask = std::bind(&Filter::doOffset, std::placeholders::_1, std::placeholders::_2, value);
     this->cook(leafs);
 
     if (mInterrupter) mInterrupter->end();
@@ -454,6 +457,6 @@ Filter<GridT, MaskT, InterruptT>::wasInterrupted()
 
 #endif // OPENVDB_TOOLS_FILTER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
