@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -27,7 +27,7 @@
 // LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
 //
 ///////////////////////////////////////////////////////////////////////////
-//
+
 /// @file Interpolation.h
 ///
 /// Sampler classes such as PointSampler and BoxSampler that are intended for use
@@ -67,20 +67,53 @@
 #ifndef OPENVDB_TOOLS_INTERPOLATION_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_INTERPOLATION_HAS_BEEN_INCLUDED
 
-#include <cmath>
-#include <boost/shared_ptr.hpp>
 #include <openvdb/version.h> // for OPENVDB_VERSION_NAME
 #include <openvdb/Platform.h> // for round()
 #include <openvdb/math/Math.h>// for SmoothUnitStep
 #include <openvdb/math/Transform.h> // for Transform
 #include <openvdb/Grid.h>
 #include <openvdb/tree/ValueAccessor.h>
+#include <cmath>
+#include <type_traits>
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace tools {
 
+/// @brief Provises a unified interface for sampling, i.e. interpolation.
+/// @details Order = 0: closest point
+///          Order = 1: tri-linear
+///          Order = 2: tri-quadratic
+///          Staggered: Set to true for MAC grids
+template <size_t Order, bool Staggered = false>
+struct Sampler
+{
+    static_assert(Order < 3, "Samplers of order higher than 2 are not supported");
+    static const char* name();
+    static int radius();
+    static bool mipmap();
+    static bool consistent();
+    static bool staggered();
+    static size_t order();
+
+    /// @brief Sample @a inTree at the floating-point index coordinate @a inCoord
+    /// and store the result in @a result.
+    ///
+    /// @return @c true if the sampled value is active.
+    template<class TreeT>
+    static bool sample(const TreeT& inTree, const Vec3R& inCoord,
+                       typename TreeT::ValueType& result);
+
+    /// @brief Sample @a inTree at the floating-point index coordinate @a inCoord.
+    ///
+    /// @return the reconstructed value
+    template<class TreeT>
+    static typename TreeT::ValueType sample(const TreeT& inTree, const Vec3R& inCoord);
+};
+
+//////////////////////////////////////// Non-Staggered Samplers
+
 // The following samplers operate in voxel space.
 // When the samplers are applied to grids holding vector or other non-scalar data,
 // the data is assumed to be collocated.  For example, using the BoxSampler on a grid
@@ -93,13 +126,20 @@ struct PointSampler
     static int radius() { return 0; }
     static bool mipmap() { return false; }
     static bool consistent() { return true; }
+    static bool staggered() { return false; }
+    static size_t order() { return 0; }
 
     /// @brief Sample @a inTree at the nearest neighbor to @a inCoord
     /// and store the result in @a result.
     /// @return @c true if the sampled value is active.
     template<class TreeT>
     static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
+                       typename TreeT::ValueType& result);
+
+    /// @brief Sample @a inTree at the nearest neighbor to @a inCoord
+    /// @return the reconstructed value
+    template<class TreeT>
+    static typename TreeT::ValueType sample(const TreeT& inTree, const Vec3R& inCoord);
 };
 
 
@@ -109,22 +149,40 @@ struct BoxSampler
     static int radius() { return 1; }
     static bool mipmap() { return true; }
     static bool consistent() { return true; }
+    static bool staggered() { return false; }
+    static size_t order() { return 1; }
 
     /// @brief Trilinearly reconstruct @a inTree at @a inCoord
     /// and store the result in @a result.
     /// @return @c true if any one of the sampled values is active.
     template<class TreeT>
     static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
+                       typename TreeT::ValueType& result);
 
     /// @brief Trilinearly reconstruct @a inTree at @a inCoord.
     /// @return the reconstructed value
     template<class TreeT>
     static typename TreeT::ValueType sample(const TreeT& inTree, const Vec3R& inCoord);
 
-private:
+    /// @brief Import all eight values from @a inTree to support
+    /// tri-linear interpolation.
+    template<class ValueT, class TreeT, size_t N>
+    static inline void getValues(ValueT (&data)[N][N][N], const TreeT& inTree, Coord ijk);
+
+    /// @brief Import all eight values from @a inTree to support
+    /// tri-linear interpolation.
+    /// @return @c true if any of the eight values are active
+    template<class ValueT, class TreeT, size_t N>
+    static inline bool probeValues(ValueT (&data)[N][N][N], const TreeT& inTree, Coord ijk);
+
+    /// @brief Find the minimum and maximum values of the eight cell
+    /// values in @ data.
     template<class ValueT, size_t N>
-    static inline ValueT trilinearInterpolation(ValueT (& data)[N][N][N], const Vec3R& uvw);
+    static inline void extrema(ValueT (&data)[N][N][N], ValueT& vMin, ValueT& vMax);
+
+    /// @return the tri-linear interpolation with the unit cell coordinates @a uvw
+    template<class ValueT, size_t N>
+    static inline ValueT trilinearInterpolation(ValueT (&data)[N][N][N], const Vec3R& uvw);
 };
 
 
@@ -134,17 +192,27 @@ struct QuadraticSampler
     static int radius() { return 1; }
     static bool mipmap() { return true; }
     static bool consistent() { return false; }
+    static bool staggered() { return false; }
+    static size_t order() { return 2; }
 
     /// @brief Triquadratically reconstruct @a inTree at @a inCoord
     /// and store the result in @a result.
     /// @return @c true if any one of the sampled values is active.
     template<class TreeT>
     static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
+                       typename TreeT::ValueType& result);
+
+    /// @brief Triquadratically reconstruct @a inTree at to @a inCoord.
+    /// @return the reconstructed value
+    template<class TreeT>
+    static typename TreeT::ValueType sample(const TreeT& inTree, const Vec3R& inCoord);
+
+    template<class ValueT, size_t N>
+    static inline ValueT triquadraticInterpolation(ValueT (&data)[N][N][N], const Vec3R& uvw);
 };
 
 
-////////////////////////////////////////
+//////////////////////////////////////// Staggered Samplers
 
 
 // The following samplers operate in voxel space and are designed for Vec3
@@ -158,13 +226,20 @@ struct StaggeredPointSampler
     static int radius() { return 0; }
     static bool mipmap() { return false; }
     static bool consistent() { return false; }
+    static bool staggered() { return true; }
+    static size_t order() { return 0; }
 
     /// @brief Sample @a inTree at the nearest neighbor to @a inCoord
     /// and store the result in @a result.
     /// @return true if the sampled value is active.
     template<class TreeT>
     static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
+                       typename TreeT::ValueType& result);
+
+    /// @brief Sample @a inTree at the nearest neighbor to @a inCoord
+    /// @return the reconstructed value
+    template<class TreeT>
+    static typename TreeT::ValueType sample(const TreeT& inTree, const Vec3R& inCoord);
 };
 
 
@@ -174,13 +249,20 @@ struct StaggeredBoxSampler
     static int radius() { return 1; }
     static bool mipmap() { return true; }
     static bool consistent() { return false; }
+    static bool staggered() { return true; }
+    static size_t order() { return 1; }
 
     /// @brief Trilinearly reconstruct @a inTree at @a inCoord
     /// and store the result in @a result.
     /// @return true if any one of the sampled value is active.
     template<class TreeT>
     static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
+                       typename TreeT::ValueType& result);
+
+    /// @brief Trilinearly reconstruct @a inTree at @a inCoord.
+    /// @return the reconstructed value
+    template<class TreeT>
+    static typename TreeT::ValueType sample(const TreeT& inTree, const Vec3R& inCoord);
 };
 
 
@@ -190,17 +272,24 @@ struct StaggeredQuadraticSampler
     static int radius() { return 1; }
     static bool mipmap() { return true; }
     static bool consistent() { return false; }
+    static bool staggered() { return true; }
+    static size_t order() { return 2; }
 
     /// @brief Triquadratically reconstruct @a inTree at @a inCoord
     /// and store the result in @a result.
     /// @return true if any one of the sampled values is active.
     template<class TreeT>
     static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
+                       typename TreeT::ValueType& result);
+
+    /// @brief Triquadratically reconstruct @a inTree at to @a inCoord.
+    /// @return the reconstructed value
+    template<class TreeT>
+    static typename TreeT::ValueType sample(const TreeT& inTree, const Vec3R& inCoord);
 };
 
 
-////////////////////////////////////////
+//////////////////////////////////////// GridSampler
 
 
 /// @brief Class that provides the interface for continuous sampling
@@ -221,11 +310,11 @@ template<typename GridOrTreeType, typename SamplerType>
 class GridSampler
 {
 public:
-    typedef boost::shared_ptr<GridSampler>                      Ptr;
-    typedef typename GridOrTreeType::ValueType                  ValueType;
-    typedef typename TreeAdapter<GridOrTreeType>::GridType      GridType;
-    typedef typename TreeAdapter<GridOrTreeType>::TreeType      TreeType;
-    typedef typename TreeAdapter<GridOrTreeType>::AccessorType  AccessorType;
+    using Ptr = SharedPtr<GridSampler>;
+    using ValueType = typename GridOrTreeType::ValueType;
+    using GridType = typename TreeAdapter<GridOrTreeType>::GridType;
+    using TreeType = typename TreeAdapter<GridOrTreeType>::TreeType;
+    using AccessorType = typename TreeAdapter<GridOrTreeType>::AccessorType;
 
      /// @param grid  a grid to be sampled
     explicit GridSampler(const GridType& grid)
@@ -289,25 +378,25 @@ private:
 
 /// @brief Specialization of GridSampler for construction from a ValueAccessor type
 ///
-/// @note This version should normally be favoured over the one above
+/// @note This version should normally be favored over the one above
 /// that takes a Grid or Tree. The reason is this version uses a
 /// ValueAccessor that performs fast (cached) access where the
-/// tree-based flavour performs slower (uncached) access.
+/// tree-based flavor performs slower (uncached) access.
 ///
 /// @warning Since this version stores a pointer to an (externally
 /// allocated) value accessor it is not threadsafe. Hence each thread
-/// should have it own instance of a GridSampler constructed from a
+/// should have its own instance of a GridSampler constructed from a
 /// local ValueAccessor. Alternatively the Grid/Tree-based GridSampler
 /// is threadsafe, but also slower.
 template<typename TreeT, typename SamplerType>
 class GridSampler<tree::ValueAccessor<TreeT>, SamplerType>
 {
 public:
-    typedef boost::shared_ptr<GridSampler>      Ptr;
-    typedef typename TreeT::ValueType           ValueType;
-    typedef TreeT                               TreeType;
-    typedef Grid<TreeType>                      GridType;
-    typedef typename tree::ValueAccessor<TreeT> AccessorType;
+    using Ptr = SharedPtr<GridSampler>;
+    using ValueType = typename TreeT::ValueType;
+    using TreeType = TreeT;
+    using GridType = Grid<TreeType>;
+    using AccessorType = typename tree::ValueAccessor<TreeT>;
 
     /// @param acc  a ValueAccessor to be sampled
     /// @param transform is used when sampling world space locations.
@@ -366,7 +455,7 @@ private:
 };//Specialization of GridSampler
 
 
-////////////////////////////////////////
+//////////////////////////////////////// DualGridSampler
 
 
 /// @brief This is a simple convenience class that allows for sampling
@@ -383,10 +472,10 @@ template<typename GridOrTreeT,
 class DualGridSampler
 {
 public:
-    typedef typename GridOrTreeT::ValueType               ValueType;
-    typedef typename TreeAdapter<GridOrTreeT>::GridType   GridType;
-    typedef typename TreeAdapter<GridOrTreeT>::TreeType   TreeType;
-    typedef typename TreeAdapter<GridType>::AccessorType  AccessorType;
+    using ValueType = typename GridOrTreeT::ValueType;
+    using GridType = typename TreeAdapter<GridOrTreeT>::GridType;
+    using TreeType = typename TreeAdapter<GridOrTreeT>::TreeType;
+    using AccessorType = typename TreeAdapter<GridType>::AccessorType;
 
     /// @brief Grid and transform constructor.
     /// @param sourceGrid Source grid.
@@ -435,10 +524,10 @@ template<typename TreeT,
 class DualGridSampler<tree::ValueAccessor<TreeT>, SamplerT>
 {
     public:
-    typedef typename TreeT::ValueType ValueType;
-    typedef TreeT                     TreeType;
-    typedef Grid<TreeType>            GridType;
-    typedef typename tree::ValueAccessor<TreeT> AccessorType;
+    using ValueType = typename TreeT::ValueType;
+    using TreeType = TreeT;
+    using GridType = Grid<TreeType>;
+    using AccessorType = typename tree::ValueAccessor<TreeT>;
 
     /// @brief ValueAccessor and transform constructor.
     /// @param sourceAccessor ValueAccessor into the source grid.
@@ -470,7 +559,7 @@ private:
     const bool             mAligned;
 };//Specialization of DualGridSampler
 
-////////////////////////////////////////
+//////////////////////////////////////// AlphaMask
 
 
 // Class to derive the normalized alpha mask
@@ -481,11 +570,12 @@ template <typename GridT,
 class AlphaMask
 {
 public:
-    BOOST_STATIC_ASSERT(boost::is_floating_point<FloatT>::value);
-    typedef GridT    GridType;
-    typedef MaskT    MaskType;
-    typedef SamplerT SamlerType;
-    typedef FloatT   FloatType;
+    static_assert(std::is_floating_point<FloatT>::value,
+        "AlphaMask requires a floating-point value type");
+    using GridType = GridT;
+    using MaskType = MaskT;
+    using SamlerType = SamplerT;
+    using FloatType = FloatT;
 
     AlphaMask(const GridT& grid, const MaskT& mask, FloatT min, FloatT max, bool invert)
         : mAcc(mask.tree())
@@ -506,7 +596,7 @@ public:
     }
 
 protected:
-    typedef typename MaskType::ConstAccessor AccT;
+    using AccT = typename MaskType::ConstAccessor;
     AccT mAcc;
     tools::DualGridSampler<AccT, SamplerT> mSampler;
     const FloatT mMin, mInvNorm;
@@ -540,25 +630,113 @@ roundVec3(const Vec3R& v)
 } // namespace local_util
 
 
-////////////////////////////////////////
+//////////////////////////////////////// PointSampler
 
 
 template<class TreeT>
 inline bool
 PointSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
+                     typename TreeT::ValueType& result)
 {
-    Vec3i inIdx = local_util::roundVec3(inCoord);
-    return inTree.probeValue(Coord(inIdx), result);
+    return inTree.probeValue(Coord(local_util::roundVec3(inCoord)), result);
+}
+
+template<class TreeT>
+inline typename TreeT::ValueType
+PointSampler::sample(const TreeT& inTree, const Vec3R& inCoord)
+{
+    return inTree.getValue(Coord(local_util::roundVec3(inCoord)));
 }
 
 
-////////////////////////////////////////
+//////////////////////////////////////// BoxSampler
+
+template<class ValueT, class TreeT, size_t N>
+inline void
+BoxSampler::getValues(ValueT (&data)[N][N][N], const TreeT& inTree, Coord ijk)
+{
+    data[0][0][0] = inTree.getValue(ijk); // i, j, k
+
+    ijk[2] += 1;
+    data[0][0][1] = inTree.getValue(ijk); // i, j, k + 1
+
+    ijk[1] += 1;
+    data[0][1][1] = inTree.getValue(ijk); // i, j+1, k + 1
+
+    ijk[2] -= 1;
+    data[0][1][0] = inTree.getValue(ijk); // i, j+1, k
+
+    ijk[0] += 1;
+    ijk[1] -= 1;
+    data[1][0][0] = inTree.getValue(ijk); // i+1, j, k
+
+    ijk[2] += 1;
+    data[1][0][1] = inTree.getValue(ijk); // i+1, j, k + 1
+
+    ijk[1] += 1;
+    data[1][1][1] = inTree.getValue(ijk); // i+1, j+1, k + 1
+
+    ijk[2] -= 1;
+    data[1][1][0] = inTree.getValue(ijk); // i+1, j+1, k
+}
+
+template<class ValueT, class TreeT, size_t N>
+inline bool
+BoxSampler::probeValues(ValueT (&data)[N][N][N], const TreeT& inTree, Coord ijk)
+{
+    bool hasActiveValues = false;
+    hasActiveValues |= inTree.probeValue(ijk, data[0][0][0]); // i, j, k
+
+    ijk[2] += 1;
+    hasActiveValues |= inTree.probeValue(ijk, data[0][0][1]); // i, j, k + 1
+
+    ijk[1] += 1;
+    hasActiveValues |= inTree.probeValue(ijk, data[0][1][1]); // i, j+1, k + 1
+
+    ijk[2] -= 1;
+    hasActiveValues |= inTree.probeValue(ijk, data[0][1][0]); // i, j+1, k
+
+    ijk[0] += 1;
+    ijk[1] -= 1;
+    hasActiveValues |= inTree.probeValue(ijk, data[1][0][0]); // i+1, j, k
+
+    ijk[2] += 1;
+    hasActiveValues |= inTree.probeValue(ijk, data[1][0][1]); // i+1, j, k + 1
+
+    ijk[1] += 1;
+    hasActiveValues |= inTree.probeValue(ijk, data[1][1][1]); // i+1, j+1, k + 1
+
+    ijk[2] -= 1;
+    hasActiveValues |= inTree.probeValue(ijk, data[1][1][0]); // i+1, j+1, k
+
+    return hasActiveValues;
+}
+
+template<class ValueT, size_t N>
+inline void
+BoxSampler::extrema(ValueT (&data)[N][N][N], ValueT& vMin, ValueT &vMax)
+{
+    vMin = vMax = data[0][0][0];
+    vMin = math::Min(vMin, data[0][0][1]);
+    vMax = math::Max(vMax, data[0][0][1]);
+    vMin = math::Min(vMin, data[0][1][0]);
+    vMax = math::Max(vMax, data[0][1][0]);
+    vMin = math::Min(vMin, data[0][1][1]);
+    vMax = math::Max(vMax, data[0][1][1]);
+    vMin = math::Min(vMin, data[1][0][0]);
+    vMax = math::Max(vMax, data[1][0][0]);
+    vMin = math::Min(vMin, data[1][0][1]);
+    vMax = math::Max(vMax, data[1][0][1]);
+    vMin = math::Min(vMin, data[1][1][0]);
+    vMax = math::Max(vMax, data[1][1][0]);
+    vMin = math::Min(vMin, data[1][1][1]);
+    vMax = math::Max(vMax, data[1][1][1]);
+}
 
 
 template<class ValueT, size_t N>
 inline ValueT
-BoxSampler::trilinearInterpolation(ValueT (& data)[N][N][N], const Vec3R& uvw)
+BoxSampler::trilinearInterpolation(ValueT (&data)[N][N][N], const Vec3R& uvw)
 {
     // Trilinear interpolation:
     // The eight surrounding latice values are used to construct the result. \n
@@ -583,37 +761,21 @@ BoxSampler::trilinearInterpolation(ValueT (& data)[N][N][N], const Vec3R& uvw)
 template<class TreeT>
 inline bool
 BoxSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
+                   typename TreeT::ValueType& result)
 {
-    typedef typename TreeT::ValueType ValueT;
+    using ValueT = typename TreeT::ValueType;
 
-    Vec3i inIdx = local_util::floorVec3(inCoord);
-    Vec3R uvw = inCoord - inIdx;
+    const Vec3i inIdx = local_util::floorVec3(inCoord);
+    const Vec3R uvw = inCoord - inIdx;
 
     // Retrieve the values of the eight voxels surrounding the
     // fractional source coordinates.
     ValueT data[2][2][2];
 
-    bool hasActiveValues = false;
-    Coord ijk(inIdx);
-    hasActiveValues |= inTree.probeValue(ijk, data[0][0][0]);  // i, j, k
-    ijk[2] += 1;
-    hasActiveValues |= inTree.probeValue(ijk, data[0][0][1]);  // i, j, k + 1
-    ijk[1] += 1;
-    hasActiveValues |= inTree.probeValue(ijk, data[0][1][1]); // i, j+1, k + 1
-    ijk[2] = inIdx[2];
-    hasActiveValues |= inTree.probeValue(ijk, data[0][1][0]);  // i, j+1, k
-    ijk[0] += 1;
-    ijk[1] = inIdx[1];
-    hasActiveValues |= inTree.probeValue(ijk, data[1][0][0]); // i+1, j, k
-    ijk[2] += 1;
-    hasActiveValues |= inTree.probeValue(ijk, data[1][0][1]); // i+1, j, k + 1
-    ijk[1] += 1;
-    hasActiveValues |= inTree.probeValue(ijk, data[1][1][1]); // i+1, j+1, k + 1
-    ijk[2] = inIdx[2];
-    hasActiveValues |= inTree.probeValue(ijk, data[1][1][0]); // i+1, j+1, k
+    const bool hasActiveValues = BoxSampler::probeValues(data, inTree, Coord(inIdx));
+
+    result = BoxSampler::trilinearInterpolation(data, uvw);
 
-    result = trilinearInterpolation(data, uvw);
     return hasActiveValues;
 }
 
@@ -622,66 +784,27 @@ template<class TreeT>
 inline typename TreeT::ValueType
 BoxSampler::sample(const TreeT& inTree, const Vec3R& inCoord)
 {
-    typedef typename TreeT::ValueType ValueT;
+    using ValueT = typename TreeT::ValueType;
 
-    Vec3i inIdx = local_util::floorVec3(inCoord);
-    Vec3R uvw = inCoord - inIdx;
+    const Vec3i inIdx = local_util::floorVec3(inCoord);
+    const Vec3R uvw = inCoord - inIdx;
 
     // Retrieve the values of the eight voxels surrounding the
     // fractional source coordinates.
     ValueT data[2][2][2];
 
-    Coord ijk(inIdx);
-    data[0][0][0] = inTree.getValue(ijk);  // i, j, k
-    ijk[2] += 1;
-    data[0][0][1] = inTree.getValue(ijk);  // i, j, k + 1
-    ijk[1] += 1;
-    data[0][1][1] = inTree.getValue(ijk); // i, j+1, k + 1
-    ijk[2] = inIdx[2];
-    data[0][1][0] = inTree.getValue(ijk);  // i, j+1, k
-    ijk[0] += 1;
-    ijk[1] = inIdx[1];
-    data[1][0][0] = inTree.getValue(ijk); // i+1, j, k
-    ijk[2] += 1;
-    data[1][0][1] = inTree.getValue(ijk); // i+1, j, k + 1
-    ijk[1] += 1;
-    data[1][1][1] = inTree.getValue(ijk); // i+1, j+1, k + 1
-    ijk[2] = inIdx[2];
-    data[1][1][0] = inTree.getValue(ijk); // i+1, j+1, k
+    BoxSampler::getValues(data, inTree, Coord(inIdx));
 
-    return trilinearInterpolation(data, uvw);
+    return BoxSampler::trilinearInterpolation(data, uvw);
 }
 
 
-////////////////////////////////////////
+//////////////////////////////////////// QuadraticSampler
 
-
-template<class TreeT>
-inline bool
-QuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
+template<class ValueT, size_t N>
+inline ValueT
+QuadraticSampler::triquadraticInterpolation(ValueT (&data)[N][N][N], const Vec3R& uvw)
 {
-    typedef typename TreeT::ValueType ValueT;
-
-    Vec3i
-        inIdx = local_util::floorVec3(inCoord),
-        inLoIdx = inIdx - Vec3i(1, 1, 1);
-    Vec3R frac = inCoord - inIdx;
-
-    // Retrieve the values of the 27 voxels surrounding the
-    // fractional source coordinates.
-    bool active = false;
-    ValueT v[3][3][3];
-    for (int dx = 0, ix = inLoIdx.x(); dx < 3; ++dx, ++ix) {
-        for (int dy = 0, iy = inLoIdx.y(); dy < 3; ++dy, ++iy) {
-            for (int dz = 0, iz = inLoIdx.z(); dz < 3; ++dz, ++iz) {
-                if (inTree.probeValue(Coord(ix, iy, iz), v[dx][dy][dz])) {
-                    active = true;
-                }
-            }
-        }
-    }
-
     /// @todo For vector types, interpolate over each component independently.
     ValueT vx[3];
     for (int dx = 0; dx < 3; ++dx) {
@@ -697,13 +820,13 @@ QuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
             // |   1^2    1   1 || c |   | v2 |
             //
             // for a, b and c.
-            const ValueT* vz = &v[dx][dy][0];
+            const ValueT* vz = &data[dx][dy][0];
             const ValueT
                 az = static_cast<ValueT>(0.5 * (vz[0] + vz[2]) - vz[1]),
                 bz = static_cast<ValueT>(0.5 * (vz[2] - vz[0])),
                 cz = static_cast<ValueT>(vz[1]);
-            vy[dy] = static_cast<ValueT>(frac.z() * (frac.z() * az + bz) + cz);
-        }
+            vy[dy] = static_cast<ValueT>(uvw.z() * (uvw.z() * az + bz) + cz);
+        }//loop over y
         // Fit a parabola to three interpolated samples in y, then
         // evaluate the parabola at y', where y' is the fractional
         // part of inCoord.y.
@@ -711,29 +834,77 @@ QuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
             ay = static_cast<ValueT>(0.5 * (vy[0] + vy[2]) - vy[1]),
             by = static_cast<ValueT>(0.5 * (vy[2] - vy[0])),
             cy = static_cast<ValueT>(vy[1]);
-        vx[dx] = static_cast<ValueT>(frac.y() * (frac.y() * ay + by) + cy);
-    }
+        vx[dx] = static_cast<ValueT>(uvw.y() * (uvw.y() * ay + by) + cy);
+    }//loop over x
     // Fit a parabola to three interpolated samples in x, then
     // evaluate the parabola at the fractional part of inCoord.x.
     const ValueT
         ax = static_cast<ValueT>(0.5 * (vx[0] + vx[2]) - vx[1]),
         bx = static_cast<ValueT>(0.5 * (vx[2] - vx[0])),
         cx = static_cast<ValueT>(vx[1]);
-    result = static_cast<ValueT>(frac.x() * (frac.x() * ax + bx) + cx);
+    return static_cast<ValueT>(uvw.x() * (uvw.x() * ax + bx) + cx);
+}
+
+template<class TreeT>
+inline bool
+QuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
+    typename TreeT::ValueType& result)
+{
+    using ValueT = typename TreeT::ValueType;
+
+    const Vec3i inIdx = local_util::floorVec3(inCoord), inLoIdx = inIdx - Vec3i(1, 1, 1);
+    const Vec3R uvw = inCoord - inIdx;
+
+    // Retrieve the values of the 27 voxels surrounding the
+    // fractional source coordinates.
+    bool active = false;
+    ValueT data[3][3][3];
+    for (int dx = 0, ix = inLoIdx.x(); dx < 3; ++dx, ++ix) {
+        for (int dy = 0, iy = inLoIdx.y(); dy < 3; ++dy, ++iy) {
+            for (int dz = 0, iz = inLoIdx.z(); dz < 3; ++dz, ++iz) {
+                if (inTree.probeValue(Coord(ix, iy, iz), data[dx][dy][dz])) active = true;
+            }
+        }
+    }
+
+    result = QuadraticSampler::triquadraticInterpolation(data, uvw);
 
     return active;
 }
 
+template<class TreeT>
+inline typename TreeT::ValueType
+QuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord)
+{
+    using ValueT = typename TreeT::ValueType;
 
-////////////////////////////////////////
+    const Vec3i inIdx = local_util::floorVec3(inCoord), inLoIdx = inIdx - Vec3i(1, 1, 1);
+    const Vec3R uvw = inCoord - inIdx;
+
+    // Retrieve the values of the 27 voxels surrounding the
+    // fractional source coordinates.
+    ValueT data[3][3][3];
+    for (int dx = 0, ix = inLoIdx.x(); dx < 3; ++dx, ++ix) {
+        for (int dy = 0, iy = inLoIdx.y(); dy < 3; ++dy, ++iy) {
+            for (int dz = 0, iz = inLoIdx.z(); dz < 3; ++dz, ++iz) {
+                data[dx][dy][dz] = inTree.getValue(Coord(ix, iy, iz));
+            }
+        }
+    }
+
+    return QuadraticSampler::triquadraticInterpolation(data, uvw);
+}
+
+
+//////////////////////////////////////// StaggeredPointSampler
 
 
 template<class TreeT>
 inline bool
 StaggeredPointSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
+                              typename TreeT::ValueType& result)
 {
-    typedef typename TreeT::ValueType ValueType;
+    using ValueType = typename TreeT::ValueType;
 
     ValueType tempX, tempY, tempZ;
     bool active = false;
@@ -749,16 +920,29 @@ StaggeredPointSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
     return active;
 }
 
+template<class TreeT>
+inline typename TreeT::ValueType
+StaggeredPointSampler::sample(const TreeT& inTree, const Vec3R& inCoord)
+{
+    using ValueT = typename TreeT::ValueType;
 
-////////////////////////////////////////
+    const ValueT tempX = PointSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.5, 0.0, 0.0));
+    const ValueT tempY = PointSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.0, 0.5, 0.0));
+    const ValueT tempZ = PointSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.0, 0.0, 0.5));
+
+    return ValueT(tempX.x(), tempY.y(), tempZ.z());
+}
+
+
+//////////////////////////////////////// StaggeredBoxSampler
 
 
 template<class TreeT>
 inline bool
 StaggeredBoxSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
+                            typename TreeT::ValueType& result)
 {
-    typedef typename TreeT::ValueType ValueType;
+    using ValueType = typename TreeT::ValueType;
 
     ValueType tempX, tempY, tempZ;
     tempX = tempY = tempZ = zeroVal<ValueType>();
@@ -775,8 +959,21 @@ StaggeredBoxSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
     return active;
 }
 
+template<class TreeT>
+inline typename TreeT::ValueType
+StaggeredBoxSampler::sample(const TreeT& inTree, const Vec3R& inCoord)
+{
+    using ValueT = typename TreeT::ValueType;
 
-////////////////////////////////////////
+    const ValueT tempX = BoxSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.5, 0.0, 0.0));
+    const ValueT tempY = BoxSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.0, 0.5, 0.0));
+    const ValueT tempZ = BoxSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.0, 0.0, 0.5));
+
+    return ValueT(tempX.x(), tempY.y(), tempZ.z());
+}
+
+
+//////////////////////////////////////// StaggeredQuadraticSampler
 
 
 template<class TreeT>
@@ -784,7 +981,7 @@ inline bool
 StaggeredQuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
     typename TreeT::ValueType& result)
 {
-    typedef typename TreeT::ValueType ValueType;
+    using ValueType = typename TreeT::ValueType;
 
     ValueType tempX, tempY, tempZ;
     bool active = false;
@@ -800,12 +997,45 @@ StaggeredQuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
     return active;
 }
 
+template<class TreeT>
+inline typename TreeT::ValueType
+StaggeredQuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord)
+{
+    using ValueT = typename TreeT::ValueType;
+
+    const ValueT tempX = QuadraticSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.5, 0.0, 0.0));
+    const ValueT tempY = QuadraticSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.0, 0.5, 0.0));
+    const ValueT tempZ = QuadraticSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.0, 0.0, 0.5));
+
+    return ValueT(tempX.x(), tempY.y(), tempZ.z());
+}
+
+//////////////////////////////////////// Sampler
+
+template <>
+struct Sampler<0, false> : public PointSampler {};
+
+template <>
+struct Sampler<1, false> : public BoxSampler {};
+
+template <>
+struct Sampler<2, false> : public QuadraticSampler {};
+
+template <>
+struct Sampler<0, true> : public StaggeredPointSampler {};
+
+template <>
+struct Sampler<1, true> : public StaggeredBoxSampler {};
+
+template <>
+struct Sampler<2, true> : public StaggeredQuadraticSampler {};
+
 } // namespace tools
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_TOOLS_INTERPOLATION_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
