@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -27,12 +27,10 @@
 // LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
 //
 ///////////////////////////////////////////////////////////////////////////
-//
-///////////////////////////////////////////////////////////////////////////
-//
+
 /// @author Ken Museth
 ///
-/// @file LevelSetAdvect.h
+/// @file tools/LevelSetAdvect.h
 ///
 /// @brief Hyperbolic advection of narrow-band level sets
 
@@ -45,8 +43,9 @@
 #include "LevelSetTracker.h"
 #include "VelocityFields.h" // for EnrightField
 #include <openvdb/math/FiniteDifference.h>
-#include <boost/math/constants/constants.hpp>
-#include <openvdb/util/CpuTimer.h>
+//#include <openvdb/util/CpuTimer.h>
+#include <functional>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -76,7 +75,7 @@ namespace tools {
 /// typically depends on the transformation of that grid. This design
 /// is chosen for performance reasons. Finally we will assume that the
 /// functor method is NOT threadsafe (typically uses a ValueAccessor)
-/// and that its lightweight enough that we can copy it per thread.    
+/// and that its lightweight enough that we can copy it per thread.
 ///
 /// The @c InterruptType template argument below refers to any class
 /// with the following interface:
@@ -84,9 +83,9 @@ namespace tools {
 /// class Interrupter {
 ///   ...
 /// public:
-///   void start(const char* name = NULL)// called when computations begin
-///   void end()                         // called when computations end
-///   bool wasInterrupted(int percent=-1)// return true to break computation
+///   void start(const char* name = nullptr) // called when computations begin
+///   void end()                             // called when computations end
+///   bool wasInterrupted(int percent=-1)    // return true to break computation
 ///};
 /// @endcode
 ///
@@ -101,50 +100,57 @@ template<typename GridT,
 class LevelSetAdvection
 {
 public:
-    typedef GridT                              GridType;
-    typedef LevelSetTracker<GridT, InterruptT> TrackerT;
-    typedef typename TrackerT::LeafRange       LeafRange;
-    typedef typename TrackerT::LeafType        LeafType;
-    typedef typename TrackerT::BufferType      BufferType;
-    typedef typename TrackerT::ValueType       ValueType;
-    typedef typename FieldT::VectorType        VectorType;
+    using GridType = GridT;
+    using TrackerT = LevelSetTracker<GridT, InterruptT>;
+    using LeafRange = typename TrackerT::LeafRange;
+    using LeafType = typename TrackerT::LeafType;
+    using BufferType = typename TrackerT::BufferType;
+    using ValueType = typename TrackerT::ValueType;
+    using VectorType = typename FieldT::VectorType;
 
     /// Main constructor
-    LevelSetAdvection(GridT& grid, const FieldT& field, InterruptT* interrupt = NULL):
+    LevelSetAdvection(GridT& grid, const FieldT& field, InterruptT* interrupt = nullptr):
         mTracker(grid, interrupt), mField(field),
         mSpatialScheme(math::HJWENO5_BIAS),
         mTemporalScheme(math::TVD_RK2) {}
 
     virtual ~LevelSetAdvection() {}
 
-    /// @return the spatial finite difference scheme
+    /// @brief Return the spatial finite difference scheme
     math::BiasedGradientScheme getSpatialScheme() const { return mSpatialScheme; }
     /// @brief Set the spatial finite difference scheme
     void setSpatialScheme(math::BiasedGradientScheme scheme) { mSpatialScheme = scheme; }
 
-    /// @return the temporal integration scheme
+    /// @brief Return the temporal integration scheme
     math::TemporalIntegrationScheme getTemporalScheme() const { return mTemporalScheme; }
     /// @brief Set the spatial finite difference scheme
     void setTemporalScheme(math::TemporalIntegrationScheme scheme) { mTemporalScheme = scheme; }
 
-    /// @return the spatial finite difference scheme
-    math::BiasedGradientScheme getTrackerSpatialScheme() const { return mTracker.getSpatialScheme(); }
+    /// @brief Return the spatial finite difference scheme
+    math::BiasedGradientScheme getTrackerSpatialScheme() const {
+        return mTracker.getSpatialScheme();
+    }
     /// @brief Set the spatial finite difference scheme
-    void setTrackerSpatialScheme(math::BiasedGradientScheme scheme) { mTracker.setSpatialScheme(scheme); }
-
-    /// @return the temporal integration scheme
-    math::TemporalIntegrationScheme getTrackerTemporalScheme() const { return mTracker.getTemporalScheme(); }
+    void setTrackerSpatialScheme(math::BiasedGradientScheme scheme) {
+        mTracker.setSpatialScheme(scheme);
+    }
+    /// @brief Return the temporal integration scheme
+    math::TemporalIntegrationScheme getTrackerTemporalScheme() const {
+        return mTracker.getTemporalScheme();
+    }
     /// @brief Set the spatial finite difference scheme
-    void setTrackerTemporalScheme(math::TemporalIntegrationScheme scheme) { mTracker.setTemporalScheme(scheme); }
+    void setTrackerTemporalScheme(math::TemporalIntegrationScheme scheme) {
+        mTracker.setTemporalScheme(scheme);
+    }
 
-    /// @return The number of normalizations performed per track or
+    /// @brief Return The number of normalizations performed per track or
     /// normalize call.
     int  getNormCount() const { return mTracker.getNormCount(); }
     /// @brief Set the number of normalizations performed per track or
     /// normalize call.
     void setNormCount(int n) { mTracker.setNormCount(n); }
 
-    /// @return the grain-size used for multi-threading
+    /// @brief Return the grain-size used for multi-threading
     int  getGrainSize() const { return mTracker.getGrainSize(); }
     /// @brief Set the grain-size used for multi-threading.
     /// @note A grain size of 0 or less disables multi-threading!
@@ -157,7 +163,6 @@ public:
     size_t advect(ValueType time0, ValueType time1);
 
 private:
-
     // disallow copy construction and copy by assinment!
     LevelSetAdvection(const LevelSetAdvection&);// not implemented
     LevelSetAdvection& operator=(const LevelSetAdvection&);// not implemented
@@ -209,10 +214,10 @@ private:
         VectorType*        mVelocity;
         size_t*            mOffsets;
         const MapT*        mMap;
-        typename boost::function<void (Advect*, const LeafRange&)> mTask;
+        typename std::function<void (Advect*, const LeafRange&)> mTask;
         const bool         mIsMaster;
     }; // end of private Advect struct
-    
+
     template<math::BiasedGradientScheme SpatialScheme>
     size_t advect1(ValueType time0, ValueType time1);
 
@@ -233,6 +238,7 @@ private:
 
 };//end of LevelSetAdvection
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
 inline size_t
 LevelSetAdvection<GridT, FieldT, InterruptT>::advect(ValueType time0, ValueType time1)
@@ -254,6 +260,7 @@ LevelSetAdvection<GridT, FieldT, InterruptT>::advect(ValueType time0, ValueType
     return 0;
 }
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
 template<math::BiasedGradientScheme SpatialScheme>
 inline size_t
@@ -272,9 +279,9 @@ LevelSetAdvection<GridT, FieldT, InterruptT>::advect1(ValueType time0, ValueType
     return 0;
 }
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
-template<math::BiasedGradientScheme SpatialScheme,
-         math::TemporalIntegrationScheme TemporalScheme>
+template<math::BiasedGradientScheme SpatialScheme, math::TemporalIntegrationScheme TemporalScheme>
 inline size_t
 LevelSetAdvection<GridT, FieldT, InterruptT>::advect2(ValueType time0, ValueType time1)
 {
@@ -282,7 +289,8 @@ LevelSetAdvection<GridT, FieldT, InterruptT>::advect2(ValueType time0, ValueType
     if (trans.mapType() == math::UniformScaleMap::mapType()) {
         return this->advect3<SpatialScheme, TemporalScheme, math::UniformScaleMap>(time0, time1);
     } else if (trans.mapType() == math::UniformScaleTranslateMap::mapType()) {
-        return this->advect3<SpatialScheme, TemporalScheme, math::UniformScaleTranslateMap>(time0, time1);
+        return this->advect3<SpatialScheme, TemporalScheme, math::UniformScaleTranslateMap>(
+            time0, time1);
     } else if (trans.mapType() == math::UnitaryMap::mapType()) {
         return this->advect3<SpatialScheme, TemporalScheme, math::UnitaryMap    >(time0, time1);
     } else if (trans.mapType() == math::TranslationMap::mapType()) {
@@ -293,10 +301,12 @@ LevelSetAdvection<GridT, FieldT, InterruptT>::advect2(ValueType time0, ValueType
     return 0;
 }
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
-template<math::BiasedGradientScheme SpatialScheme,
-         math::TemporalIntegrationScheme TemporalScheme,
-         typename MapT>
+template<
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme,
+    typename MapT>
 inline size_t
 LevelSetAdvection<GridT, FieldT, InterruptT>::advect3(ValueType time0, ValueType time1)
 {
@@ -309,24 +319,29 @@ LevelSetAdvection<GridT, FieldT, InterruptT>::advect3(ValueType time0, ValueType
 
 
 template<typename GridT, typename FieldT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
+template<
+    typename MapT,
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme>
 inline
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 Advect<MapT, SpatialScheme, TemporalScheme>::
 Advect(LevelSetAdvection& parent)
     : mParent(parent)
-    , mVelocity(NULL)
-    , mOffsets(NULL)
+    , mVelocity(nullptr)
+    , mOffsets(nullptr)
     , mMap(parent.mTracker.grid().transform().template constMap<MapT>().get())
     , mTask(0)
     , mIsMaster(true)
 {
 }
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
+template<
+    typename MapT,
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme>
 inline
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 Advect<MapT, SpatialScheme, TemporalScheme>::
@@ -339,15 +354,20 @@ Advect(const Advect& other)
     , mIsMaster(false)
 {
 }
-   
+
+
 template<typename GridT, typename FieldT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
+template<
+    typename MapT,
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme>
 inline size_t
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 Advect<MapT, SpatialScheme, TemporalScheme>::
 advect(ValueType time0, ValueType time1)
 {
+    namespace ph = std::placeholders;
+
     //util::CpuTimer timer;
     size_t countCFL = 0;
     if ( math::isZero(time0 - time1) ) return countCFL;
@@ -357,7 +377,7 @@ advect(ValueType time0, ValueType time1)
         //timer.start( "\nallocate buffers" );
         mParent.mTracker.leafs().rebuildAuxBuffers(TemporalScheme == math::TVD_RK3 ? 2 : 1);
         //timer.stop();
-        
+
         const ValueType dt = this->sampleField(time0, time1);
         if ( math::isZero(dt) ) break;//V is essentially zero so terminate
 
@@ -366,7 +386,7 @@ advect(ValueType time0, ValueType time1)
         case math::TVD_RK1:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * VdotG_t0(0)
-            mTask = boost::bind(&Advect::euler01, _1, _2, dt);
+            mTask = std::bind(&Advect::euler01, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook("Advecting level set using TVD_RK1", 1);
@@ -374,14 +394,14 @@ advect(ValueType time0, ValueType time1)
         case math::TVD_RK2:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * VdotG_t0(0)
-            mTask = boost::bind(&Advect::euler01, _1, _2, dt);
+            mTask = std::bind(&Advect::euler01, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook("Advecting level set using TVD_RK1 (step 1 of 2)", 1);
 
             // Convex combine explict Euler step: t2 = t0 + dt
             // Phi_t2(1) = 1/2 * Phi_t0(1) + 1/2 * (Phi_t1(0) - dt * V.Grad_t1(0))
-            mTask = boost::bind(&Advect::euler12, _1, _2, dt);
+            mTask = std::bind(&Advect::euler12, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 1 such that Phi_t2(0) and Phi_t1(1)
             this->cook("Advecting level set using TVD_RK1 (step 2 of 2)", 1);
@@ -389,21 +409,21 @@ advect(ValueType time0, ValueType time1)
         case math::TVD_RK3:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * VdotG_t0(0)
-            mTask = boost::bind(&Advect::euler01, _1, _2, dt);
+            mTask = std::bind(&Advect::euler01, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook("Advecting level set using TVD_RK3 (step 1 of 3)", 1);
 
             // Convex combine explict Euler step: t2 = t0 + dt/2
             // Phi_t2(2) = 3/4 * Phi_t0(1) + 1/4 * (Phi_t1(0) - dt * V.Grad_t1(0))
-            mTask = boost::bind(&Advect::euler34, _1, _2, dt);
+            mTask = std::bind(&Advect::euler34, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 2 such that Phi_t2(0) and Phi_t1(2)
             this->cook("Advecting level set using TVD_RK3 (step 2 of 3)", 2);
 
             // Convex combine explict Euler step: t3 = t0 + dt
             // Phi_t3(2) = 1/3 * Phi_t0(1) + 2/3 * (Phi_t2(0) - dt * V.Grad_t2(0)
-            mTask = boost::bind(&Advect::euler13, _1, _2, dt);
+            mTask = std::bind(&Advect::euler13, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 2 such that Phi_t3(0) and Phi_t2(2)
             this->cook("Advecting level set using TVD_RK3 (step 3 of 3)", 2);
@@ -412,7 +432,7 @@ advect(ValueType time0, ValueType time1)
             OPENVDB_THROW(ValueError, "Temporal integration scheme not supported!");
         }//end of compile-time resolved switch
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_END
-            
+
         time0 += isForward ? dt : -dt;
         ++countCFL;
         mParent.mTracker.leafs().removeAuxBuffers();
@@ -423,26 +443,31 @@ advect(ValueType time0, ValueType time1)
     return countCFL;//number of CLF propagation steps
 }
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
-template<typename MapT, math::BiasedGradientScheme SpatialScheme,
-	math::TemporalIntegrationScheme TemporalScheme>
+template<
+    typename MapT,
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme>
 inline typename GridT::ValueType
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 Advect<MapT, SpatialScheme, TemporalScheme>::
 sampleField(ValueType time0, ValueType time1)
 {
+    namespace ph = std::placeholders;
+
     const int grainSize = mParent.mTracker.getGrainSize();
     const size_t leafCount = mParent.mTracker.leafs().leafCount();
     if (leafCount==0) return ValueType(0.0);
 
-    // Compute the pre-fix sum of offsets to active voxels
-    size_t size=0, voxelCount=mParent.mTracker.leafs().getPreFixSum(mOffsets, size, grainSize);
+    // Compute the prefix sum of offsets to active voxels
+    size_t size=0, voxelCount=mParent.mTracker.leafs().getPrefixSum(mOffsets, size, grainSize);
 
     // Sample the velocity field
     if (mParent.mField.transform() == mParent.mTracker.grid().transform()) {
-        mTask = boost::bind(&Advect::sampleAligned, _1, _2, time0, time1);
+        mTask = std::bind(&Advect::sampleAligned, ph::_1, ph::_2, time0, time1);
     } else {
-        mTask = boost::bind(&Advect::sampleXformed, _1, _2, time0, time1);
+        mTask = std::bind(&Advect::sampleXformed, ph::_1, ph::_2, time0, time1);
     }
     assert(voxelCount == mParent.mTracker.grid().activeVoxelCount());
     mVelocity = new VectorType[ voxelCount ];
@@ -451,7 +476,9 @@ sampleField(ValueType time0, ValueType time1)
     // Find the extrema of the magnitude of the velocities
     ValueType maxAbsV = 0;
     VectorType* v = mVelocity;
-    for (size_t i=0; i<voxelCount; ++i, ++v) maxAbsV = math::Max(maxAbsV, ValueType(v->lengthSqr()));
+    for (size_t i = 0; i < voxelCount; ++i, ++v) {
+        maxAbsV = math::Max(maxAbsV, ValueType(v->lengthSqr()));
+    }
 
     // Compute the CFL number
     if (math::isApproxZero(maxAbsV, math::Delta<ValueType>::value())) return ValueType(0);
@@ -465,17 +492,20 @@ sampleField(ValueType time0, ValueType time1)
     return math::Min(dt, ValueType(CFL*dx/math::Sqrt(maxAbsV)));
 }
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
-template <bool Aligned>
+template<
+    typename MapT,
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme>
+template<bool Aligned>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 Advect<MapT, SpatialScheme, TemporalScheme>::
 sample(const LeafRange& range, ValueType time0, ValueType time1)
 {
     const bool isForward = time0 < time1;
-    typedef typename LeafType::ValueOnCIter VoxelIterT;
+    using VoxelIterT = typename LeafType::ValueOnCIter;
     const MapT& map = *mMap;
     const FieldT field( mParent.mField );
     mParent.mTracker.checkInterrupter();
@@ -489,23 +519,29 @@ sample(const LeafRange& range, ValueType time0, ValueType time1)
     }
 }
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
+template<
+    typename MapT,
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 Advect<MapT, SpatialScheme, TemporalScheme>::
 clearField()
 {
-    delete [] mOffsets; 
+    delete [] mOffsets;
     delete [] mVelocity;
-    mOffsets  = NULL;
-    mVelocity = NULL;
+    mOffsets = nullptr;
+    mVelocity = nullptr;
 }
 
+
 template<typename GridT, typename FieldT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
+template<
+    typename MapT,
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 Advect<MapT, SpatialScheme, TemporalScheme>::
@@ -523,21 +559,24 @@ cook(const char* msg, size_t swapBuffer)
     mParent.mTracker.endInterrupter();
 }
 
+
 // Convex combination of Phi and a forward Euler advection steps:
 // Phi(result) = alpha * Phi(phi) + (1-alpha) * (Phi(0) - dt * V.Grad(0));
 template<typename GridT, typename FieldT, typename InterruptT>
-template<typename MapT, math::BiasedGradientScheme SpatialScheme,
-         math::TemporalIntegrationScheme TemporalScheme>
+template<
+    typename MapT,
+    math::BiasedGradientScheme SpatialScheme,
+    math::TemporalIntegrationScheme TemporalScheme>
 template <int Nominator, int Denominator>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 Advect<MapT, SpatialScheme, TemporalScheme>::
 euler(const LeafRange& range, ValueType dt, Index phiBuffer, Index resultBuffer)
 {
-    typedef math::BIAS_SCHEME<SpatialScheme>                             SchemeT;
-    typedef typename SchemeT::template ISStencil<GridType>::StencilType  StencilT;
-    typedef typename LeafType::ValueOnCIter                              VoxelIterT;
-    typedef math::GradientBiased<MapT, SpatialScheme>                    GradT;
+    using SchemeT = math::BIAS_SCHEME<SpatialScheme>;
+    using StencilT = typename SchemeT::template ISStencil<GridType>::StencilType;
+    using VoxelIterT = typename LeafType::ValueOnCIter;
+    using GradT = math::GradientBiased<MapT, SpatialScheme>;
 
     static const ValueType Alpha = ValueType(Nominator)/ValueType(Denominator);
     static const ValueType Beta  = ValueType(1) - Alpha;
@@ -552,7 +591,8 @@ euler(const LeafRange& range, ValueType dt, Index phiBuffer, Index resultBuffer)
         for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter, ++vel) {
             const Index i = voxelIter.pos();
             stencil.moveTo(voxelIter);
-            const ValueType a = stencil.getValue() - dt * vel->dot(GradT::result(map, stencil, *vel));
+            const ValueType a =
+                stencil.getValue() - dt * vel->dot(GradT::result(map, stencil, *vel));
             result[i] = Nominator ? Alpha * phi[i] + Beta * a : a;
         }//loop over active voxels in the leaf of the mask
     }//loop over leafs of the level set
@@ -564,6 +604,6 @@ euler(const LeafRange& range, ValueType dt, Index phiBuffer, Index resultBuffer)
 
 #endif // OPENVDB_TOOLS_LEVEL_SET_ADVECT_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
