@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -27,12 +27,10 @@
 // LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
 //
 ///////////////////////////////////////////////////////////////////////////
-//
-///////////////////////////////////////////////////////////////////////////
-//
+
 /// @author Ken Museth
 ///
-/// @file LevelSetMorph.h
+/// @file tools/LevelSetMorph.h
 ///
 /// @brief Shape morphology of level sets. Morphing from a source
 /// narrow-band level sets to a target narrow-band level set.
@@ -43,13 +41,15 @@
 #include "LevelSetTracker.h"
 #include "Interpolation.h" // for BoxSampler, etc.
 #include <openvdb/math/FiniteDifference.h>
+#include <functional>
+#include <limits>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace tools {
 
-
 /// @brief Shape morphology of level sets. Morphing from a source
 /// narrow-band level sets to a target narrow-band level set.
 ///
@@ -60,9 +60,9 @@ namespace tools {
 /// class Interrupter {
 ///   ...
 /// public:
-///   void start(const char* name = NULL)// called when computations begin
-///   void end()                         // called when computations end
-///   bool wasInterrupted(int percent=-1)// return true to break computation
+///   void start(const char* name = nullptr) // called when computations begin
+///   void end()                             // called when computations end
+///   bool wasInterrupted(int percent=-1)    // return true to break computation
 /// };
 /// @endcode
 ///
@@ -74,21 +74,19 @@ template<typename GridT,
 class LevelSetMorphing
 {
 public:
-    typedef GridT                              GridType;
-    typedef typename GridT::TreeType           TreeType;
-    typedef LevelSetTracker<GridT, InterruptT> TrackerT;
-    typedef typename TrackerT::LeafRange       LeafRange;
-    typedef typename TrackerT::LeafType        LeafType;
-    typedef typename TrackerT::BufferType      BufferType;
-    typedef typename TrackerT::ValueType       ValueType;
+    using GridType = GridT;
+    using TreeType = typename GridT::TreeType;
+    using TrackerT = LevelSetTracker<GridT, InterruptT>;
+    using LeafRange = typename TrackerT::LeafRange;
+    using LeafType = typename TrackerT::LeafType;
+    using BufferType = typename TrackerT::BufferType;
+    using ValueType = typename TrackerT::ValueType;
 
     /// Main constructor
-    LevelSetMorphing(GridT& sourceGrid,
-                     const GridT& targetGrid,
-                     InterruptT* interrupt = NULL)
+    LevelSetMorphing(GridT& sourceGrid, const GridT& targetGrid, InterruptT* interrupt = nullptr)
         : mTracker(sourceGrid, interrupt)
         , mTarget(&targetGrid)
-        , mMask(NULL)
+        , mMask(nullptr)
         , mSpatialScheme(math::HJWENO5_BIAS)
         , mTemporalScheme(math::TVD_RK2)
         , mMinMask(0)
@@ -256,7 +254,7 @@ private:
         inline void euler34(const LeafRange& r, ValueType t) {this->euler<3,4>(r, t, 1, 2, 3);}
         inline void euler13(const LeafRange& r, ValueType t) {this->euler<1,3>(r, t, 1, 2, 3);}
 
-        typedef typename boost::function<void (Morph*, const LeafRange&)> FuncType;
+        using FuncType = typename std::function<void (Morph*, const LeafRange&)>;
         LevelSetMorphing* mParent;
         ValueType         mMinAbsS, mMaxAbsS;
         const MapT*       mMap;
@@ -280,6 +278,10 @@ LevelSetMorphing<GridT, InterruptT>::advect(ValueType time0, ValueType time1)
     //return this->advect1<math::WENO5_BIAS  >(time0, time1);
     case math::HJWENO5_BIAS:
         return this->advect1<math::HJWENO5_BIAS>(time0, time1);
+    case math::SECOND_BIAS:
+    case math::THIRD_BIAS:
+    case math::WENO5_BIAS:
+    case math::UNKNOWN_BIAS:
     default:
         OPENVDB_THROW(ValueError, "Spatial difference scheme not supported!");
     }
@@ -298,6 +300,7 @@ LevelSetMorphing<GridT, InterruptT>::advect1(ValueType time0, ValueType time1)
         return this->advect2<SpatialScheme, math::TVD_RK2>(time0, time1);
     case math::TVD_RK3:
         return this->advect2<SpatialScheme, math::TVD_RK3>(time0, time1);
+    case math::UNKNOWN_TIS:
     default:
         OPENVDB_THROW(ValueError, "Temporal integration scheme not supported!");
     }
@@ -392,6 +395,8 @@ LevelSetMorphing<GridT, InterruptT>::
 Morph<MapT, SpatialScheme, TemporalScheme>::
 advect(ValueType time0, ValueType time1)
 {
+    namespace ph = std::placeholders;
+
     // Make sure we have enough temporal auxiliary buffers for the time
     // integration AS WELL AS an extra buffer with the speed function!
     static const Index auxBuffers = 1 + (TemporalScheme == math::TVD_RK3 ? 2 : 1);
@@ -407,7 +412,7 @@ advect(ValueType time0, ValueType time1)
         case math::TVD_RK1:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * Speed(2) * |Grad[Phi(0)]|
-            mTask = boost::bind(&Morph::euler01, _1, _2, dt, /*speed*/2);
+            mTask = std::bind(&Morph::euler01, ph::_1, ph::_2, dt, /*speed*/2);
 
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
@@ -415,14 +420,14 @@ advect(ValueType time0, ValueType time1)
         case math::TVD_RK2:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * Speed(2) * |Grad[Phi(0)]|
-            mTask = boost::bind(&Morph::euler01, _1, _2, dt, /*speed*/2);
+            mTask = std::bind(&Morph::euler01, ph::_1, ph::_2, dt, /*speed*/2);
 
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
 
             // Convex combine explict Euler step: t2 = t0 + dt
             // Phi_t2(1) = 1/2 * Phi_t0(1) + 1/2 * (Phi_t1(0) - dt * Speed(2) * |Grad[Phi(0)]|)
-            mTask = boost::bind(&Morph::euler12, _1, _2, dt);
+            mTask = std::bind(&Morph::euler12, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 1 such that Phi_t2(0) and Phi_t1(1)
             this->cook(PARALLEL_FOR, 1);
@@ -430,25 +435,26 @@ advect(ValueType time0, ValueType time1)
         case math::TVD_RK3:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * Speed(3) * |Grad[Phi(0)]|
-            mTask = boost::bind(&Morph::euler01, _1, _2, dt, /*speed*/3);
+            mTask = std::bind(&Morph::euler01, ph::_1, ph::_2, dt, /*speed*/3);
 
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
 
             // Convex combine explict Euler step: t2 = t0 + dt/2
             // Phi_t2(2) = 3/4 * Phi_t0(1) + 1/4 * (Phi_t1(0) - dt * Speed(3) * |Grad[Phi(0)]|)
-            mTask = boost::bind(&Morph::euler34, _1, _2, dt);
+            mTask = std::bind(&Morph::euler34, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 2 such that Phi_t2(0) and Phi_t1(2)
             this->cook(PARALLEL_FOR, 2);
 
             // Convex combine explict Euler step: t3 = t0 + dt
             // Phi_t3(2) = 1/3 * Phi_t0(1) + 2/3 * (Phi_t2(0) - dt * Speed(3) * |Grad[Phi(0)]|)
-            mTask = boost::bind(&Morph::euler13, _1, _2, dt);
+            mTask = std::bind(&Morph::euler13, ph::_1, ph::_2, dt);
 
             // Cook and swap buffer 0 and 2 such that Phi_t3(0) and Phi_t2(2)
             this->cook(PARALLEL_FOR, 2);
             break;
+        case math::UNKNOWN_TIS:
         default:
             OPENVDB_THROW(ValueError, "Temporal integration scheme not supported!");
         }//end of compile-time resolved switch
@@ -473,16 +479,18 @@ LevelSetMorphing<GridT, InterruptT>::
 Morph<MapT, SpatialScheme, TemporalScheme>::
 sampleSpeed(ValueType time0, ValueType time1, Index speedBuffer)
 {
+    namespace ph = std::placeholders;
+
     mMaxAbsS = mMinAbsS;
     const size_t leafCount = mParent->mTracker.leafs().leafCount();
     if (leafCount==0 || time0 >= time1) return ValueType(0);
 
     const math::Transform& xform  = mParent->mTracker.grid().transform();
     if (mParent->mTarget->transform() == xform &&
-        (mParent->mMask == NULL || mParent->mMask->transform() == xform)) {
-        mTask = boost::bind(&Morph::sampleAlignedSpeed, _1, _2, speedBuffer);
+        (mParent->mMask == nullptr || mParent->mMask->transform() == xform)) {
+        mTask = std::bind(&Morph::sampleAlignedSpeed, ph::_1, ph::_2, speedBuffer);
     } else {
-        mTask = boost::bind(&Morph::sampleXformedSpeed, _1, _2, speedBuffer);
+        mTask = std::bind(&Morph::sampleXformedSpeed, ph::_1, ph::_2, speedBuffer);
     }
     this->cook(PARALLEL_REDUCE);
     if (math::isApproxEqual(mMinAbsS, mMaxAbsS)) return ValueType(0);//speed is essentially zero
@@ -501,14 +509,15 @@ LevelSetMorphing<GridT, InterruptT>::
 Morph<MapT, SpatialScheme, TemporalScheme>::
 sampleXformedSpeed(const LeafRange& range, Index speedBuffer)
 {
-    typedef typename LeafType::ValueOnCIter VoxelIterT;
-    typedef tools::GridSampler<typename GridT::ConstAccessor, tools::BoxSampler> SamplerT;
+    using VoxelIterT = typename LeafType::ValueOnCIter;
+    using SamplerT = tools::GridSampler<typename GridT::ConstAccessor, tools::BoxSampler>;
+
     const MapT& map = *mMap;
     mParent->mTracker.checkInterrupter();
 
     typename GridT::ConstAccessor targetAcc = mParent->mTarget->getAccessor();
     SamplerT target(targetAcc, mParent->mTarget->transform());
-    if (mParent->mMask == NULL) {
+    if (mParent->mMask == nullptr) {
         for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
             ValueType* speed = leafIter.buffer(speedBuffer).data();
             bool isZero = true;
@@ -550,12 +559,13 @@ LevelSetMorphing<GridT, InterruptT>::
 Morph<MapT, SpatialScheme, TemporalScheme>::
 sampleAlignedSpeed(const LeafRange& range, Index speedBuffer)
 {
-    typedef typename LeafType::ValueOnCIter VoxelIterT;
+    using VoxelIterT = typename LeafType::ValueOnCIter;
+
     mParent->mTracker.checkInterrupter();
 
     typename GridT::ConstAccessor target = mParent->mTarget->getAccessor();
 
-    if (mParent->mMask == NULL) {
+    if (mParent->mMask == nullptr) {
         for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
             ValueType* speed = leafIter.buffer(speedBuffer).data();
             bool isZero = true;
@@ -608,7 +618,8 @@ cook(ThreadingMode mode, size_t swapBuffer)
     } else if (mode == PARALLEL_REDUCE) {
         tbb::parallel_reduce(range, *this);
     } else {
-        throw std::runtime_error("Undefined threading mode");
+        OPENVDB_THROW(ValueError, "expected threading mode " << int(PARALLEL_FOR)
+            << " or " << int(PARALLEL_REDUCE) << ", got " << int(mode));
     }
 
     mParent->mTracker.leafs().swapLeafBuffer(swapBuffer, grainSize == 0);
@@ -626,10 +637,10 @@ Morph<MapT, SpatialScheme, TemporalScheme>::
 euler(const LeafRange& range, ValueType dt,
       Index phiBuffer, Index resultBuffer, Index speedBuffer)
 {
-    typedef math::BIAS_SCHEME<SpatialScheme>                             SchemeT;
-    typedef typename SchemeT::template ISStencil<GridType>::StencilType  StencilT;
-    typedef typename LeafType::ValueOnCIter                              VoxelIterT;
-    typedef math::GradientNormSqrd<MapT, SpatialScheme>                  NumGrad;
+    using SchemeT = math::BIAS_SCHEME<SpatialScheme>;
+    using StencilT = typename SchemeT::template ISStencil<GridType>::StencilType;
+    using VoxelIterT = typename LeafType::ValueOnCIter;
+    using NumGrad = math::GradientNormSqrd<MapT, SpatialScheme>;
 
     static const ValueType Alpha = ValueType(Nominator)/ValueType(Denominator);
     static const ValueType Beta  = ValueType(1) - Alpha;
@@ -659,6 +670,6 @@ euler(const LeafRange& range, ValueType dt,
 
 #endif // OPENVDB_TOOLS_LEVEL_SET_MORPH_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
