@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -46,22 +46,20 @@
 #ifndef OPENVDB_TOOLS_MORPHOLOGY_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_MORPHOLOGY_HAS_BEEN_INCLUDED
 
-#include <tbb/tbb_thread.h>
-#include <tbb/task_scheduler_init.h>
-#include <tbb/enumerable_thread_specific.h>
-#include <tbb/parallel_for.h>
 #include <openvdb/Types.h>
 #include <openvdb/Grid.h>
 #include <openvdb/math/Math.h> // for isApproxEqual()
 #include <openvdb/tree/TreeIterator.h>
 #include <openvdb/tree/ValueAccessor.h>
 #include <openvdb/tree/LeafManager.h>
-#include <boost/scoped_array.hpp>
-#include <boost/bind.hpp>
-#include <boost/utility/enable_if.hpp>
-#include <boost/type_traits/is_same.hpp>
 #include "Prune.h"// for pruneLevelSet
 #include "ValueTransformer.h" // for foreach()
+#include <tbb/tbb_thread.h>
+#include <tbb/task_scheduler_init.h>
+#include <tbb/enumerable_thread_specific.h>
+#include <tbb/parallel_for.h>
+#include <type_traits>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -134,7 +132,9 @@ inline void dilateActiveValues(TreeType& tree,
 ///
 /// @note This method is fully multi-threaded and support active tiles!
 ///
-/// @param manager       Leaf node manager for the tree to be dilated
+/// @param manager       Leaf node manager for the tree to be dilated.
+///                      On exit it is updated to include all the leaf
+///                      nodes of the dilated tree.
 /// @param iterations    number of iterations to apply the dilation
 /// @param nn            connectivity pattern of the dilation: either
 ///     face-adjacent (6 nearest neighbors), face- and edge-adjacent
@@ -173,6 +173,8 @@ inline void dilateVoxels(TreeType& tree,
 /// @warning This method is NOT multi-threaded and ignores active tiles!
 ///
 /// @param manager       LeafManager containing the tree to be dilated.
+///                      On exit it is updated to include all the leaf
+///                      nodes of the dilated tree.
 /// @param iterations    number of iterations to apply the dilation
 /// @param nn           connectivity pattern of the dilation: either
 ///     face-adjacent (6 nearest neighbors), face- and edge-adjacent
@@ -291,8 +293,8 @@ protected:
         LeafType* leaf;//null if a tile
         bool      init;//true if initialization is required
         bool      isOn;//true if an active tile
-        Neighbor() : leaf(NULL), init(true) {}
-        inline void clear() { leaf = NULL; init = true; }
+        Neighbor() : leaf(nullptr), init(true) {}
+        inline void clear() { leaf = nullptr; init = true; }
         template<int DX, int DY, int DZ>
         void scatter(AccessorType& acc, const Coord &xyz, int indx, Word mask)
         {
@@ -300,7 +302,7 @@ protected:
                 init = false;
                 Coord orig = xyz.offsetBy(DX*LEAF_DIM, DY*LEAF_DIM, DZ*LEAF_DIM);
                 leaf = acc.probeLeaf(orig);
-                if (leaf==NULL && !acc.isValueOn(orig)) leaf = acc.touchLeaf(orig);
+                if ((leaf == nullptr) && !acc.isValueOn(orig)) leaf = acc.touchLeaf(orig);
             }
 #ifndef _MSC_VER // Visual C++ doesn't guarantee thread-safe initialization of local statics
             static
@@ -336,7 +338,7 @@ protected:
         }
         ~LeafCache() { delete [] leafs; }
         LeafType*& operator[](int offset) { return leafs[offset]; }
-        inline void clear() { for (size_t i=0; i<size; ++i) leafs[i]=NULL; }
+        inline void clear() { for (size_t i = 0; i < size; ++i) leafs[i] = nullptr; }
         inline void setOrigin(const Coord& xyz) { origin = &xyz; }
         inline void scatter(int n, int indx)
         {
@@ -396,7 +398,7 @@ protected:
         void erode18(const RangeT&) const;
         void erode26(const RangeT&) const;
     private:
-        typedef typename boost::function<void (ErodeVoxelsOp*, const RangeT&)> FuncT;
+        using FuncT = typename std::function<void (ErodeVoxelsOp*, const RangeT&)>;
         FuncT                  mTask;
         std::vector<MaskType>& mSavedMasks;
         ManagerType&           mManager;
@@ -462,6 +464,7 @@ Morphology<TreeType>::dilateVoxels(int iterations, NearestNeighbors nn)
         case NN_FACE_EDGE_VERTEX:
             this->dilateVoxels26();
             break;
+        case NN_FACE:
         default:
             this->dilateVoxels6();
         }
@@ -700,15 +703,17 @@ template<typename TreeType>
 inline void
 Morphology<TreeType>::ErodeVoxelsOp::runParallel(NearestNeighbors nn)
 {
+    namespace ph = std::placeholders;
     switch (nn) {
     case NN_FACE_EDGE:
-        mTask = boost::bind(&ErodeVoxelsOp::erode18, _1, _2);
+        mTask = std::bind(&ErodeVoxelsOp::erode18, ph::_1, ph::_2);
         break;
     case NN_FACE_EDGE_VERTEX:
-        mTask = boost::bind(&ErodeVoxelsOp::erode26, _1, _2);
+        mTask = std::bind(&ErodeVoxelsOp::erode26, ph::_1, ph::_2);
         break;
+    case NN_FACE:
     default:
-        mTask = boost::bind(&ErodeVoxelsOp::erode6, _1, _2);
+        mTask = std::bind(&ErodeVoxelsOp::erode6, ph::_1, ph::_2);
     }
     tbb::parallel_for(mManager.getRange(), *this);
 }
@@ -999,7 +1004,7 @@ class DilationOp
 public:
 
     DilationOp(TreeT &tree, int iterations, NearestNeighbors nn, TilePolicy mode)
-        : mIter(iterations), mNN(nn), mPool(NULL), mLeafs(NULL)
+        : mIter(iterations), mNN(nn), mPool(nullptr), mLeafs(nullptr)
     {
         const size_t numLeafs = this->init( tree, mode );
         const size_t numThreads = size_t(tbb::task_scheduler_init::default_num_threads());
@@ -1043,7 +1048,7 @@ private:
     // Convert active tiles to leafs and de-construct the tree into a linear array of leafs.
     size_t linearize(MaskT& mask, TilePolicy mode)
     {
-        if (mode != IGNORE_TILES) mask.voxelizeActiveTiles();// light-weight since this is a mask tree
+        if (mode != IGNORE_TILES) mask.voxelizeActiveTiles();//lightweight since this is a mask tree
         const size_t numLeafs = mask.leafCount();
         mLeafs = new LeafT*[numLeafs];// fast pre-allocation
         MyArray tmp(mLeafs);
@@ -1051,14 +1056,16 @@ private:
         return numLeafs;
     }
 
-    template <typename T>
-    typename boost::enable_if<boost::is_same<T,MaskT>,size_t>::type init(T& tree, TilePolicy mode)
+    template<typename T>
+    typename std::enable_if<std::is_same<T, MaskT>::value, size_t>::type
+    init(T& tree, TilePolicy mode)
     {
         return this->linearize(tree, mode);
     }
 
-    template <typename T>
-    typename boost::disable_if<boost::is_same<T,MaskT>,size_t>::type init(const T& tree, TilePolicy mode)
+    template<typename T>
+    typename std::enable_if<!std::is_same<T, MaskT>::value, size_t>::type
+    init(const T& tree, TilePolicy mode)
     {
         MaskT mask(tree, false, true, TopologyCopy());
         return this->linearize(mask, mode);
@@ -1092,6 +1099,6 @@ dilateActiveValues(tree::LeafManager<TreeType>& manager,
 
 #endif // OPENVDB_TOOLS_MORPHOLOGY_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
