@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -27,10 +27,10 @@
 // LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
 //
 ///////////////////////////////////////////////////////////////////////////
-//
+
 /// @author Ken Museth
 ///
-/// @file ParticlesToLevelSet.h
+/// @file tools/ParticlesToLevelSet.h
 ///
 /// @brief This tool converts particles (with position, radius and velocity)
 /// into a signed distance field encoded as a narrow band level set.
@@ -39,8 +39,8 @@
 /// as the level set grid.
 ///
 /// @note This fast particle to level set converter is always intended
-/// to be combined with some kind of surface postprocessing,
-/// i.e. tools::Filter. Without such postprocessing the generated
+/// to be combined with some kind of surface post processing,
+/// i.e. tools::Filter. Without such post processing the generated
 /// surface is typically too noisy and blobby. However it serves as a
 /// great and fast starting point for subsequent level set surface
 /// processing and convolution.
@@ -53,10 +53,11 @@
 /// class ParticleList {
 ///   ...
 /// public:
+///   using PosType = openvdb::Vec3R;
 ///
 ///   // Return the total number of particles in list.
 ///   // Always required!
-///   size_t         size()          const;
+///   size_t size() const;
 ///
 ///   // Get the world space position of the nth particle.
 ///   // Required by ParticledToLevelSet::rasterizeSphere(*this,radius).
@@ -84,9 +85,9 @@
 /// class Interrupter {
 ///   ...
 /// public:
-///   void start(const char* name = NULL)// called when computations begin
-///   void end()                         // called when computations end
-///   bool wasInterrupted(int percent=-1)// return true to break computation
+///   void start(const char* name = nullptr) // called when computations begin
+///   void end()                             // called when computations end
+///   bool wasInterrupted(int percent=-1)    // return true to break computation
 /// };
 /// @endcode
 ///
@@ -99,11 +100,6 @@
 
 #include <tbb/parallel_reduce.h>
 #include <tbb/blocked_range.h>
-#include <boost/bind.hpp>
-#include <boost/function.hpp>
-#include <boost/type_traits/is_floating_point.hpp>
-#include <boost/utility/enable_if.hpp>
-#include <boost/mpl/if.hpp>
 #include <openvdb/Types.h>
 #include <openvdb/Grid.h>
 #include <openvdb/math/Math.h>
@@ -113,6 +109,9 @@
 #include "PointPartitioner.h"
 #include "Prune.h"
 #include "SignedFloodFill.h"
+#include <iostream>
+#include <type_traits>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -122,7 +121,7 @@ namespace tools {
 namespace p2ls_internal {
 // This is a simple type that combines a distance value and a particle
 // attribute. It's required for attribute transfer which is performed
-// in the ParticlesToLevelSet::Raster memberclass defined below.
+// in the ParticlesToLevelSet::Raster member class defined below.
 template<typename VisibleT, typename BlindT> class BlindData;
 }// namespace p2ls_internal
 
@@ -133,16 +132,17 @@ template<typename SdfGridT,
 class ParticlesToLevelSet
 {
 public:
-    typedef typename boost::is_void<AttributeT>::type DisableT;
-    typedef InterrupterT                          InterrupterType;
+    using DisableT = typename std::is_void<AttributeT>::type;
+    using InterrupterType = InterrupterT;
 
-    typedef SdfGridT                              SdfGridType;
-    typedef typename SdfGridT::ValueType          SdfType;
+    using SdfGridType = SdfGridT;
+    using SdfType = typename SdfGridT::ValueType;
 
-    typedef typename boost::mpl::if_<DisableT, size_t, AttributeT>::type  AttType;
-    typedef typename SdfGridT::template ValueConverter<AttType>::Type AttGridType;
+    using AttType = typename std::conditional<DisableT::value, size_t, AttributeT>::type;
+    using AttGridType = typename SdfGridT::template ValueConverter<AttType>::Type;
 
-    BOOST_STATIC_ASSERT(boost::is_floating_point<SdfType>::value);
+    static_assert(std::is_floating_point<SdfType>::value,
+        "ParticlesToLevelSet requires an SDF grid with floating-point values");
 
     /// @brief Constructor using an exiting signed distance,
     /// i.e. narrow band level set, grid.
@@ -151,8 +151,8 @@ public:
     /// @param interrupt Callback to interrupt a long-running process
     ///
     /// @note The input grid is assumed to be a valid level set and if
-    /// it already contains voxels (with SDF values) partices are unioned
-    /// onto the exisinting level set surface. However, if attribute tranfer
+    /// it already contains voxels (with SDF values) particles are unioned
+    /// onto the existing level set surface. However, if attribute transfer
     /// is enabled, i.e. AttributeT != void, attributes are only
     /// generated for voxels that overlap with particles, not the existing
     /// voxels in the input grid (for which no attributes exist!).
@@ -165,7 +165,7 @@ public:
     /// band level sets. Finally the default NullInterrupter should
     /// compile out interruption checks during optimization, thus
     /// incurring no run-time overhead.
-    explicit ParticlesToLevelSet(SdfGridT& grid, InterrupterT* interrupt = NULL);
+    explicit ParticlesToLevelSet(SdfGridT& grid, InterrupterT* interrupt = nullptr);
 
     /// Destructor
     ~ParticlesToLevelSet() { delete mBlindGrid; }
@@ -174,7 +174,7 @@ public:
     /// and therefore needs to be called before any of these grids are
     /// used and after the last call to any of the rasterizer methods.
     ///
-    /// @note Avoid calling this method more then once and only after
+    /// @note Avoid calling this method more than once and only after
     /// all the particles have been rasterized. It has no effect or
     /// overhead if attribute transfer is disabled, i.e. AttributeT =
     /// void and prune is false.
@@ -184,7 +184,7 @@ public:
     /// (optional) attribute.
     ///
     /// @warning If attribute transfer was disabled, i.e. AttributeT =
-    /// void, or finalize() was not called the pointer is NULL!
+    /// void, or finalize() was not called the pointer is nullptr!
     typename AttGridType::Ptr attributeGrid() { return mAttGrid; }
 
     /// @brief Return the size of a voxel in world units
@@ -210,7 +210,7 @@ public:
     /// @brief set the largest radius allowed in voxel units
     void setRmax(Real Rmax) { mRmax = math::Max(mRmin,Rmax); }
 
-    /// @brief Rreturn the grain-size used for multi-threading
+    /// @brief Returns the grain-size used for multi-threading
     int  getGrainSize() const { return mGrainSize; }
     /// @brief Set the grain-size used for multi-threading.
     /// @note A grainsize of 0 or less disables multi-threading!
@@ -251,8 +251,8 @@ public:
     void rasterizeTrails(const ParticleListT& pa, Real delta=1.0);
 
 private:
-    typedef p2ls_internal::BlindData<SdfType, AttType> BlindType;
-    typedef typename SdfGridT::template ValueConverter<BlindType>::Type BlindGridType;
+    using BlindType = p2ls_internal::BlindData<SdfType, AttType>;
+    using BlindGridType = typename SdfGridT::template ValueConverter<BlindType>::Type;
 
     /// Class with multi-threaded implementation of particle rasterization
     template<typename ParticleListT, typename GridT> struct Raster;
@@ -272,7 +272,7 @@ template<typename SdfGridT, typename AttributeT, typename InterrupterT>
 inline ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::
 ParticlesToLevelSet(SdfGridT& grid, InterrupterT* interrupter) :
     mSdfGrid(&grid),
-    mBlindGrid(NULL),
+    mBlindGrid(nullptr),
     mInterrupter(interrupter),
     mDx(grid.voxelSize()[0]),
     mHalfWidth(grid.background()/mDx),
@@ -344,16 +344,16 @@ template<typename SdfGridT, typename AttributeT, typename InterrupterT>
 inline void
 ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::finalize(bool prune)
 {
-    if (mBlindGrid==NULL) {
+    if (mBlindGrid == nullptr) {
         if (prune) tools::pruneLevelSet(mSdfGrid->tree());
         return;
     } else {
         if (prune) tools::prune(mBlindGrid->tree());
     }
 
-    typedef typename SdfGridType::TreeType   SdfTreeT;
-    typedef typename AttGridType::TreeType   AttTreeT;
-    typedef typename BlindGridType::TreeType BlindTreeT;
+    using SdfTreeT = typename SdfGridType::TreeType;
+    using AttTreeT = typename AttGridType::TreeType;
+    using BlindTreeT = typename BlindGridType::TreeType;
     // Use topology copy constructors since output grids have the same topology as mBlindDataGrid
     const BlindTreeT& tree = mBlindGrid->tree();
 
@@ -370,14 +370,14 @@ ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::finalize(bool prune)
     // Extract the level set and IDs from mBlindDataGrid. We will
     // explore the fact that by design active values always live
     // at the leaf node level, i.e. level sets have no active tiles!
-    typedef typename BlindTreeT::LeafCIter    LeafIterT;
-    typedef typename BlindTreeT::LeafNodeType LeafT;
-    typedef typename SdfTreeT::LeafNodeType   SdfLeafT;
-    typedef typename AttTreeT::LeafNodeType   AttLeafT;
+    using LeafIterT = typename BlindTreeT::LeafCIter;
+    using LeafT = typename BlindTreeT::LeafNodeType;
+    using SdfLeafT = typename SdfTreeT::LeafNodeType;
+    using AttLeafT = typename AttTreeT::LeafNodeType;
     for (LeafIterT n = tree.cbeginLeaf(); n; ++n) {
         const LeafT& leaf = *n;
         const openvdb::Coord xyz = leaf.origin();
-        // Get leafnodes that were allocated during topology contruction!
+        // Get leafnodes that were allocated during topology construction!
         SdfLeafT* sdfLeaf = sdfTree->probeLeaf(xyz);
         AttLeafT* attLeaf = attTree->probeLeaf(xyz);
         // Use linear offset (vs coordinate) access for better performance!
@@ -397,7 +397,7 @@ ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::finalize(bool prune)
             }
         }
     }
-    
+
     tools::signedFloodFill(*sdfTree);//required since we only transferred active voxels!
 
     if (mSdfGrid->empty()) {
@@ -413,15 +413,15 @@ template<typename SdfGridT, typename AttributeT, typename InterrupterT>
 template<typename ParticleListT, typename GridT>
 struct ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::Raster
 {
-    typedef typename boost::is_void<AttributeT>::type DisableT;
-    typedef ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT> ParticlesToLevelSetT;
-    typedef typename ParticlesToLevelSetT::SdfType   SdfT;//type of signed distance values
-    typedef typename ParticlesToLevelSetT::AttType   AttT;//type of particle attribute
-    typedef typename GridT::ValueType                ValueT;
-    typedef typename GridT::Accessor                 AccessorT;
-    typedef typename GridT::TreeType                 TreeT;
-    typedef typename TreeT::LeafNodeType             LeafNodeT;
-    typedef PointPartitioner<Index32, LeafNodeT::LOG2DIM> PointPartitionerT;
+    using DisableT = typename std::is_void<AttributeT>::type;
+    using ParticlesToLevelSetT = ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>;
+    using SdfT = typename ParticlesToLevelSetT::SdfType; // type of signed distance values
+    using AttT = typename ParticlesToLevelSetT::AttType; // type of particle attribute
+    using ValueT = typename GridT::ValueType;
+    using AccessorT = typename GridT::Accessor;
+    using TreeT = typename GridT::TreeType;
+    using LeafNodeT = typename TreeT::LeafNodeType;
+    using PointPartitionerT = PointPartitioner<Index32, LeafNodeT::LOG2DIM>;
 
 
     /// @brief Main constructor
@@ -472,7 +472,7 @@ struct ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::Raster
         if (mParent.mInterrupter) {
             mParent.mInterrupter->start("Rasterizing particles to level set using spheres");
         }
-        mTask = boost::bind(&Raster::rasterSpheres, _1, _2);
+        mTask = std::bind(&Raster::rasterSpheres, std::placeholders::_1, std::placeholders::_2);
         this->cook();
         if (mParent.mInterrupter) mParent.mInterrupter->end();
     }
@@ -491,7 +491,8 @@ struct ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::Raster
                 mParent.mInterrupter->start(
                     "Rasterizing particles to level set using const spheres");
             }
-            mTask = boost::bind(&Raster::rasterFixedSpheres, _1, _2, SdfT(radius));
+            mTask = std::bind(&Raster::rasterFixedSpheres,
+                std::placeholders::_1, std::placeholders::_2, SdfT(radius));
             this->cook();
             if (mParent.mInterrupter) mParent.mInterrupter->end();
         }
@@ -516,7 +517,8 @@ struct ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::Raster
         if (mParent.mInterrupter) {
             mParent.mInterrupter->start("Rasterizing particles to level set using trails");
         }
-        mTask = boost::bind(&Raster::rasterTrails, _1, _2, SdfT(delta));
+        mTask = std::bind(&Raster::rasterTrails,
+            std::placeholders::_1, std::placeholders::_2, SdfT(delta));
         this->cook();
         if (mParent.mInterrupter) mParent.mInterrupter->end();
     }
@@ -539,7 +541,7 @@ struct ParticlesToLevelSet<SdfGridT, AttributeT, InterrupterT>::Raster
     }
 private:
     /// Disallow assignment since some of the members are references
-    Raster& operator=(const Raster& other) { return *this; }
+    Raster& operator=(const Raster&) { return *this; }
 
     /// @return true if the particle is too small or too large
     bool ignoreParticle(SdfT R)
@@ -706,9 +708,9 @@ private:
     /// @note For best performance all computations are performed in
     /// voxel-space with the important exception of the final level set
     /// value that is converted to world units (e.g. the grid stores
-    /// the closest Euclidian signed distances measured in world
+    /// the closest Euclidean signed distances measured in world
     /// units). Also note we use the convention of positive distances
-    /// outside the surface an negative distances inside the surface.
+    /// outside the surface and negative distances inside the surface.
     bool makeSphere(const Vec3R &P, SdfT R, const AttT& att, AccessorT& acc)
     {
         const ValueT inside = -mGrid->background();
@@ -746,22 +748,22 @@ private:
         }//end loop over x
         return true;
     }
-    typedef typename boost::function<void (Raster*, const tbb::blocked_range<size_t>&)> FuncType;
+    using FuncType = typename std::function<void (Raster*, const tbb::blocked_range<size_t>&)>;
 
-    template <typename DisableType>
-    typename boost::enable_if<DisableType>::type
-    getAtt(size_t, AttT&) const {;}
+    template<typename DisableType>
+    typename std::enable_if<DisableType::value>::type
+    getAtt(size_t, AttT&) const {}
 
-    template <typename DisableType>
-    typename boost::disable_if<DisableType>::type
+    template<typename DisableType>
+    typename std::enable_if<!DisableType::value>::type
     getAtt(size_t n, AttT& a) const { mParticles.getAtt(n, a); }
 
-    template <typename T>
-    typename boost::enable_if<boost::is_same<T,ValueT>, ValueT>::type
+    template<typename T>
+    typename std::enable_if<std::is_same<T, ValueT>::value, ValueT>::type
     Merge(T s, const AttT&) const { return s; }
 
-    template <typename T>
-    typename boost::disable_if<boost::is_same<T,ValueT>, ValueT>::type
+    template<typename T>
+    typename std::enable_if<!std::is_same<T, ValueT>::value, ValueT>::type
     Merge(T s, const AttT& a) const { return ValueT(s,a); }
 
     ParticlesToLevelSetT& mParent;
@@ -786,27 +788,25 @@ template<typename VisibleT, typename BlindT>
 class BlindData
 {
 public:
-    typedef VisibleT type;
-    typedef VisibleT VisibleType;
-    typedef BlindT   BlindType;
+    using type = VisibleT;
+    using VisibleType = VisibleT;
+    using BlindType = BlindT;
 
     BlindData() {}
     explicit BlindData(VisibleT v) : mVisible(v), mBlind(zeroVal<BlindType>()) {}
     BlindData(VisibleT v, BlindT b) : mVisible(v), mBlind(b) {}
-    BlindData& operator=(const BlindData& rhs)
-    {
-        mVisible = rhs.mVisible;
-        mBlind = rhs.mBlind;
-        return *this;
-    }
+    BlindData(const BlindData&) = default;
+    BlindData& operator=(const BlindData&) = default;
     const VisibleT& visible() const { return mVisible; }
     const BlindT&   blind()   const { return mBlind; }
+    OPENVDB_NO_FP_EQUALITY_WARNING_BEGIN
     bool operator==(const BlindData& rhs)     const { return mVisible == rhs.mVisible; }
-    bool operator< (const BlindData& rhs)     const { return mVisible <  rhs.mVisible; };
-    bool operator> (const BlindData& rhs)     const { return mVisible >  rhs.mVisible; };
-    BlindData operator+(const BlindData& rhs) const { return BlindData(mVisible + rhs.mVisible); };
-    BlindData operator+(const VisibleT&  rhs) const { return BlindData(mVisible + rhs); };
-    BlindData operator-(const BlindData& rhs) const { return BlindData(mVisible - rhs.mVisible); };
+    OPENVDB_NO_FP_EQUALITY_WARNING_END
+    bool operator< (const BlindData& rhs)     const { return mVisible <  rhs.mVisible; }
+    bool operator> (const BlindData& rhs)     const { return mVisible >  rhs.mVisible; }
+    BlindData operator+(const BlindData& rhs) const { return BlindData(mVisible + rhs.mVisible); }
+    BlindData operator+(const VisibleT&  rhs) const { return BlindData(mVisible + rhs); }
+    BlindData operator-(const BlindData& rhs) const { return BlindData(mVisible - rhs.mVisible); }
     BlindData operator-() const { return BlindData(-mVisible, mBlind); }
 
 protected:
@@ -839,6 +839,6 @@ inline BlindData<VisibleT, BlindT> Abs(const BlindData<VisibleT, BlindT>& x)
 
 #endif // OPENVDB_TOOLS_PARTICLES_TO_LEVELSET_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
