@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -30,16 +30,16 @@
 //
 /// @author Ken Museth
 ///
-/// @file PointScatter.h
+/// @file tools/PointScatter.h
 ///
 /// @brief We offer three different algorithms (each in its own class)
-///        for scattering of point in active voxels:
+///        for scattering of points in active voxels:
 ///
 /// 1) UniformPointScatter. Has two modes: Either randomly distributes
-///    a fixed number of points in the active voxels, or the user can
+///    a fixed number of points into the active voxels, or the user can
 ///    specify a fixed probability of having a points per unit of volume.
 ///
-/// 2) DenseUniformPointScatter. Randomly distributes points in active
+/// 2) DenseUniformPointScatter. Randomly distributes points into active
 ///    voxels using a fixed number of points per voxel.
 ///
 /// 3) NonIniformPointScatter. Define the local probability of having
@@ -55,7 +55,9 @@
 #include <openvdb/util/NullInterrupter.h>
 #include <tbb/parallel_sort.h>
 #include <tbb/parallel_for.h>
-#include <boost/scoped_array.hpp>
+#include <iostream>
+#include <memory>
+#include <string>
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -88,9 +90,9 @@ class BasePointScatter;
 /// class Interrupter {
 ///   ...
 /// public:
-///   void start(const char* name = NULL)// called when computations begin
-///   void end()                         // called when computations end
-///   bool wasInterrupted(int percent=-1)// return true to break computation
+///   void start(const char* name = nullptr) // called when computations begin
+///   void end()                             // called when computations end
+///   bool wasInterrupted(int percent=-1)    // return true to break computation
 ///};
 /// @endcode
 ///
@@ -99,7 +101,7 @@ class BasePointScatter;
 /// interrupter calls are no-ops (i.e. incurs no computational overhead).
 
 
-/// @brief Uniform scatters of point in the active voxels.
+/// @brief Uniformly scatters points in the active voxels.
 /// The point count is either explicitly defined or implicitly
 /// through the specification of a global density (=points-per-volume)
 ///
@@ -114,13 +116,14 @@ class UniformPointScatter : public BasePointScatter<PointAccessorType,
                                                     InterruptType>
 {
 public:
-    typedef BasePointScatter<PointAccessorType, RandomGenerator, InterruptType> BaseT;
+    using BaseT = BasePointScatter<PointAccessorType, RandomGenerator, InterruptType>;
 
     UniformPointScatter(PointAccessorType& points,
                         Index64 pointCount,
                         RandomGenerator& randGen,
-                        InterruptType* interrupt = NULL)
-        : BaseT(points, randGen, interrupt)
+                        double spread = 1.0,
+                        InterruptType* interrupt = nullptr)
+        : BaseT(points, randGen, spread, interrupt)
         , mTargetPointCount(pointCount)
         , mPointsPerVolume(0.0f)
     {
@@ -128,8 +131,9 @@ public:
     UniformPointScatter(PointAccessorType& points,
                         float pointsPerVolume,
                         RandomGenerator& randGen,
-                        InterruptType* interrupt = NULL)
-        : BaseT(points, randGen, interrupt)
+                        double spread = 1.0,
+                        InterruptType* interrupt = nullptr)
+        : BaseT(points, randGen, spread, interrupt)
         , mTargetPointCount(0)
         , mPointsPerVolume(pointsPerVolume)
     {
@@ -153,17 +157,18 @@ public:
             return false;
         }
 
-        boost::scoped_array<Index64> list(new Index64[mTargetPointCount]);
+        std::unique_ptr<Index64[]> idList{new Index64[mTargetPointCount]};
         math::RandInt<Index64, RandomGenerator> rand(BaseT::mRand01.engine(), 0, mVoxelCount-1);
-        for (Index64 i=0; i<mTargetPointCount; ++i) list[i] = rand();
-        tbb::parallel_sort(list.get(), list.get() + mTargetPointCount);
+        for (Index64 i=0; i<mTargetPointCount; ++i) idList[i] = rand();
+        tbb::parallel_sort(idList.get(), idList.get() + mTargetPointCount);
 
         CoordBBox bbox;
         const Vec3R offset(0.5, 0.5, 0.5);
         typename GridT::ValueOnCIter valueIter = grid.cbeginValueOn();
+
         for (Index64 i=0, n=valueIter.getVoxelCount() ; i != mTargetPointCount; ++i) {
             if (BaseT::interrupt()) return false;
-            const Index64 voxelId = list[i];
+            const Index64 voxelId = idList[i];
             while ( n <= voxelId ) {
                 ++valueIter;
                 n += valueIter.getVoxelCount();
@@ -175,6 +180,7 @@ public:
                 BaseT::addPoint(grid, bbox.min() - offset, bbox.extents());
             }
         }//loop over all the active voxels and tiles
+        //}
 
         BaseT::end();
         return true;
@@ -184,7 +190,7 @@ public:
     // the operator() method was called
     void print(const std::string &name, std::ostream& os = std::cout) const
     {
-        os << "Uniformely scattered " << mPointCount << " points into " << mVoxelCount
+        os << "Uniformly scattered " << mPointCount << " points into " << mVoxelCount
            << " active voxels in \"" << name << "\" corresponding to "
            << mPointsPerVolume << " points per volume." << std::endl;
     }
@@ -211,13 +217,14 @@ class DenseUniformPointScatter : public BasePointScatter<PointAccessorType,
                                                          InterruptType>
 {
 public:
-    typedef BasePointScatter<PointAccessorType, RandomGenerator, InterruptType> BaseT;
+    using BaseT = BasePointScatter<PointAccessorType, RandomGenerator, InterruptType>;
 
     DenseUniformPointScatter(PointAccessorType& points,
                              float pointsPerVoxel,
                              RandomGenerator& randGen,
-                             InterruptType* interrupt = NULL)
-        : BaseT(points, randGen, interrupt)
+                             double spread = 1.0,
+                             InterruptType* interrupt = nullptr)
+        : BaseT(points, randGen, spread, interrupt)
         , mPointsPerVoxel(pointsPerVoxel)
     {
     }
@@ -226,7 +233,7 @@ public:
     template<typename GridT>
     bool operator()(const GridT& grid)
     {
-        typedef typename GridT::ValueOnCIter ValueIter;
+        using ValueIter = typename GridT::ValueOnCIter;
         if (mPointsPerVoxel < 1.0e-6) return false;
         mVoxelCount = grid.activeVoxelCount();
         if (mVoxelCount == 0) return false;
@@ -243,7 +250,7 @@ public:
             if (iter.isVoxelValue()) {// a majority is expected to be voxels
                 const Vec3R dmin = iter.getCoord() - offset;
                 for (int n = 0; n != ppv; ++n) BaseT::addPoint(grid, dmin);
-                if (fractional && BaseT::getRand() < delta) BaseT::addPoint(grid, dmin);
+                if (fractional && BaseT::getRand01() < delta) BaseT::addPoint(grid, dmin);
             } else {// tiles contain multiple (virtual) voxels
                 iter.getBoundingBox(bbox);
                 const Coord size(bbox.extents());
@@ -251,10 +258,10 @@ public:
                 const double d = mPointsPerVoxel * iter.getVoxelCount();
                 const int m = math::Floor(d);
                 for (int n = 0; n != m; ++n)  BaseT::addPoint(grid, dmin, size);
-                if (BaseT::getRand() < d - m) BaseT::addPoint(grid, dmin, size);
+                if (BaseT::getRand01() < d - m) BaseT::addPoint(grid, dmin, size);
             }
         }//loop over all the active voxels and tiles
-
+        //}
         BaseT::end();
         return true;
     }
@@ -292,13 +299,14 @@ class NonUniformPointScatter : public BasePointScatter<PointAccessorType,
                                                        InterruptType>
 {
 public:
-    typedef BasePointScatter<PointAccessorType, RandomGenerator, InterruptType> BaseT;
+    using BaseT = BasePointScatter<PointAccessorType, RandomGenerator, InterruptType>;
 
     NonUniformPointScatter(PointAccessorType& points,
                            float pointsPerVolume,
                            RandomGenerator& randGen,
-                           InterruptType* interrupt = NULL)
-        : BaseT(points, randGen, interrupt)
+                           double spread = 1.0,
+                           InterruptType* interrupt = nullptr)
+        : BaseT(points, randGen, spread, interrupt)
         , mPointsPerVolume(pointsPerVolume)//note this is merely a
                                            //multiplier for the local point density
     {
@@ -324,13 +332,13 @@ public:
             if (iter.isVoxelValue()) { // a majority is expected to be voxels
                 const Vec3R dmin =iter.getCoord() - offset;
                 for (int i = 0; i < n; ++i) BaseT::addPoint(grid, dmin);
-                if (BaseT::getRand() < (d - n)) BaseT::addPoint(grid, dmin);
+                if (BaseT::getRand01() < (d - n)) BaseT::addPoint(grid, dmin);
             } else { // tiles contain multiple (virtual) voxels
                 iter.getBoundingBox(bbox);
                 const Coord size(bbox.extents());
                 const Vec3R dmin = bbox.min() - offset;
                 for (int i = 0; i < n; ++i) BaseT::addPoint(grid, dmin, size);
-                if (BaseT::getRand() < (d - n)) BaseT::addPoint(grid, dmin, size);
+                if (BaseT::getRand01() < (d - n)) BaseT::addPoint(grid, dmin, size);
             }
         }//loop over all the active voxels and tiles
         BaseT::end();
@@ -367,26 +375,29 @@ public:
 
 protected:
 
-    /// This is a base class so the constructor is protected
-    BasePointScatter(PointAccessorType& points,
-                     RandomGenerator& randGen,
-                     InterruptType* interrupt = NULL)
-        : mPoints(points)
-        , mInterrupter(interrupt)
-        , mPointCount(0)
-        , mVoxelCount(0)
-        , mInterruptCount(0)
-        , mRand01(randGen)
-    {
-    }
-
     PointAccessorType&        mPoints;
     InterruptType*            mInterrupter;
     Index64                   mPointCount;
     Index64                   mVoxelCount;
     Index64                   mInterruptCount;
+    const double              mSpread;
     math::Rand01<double, RandomGenerator> mRand01;
 
+    /// This is a base class so the constructor is protected
+    BasePointScatter(PointAccessorType& points,
+                     RandomGenerator& randGen,
+                     double spread,
+                     InterruptType* interrupt = nullptr)
+        : mPoints(points)
+        , mInterrupter(interrupt)
+        , mPointCount(0)
+        , mVoxelCount(0)
+        , mInterruptCount(0)
+        , mSpread(math::Clamp01(spread))
+        , mRand01(randGen)
+    {
+    }
+
     inline void start(const char* name)
     {
         if (mInterrupter) mInterrupter->start(name);
@@ -403,7 +414,11 @@ protected:
         return !(mInterruptCount++ & ((1<<5)-1)) && util::wasInterrupted(mInterrupter);
     }
 
-    inline double getRand() { return mRand01(); }
+    /// @brief Return a random floating point number between zero and one
+    inline double getRand01() { return mRand01(); }
+
+    /// @brief Return a random floating point number between 0.5 -+ mSpread/2
+    inline double getRand() { return 0.5 + mSpread * (mRand01() - 0.5); }
 
     template <typename GridT>
     inline void addPoint(const GridT &grid, const Vec3R &dmin)
@@ -432,6 +447,6 @@ protected:
 
 #endif // OPENVDB_TOOLS_POINT_SCATTER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
