@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -27,7 +27,7 @@
 // LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
 //
 ///////////////////////////////////////////////////////////////////////////
-//
+
 /// @file LeafManager.h
 ///
 /// @brief A LeafManager manages a linear array of pointers to a given tree's
@@ -42,18 +42,14 @@
 #ifndef OPENVDB_TREE_LEAFMANAGER_HAS_BEEN_INCLUDED
 #define OPENVDB_TREE_LEAFMANAGER_HAS_BEEN_INCLUDED
 
-#include <boost/shared_ptr.hpp>
-#include <boost/bind.hpp>
-#include <boost/function.hpp>
-#include <boost/mpl/if.hpp>
-#include <boost/type_traits/is_const.hpp>
-#include <boost/type_traits/is_pointer.hpp>
-#include <boost/type_traits/is_same.hpp>
-#include <boost/type_traits/remove_pointer.hpp>
-#include <tbb/blocked_range.h>
-#include <tbb/parallel_for.h>
 #include <openvdb/Types.h>
 #include "TreeIterator.h" // for CopyConstness
+#include <tbb/blocked_range.h>
+#include <tbb/parallel_for.h>
+#include <tbb/parallel_reduce.h>
+#include <functional>
+#include <type_traits>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -66,11 +62,11 @@ namespace leafmgr {
 /// Useful traits for Tree types
 template<typename TreeT> struct TreeTraits {
     static const bool IsConstTree = false;
-    typedef typename TreeT::LeafIter LeafIterType;
+    using LeafIterType = typename TreeT::LeafIter;
 };
 template<typename TreeT> struct TreeTraits<const TreeT> {
     static const bool IsConstTree = true;
-    typedef typename TreeT::LeafCIter LeafIterType;
+    using LeafIterType = typename TreeT::LeafCIter;
 };
 //@}
 
@@ -82,12 +78,12 @@ template<typename TreeT> struct TreeTraits<const TreeT> {
 template<typename ManagerT>
 struct LeafManagerImpl
 {
-    typedef typename ManagerT::RangeType  RangeT;
-    typedef typename ManagerT::LeafType   LeafT;
-    typedef typename ManagerT::BufferType BufT;
+    using RangeT = typename ManagerT::RangeType;
+    using LeafT = typename ManagerT::LeafType;
+    using BufT = typename ManagerT::BufferType;
 
     static inline void doSwapLeafBuffer(const RangeT& r, size_t auxBufferIdx,
-        LeafT** leafs, BufT* bufs, size_t bufsPerLeaf)
+                                        LeafT** leafs, BufT* bufs, size_t bufsPerLeaf)
     {
         for (size_t n = r.begin(), m = r.end(), N = bufsPerLeaf; n != m; ++n) {
             leafs[n]->swap(bufs[n * N + auxBufferIdx]);
@@ -114,17 +110,17 @@ template<typename TreeT>
 class LeafManager
 {
 public:
-    typedef TreeT                                                      TreeType;
-    typedef typename TreeT::ValueType                                  ValueType;
-    typedef typename TreeT::RootNodeType                               RootNodeType;
-    typedef typename TreeType::LeafNodeType                            NonConstLeafType;
-    typedef typename CopyConstness<TreeType, NonConstLeafType>::Type   LeafType;
-    typedef LeafType                                                   LeafNodeType;
-    typedef typename leafmgr::TreeTraits<TreeT>::LeafIterType          LeafIterType;
-    typedef typename LeafType::Buffer                                  NonConstBufferType;
-    typedef typename CopyConstness<TreeType, NonConstBufferType>::Type BufferType;
-    typedef tbb::blocked_range<size_t>                                 RangeType;//leaf index range
-    static const Index DEPTH = 2;//root + leafs
+    using TreeType = TreeT;
+    using ValueType = typename TreeT::ValueType;
+    using RootNodeType = typename TreeT::RootNodeType;
+    using NonConstLeafType = typename TreeType::LeafNodeType;
+    using LeafType = typename CopyConstness<TreeType, NonConstLeafType>::Type;
+    using LeafNodeType = LeafType;
+    using LeafIterType = typename leafmgr::TreeTraits<TreeT>::LeafIterType;
+    using NonConstBufferType = typename LeafType::Buffer;
+    using BufferType = typename CopyConstness<TreeType, NonConstBufferType>::Type;
+    using RangeType = tbb::blocked_range<size_t>; // leaf index range
+    static const Index DEPTH = 2; // root + leaf nodes
 
     static const bool IsConstTree = leafmgr::TreeTraits<TreeT>::IsConstTree;
 
@@ -138,10 +134,8 @@ public:
             {
                 assert(this->isValid());
             }
-            Iterator& operator=(const Iterator& other)
-            {
-                mRange = other.mRange; mPos = other.mPos; return *this;
-            }
+            Iterator(const Iterator&) = default;
+            Iterator& operator=(const Iterator&) = default;
             /// Advance to the next leaf node.
             Iterator& operator++() { ++mPos; return *this; }
             /// Return a reference to the leaf node to which this iterator is pointing.
@@ -156,6 +150,7 @@ public:
             }
             /// Return the index into the leaf array of the current leaf node.
             size_t pos() const { return mPos; }
+            /// Return @c true if the position of this iterator is in a valid range.
             bool isValid() const { return mPos>=mRange.mBegin && mPos<=mRange.mEnd; }
             /// Return @c true if this iterator is not yet exhausted.
             bool test() const { return mPos < mRange.mEnd; }
@@ -175,8 +170,13 @@ public:
             size_t mPos;
         };// end Iterator
 
-        LeafRange(size_t begin, size_t end, const LeafManager& leafManager, size_t grainSize=1):
-            mEnd(end), mBegin(begin), mGrainSize(grainSize), mLeafManager(leafManager) {}
+        LeafRange(size_t begin, size_t end, const LeafManager& leafManager, size_t grainSize=1)
+            : mEnd(end)
+            , mBegin(begin)
+            , mGrainSize(grainSize)
+            , mLeafManager(leafManager)
+        {
+        }
 
         Iterator begin() const {return Iterator(*this, mBegin);}
 
@@ -192,9 +192,13 @@ public:
 
         bool is_divisible() const {return mGrainSize < this->size();}
 
-        LeafRange(LeafRange& r, tbb::split):
-            mEnd(r.mEnd), mBegin(doSplit(r)), mGrainSize(r.mGrainSize),
-              mLeafManager(r.mLeafManager) {}
+        LeafRange(LeafRange& r, tbb::split)
+            : mEnd(r.mEnd)
+            , mBegin(doSplit(r))
+            , mGrainSize(r.mGrainSize)
+            , mLeafManager(r.mLeafManager)
+        {
+        }
 
     private:
         size_t mEnd, mBegin, mGrainSize;
@@ -210,32 +214,52 @@ public:
     };// end of LeafRange
 
     /// @brief Constructor from a tree reference and an auxiliary buffer count
-    /// (default is no auxiliary buffers)
-    LeafManager(TreeType& tree, size_t auxBuffersPerLeaf=0, bool serial=false):
-        mTree(&tree),
-        mLeafCount(0),
-        mAuxBufferCount(0),
-        mAuxBuffersPerLeaf(auxBuffersPerLeaf),
-        mLeafs(NULL),
-        mAuxBuffers(NULL),
-        mTask(0),
-        mIsMaster(true)
+    /// @note  The default is no auxiliary buffers
+    LeafManager(TreeType& tree, size_t auxBuffersPerLeaf=0, bool serial=false)
+        : mTree(&tree)
+        , mLeafCount(0)
+        , mAuxBufferCount(0)
+        , mAuxBuffersPerLeaf(auxBuffersPerLeaf)
+        , mLeafs(nullptr)
+        , mAuxBuffers(nullptr)
+        , mTask(0)
+        , mIsMaster(true)
     {
         this->rebuild(serial);
     }
 
+    /// @brief Construct directly from an existing array of leafnodes.
+    /// @warning The leafnodes are implicitly assumed to exist in the
+    ///          input @a tree.
+    LeafManager(TreeType& tree, LeafType** begin, LeafType** end,
+                size_t auxBuffersPerLeaf=0, bool serial=false)
+        : mTree(&tree)
+        , mLeafCount(end-begin)
+        , mAuxBufferCount(0)
+        , mAuxBuffersPerLeaf(auxBuffersPerLeaf)
+        , mLeafs(new LeafType*[mLeafCount])
+        , mAuxBuffers(nullptr)
+        , mTask(0)
+        , mIsMaster(true)
+    {
+        size_t n = mLeafCount;
+        LeafType **target = mLeafs, **source = begin;
+        while (n--) *target++ = *source++;
+        if (auxBuffersPerLeaf) this->initAuxBuffers(serial);
+    }
+
     /// Shallow copy constructor called by tbb::parallel_for() threads
     ///
     /// @note This should never get called directly
-    LeafManager(const LeafManager& other):
-        mTree(other.mTree),
-        mLeafCount(other.mLeafCount),
-        mAuxBufferCount(other.mAuxBufferCount),
-        mAuxBuffersPerLeaf(other.mAuxBuffersPerLeaf),
-        mLeafs(other.mLeafs),
-        mAuxBuffers(other.mAuxBuffers),
-        mTask(other.mTask),
-        mIsMaster(false)
+    LeafManager(const LeafManager& other)
+        : mTree(other.mTree)
+        , mLeafCount(other.mLeafCount)
+        , mAuxBufferCount(other.mAuxBufferCount)
+        , mAuxBuffersPerLeaf(other.mAuxBuffersPerLeaf)
+        , mLeafs(other.mLeafs)
+        , mAuxBuffers(other.mAuxBuffers)
+        , mTask(other.mTask)
+        , mIsMaster(false)
     {
     }
 
@@ -295,14 +319,26 @@ public:
         this->initLeafArray();
     }
 
-    /// Return the total number of allocated auxiliary buffers.
+    /// @brief Return the total number of allocated auxiliary buffers.
     size_t auxBufferCount() const { return mAuxBufferCount; }
-    /// Return the number of auxiliary buffers per leaf node.
+    /// @brief Return the number of auxiliary buffers per leaf node.
     size_t auxBuffersPerLeaf() const { return mAuxBuffersPerLeaf; }
 
-    /// Return the number of leaf nodes.
+    /// @brief Return the number of leaf nodes.
     size_t leafCount() const { return mLeafCount; }
 
+    /// @brief Return the number of active voxels in the leaf nodes.
+    /// @note Multi-threaded for better performance than Tree::activeLeafVoxelCount
+    Index64 activeLeafVoxelCount() const
+    {
+        return tbb::parallel_reduce(this->leafRange(), Index64(0),
+            [] (const LeafRange& range, Index64 sum) -> Index64 {
+                for (const auto& leaf: range) { sum += leaf.onVoxelCount(); }
+                return sum;
+            },
+            [] (Index64 n, Index64 m) -> Index64 { return n + m; });
+    }
+
     /// Return a const reference to tree associated with this manager.
     const TreeType& tree() const { return *mTree; }
 
@@ -337,7 +373,7 @@ public:
         assert(leafIdx < mLeafCount);
         assert(bufferIdx == 0 || bufferIdx - 1 < mAuxBuffersPerLeaf);
         return bufferIdx == 0 ? mLeafs[leafIdx]->buffer()
-            : mAuxBuffers[leafIdx * mAuxBuffersPerLeaf + bufferIdx - 1];
+             : mAuxBuffers[leafIdx * mAuxBuffersPerLeaf + bufferIdx - 1];
     }
 
     /// @brief Return a @c tbb::blocked_range of leaf array indices.
@@ -363,8 +399,9 @@ public:
     /// the first auxiliary buffer.
     bool swapLeafBuffer(size_t bufferIdx, bool serial = false)
     {
+        namespace ph = std::placeholders;
         if (bufferIdx == 0 || bufferIdx > mAuxBuffersPerLeaf || this->isConstTree()) return false;
-        mTask = boost::bind(&LeafManager::doSwapLeafBuffer, _1, _2, bufferIdx - 1);
+        mTask = std::bind(&LeafManager::doSwapLeafBuffer, ph::_1, ph::_2, bufferIdx - 1);
         this->cook(serial ? 0 : 512);
         return true;//success
     }
@@ -374,14 +411,15 @@ public:
     /// the first auxiliary buffer.
     bool swapBuffer(size_t bufferIdx1, size_t bufferIdx2, bool serial = false)
     {
+        namespace ph = std::placeholders;
         const size_t b1 = std::min(bufferIdx1, bufferIdx2);
         const size_t b2 = std::max(bufferIdx1, bufferIdx2);
         if (b1 == b2 || b2 > mAuxBuffersPerLeaf) return false;
         if (b1 == 0) {
             if (this->isConstTree()) return false;
-            mTask = boost::bind(&LeafManager::doSwapLeafBuffer, _1, _2, b2-1);
+            mTask = std::bind(&LeafManager::doSwapLeafBuffer, ph::_1, ph::_2, b2-1);
         } else {
-            mTask = boost::bind(&LeafManager::doSwapAuxBuffer, _1, _2, b1-1, b2-1);
+            mTask = std::bind(&LeafManager::doSwapAuxBuffer, ph::_1, ph::_2, b1-1, b2-1);
         }
         this->cook(serial ? 0 : 512);
         return true;//success
@@ -397,8 +435,9 @@ public:
     /// the first auxiliary buffer.
     bool syncAuxBuffer(size_t bufferIdx, bool serial = false)
     {
+        namespace ph = std::placeholders;
         if (bufferIdx == 0 || bufferIdx > mAuxBuffersPerLeaf) return false;
-        mTask = boost::bind(&LeafManager::doSyncAuxBuffer, _1, _2, bufferIdx - 1);
+        mTask = std::bind(&LeafManager::doSyncAuxBuffer, ph::_1, ph::_2, bufferIdx - 1);
         this->cook(serial ? 0 : 64);
         return true;//success
     }
@@ -408,18 +447,22 @@ public:
     /// @param serial  if false, sync buffers in parallel using multiple threads.
     bool syncAllBuffers(bool serial = false)
     {
+        namespace ph = std::placeholders;
         switch (mAuxBuffersPerLeaf) {
             case 0: return false;//nothing to do
-            case 1: mTask = boost::bind(&LeafManager::doSyncAllBuffers1, _1, _2); break;
-            case 2: mTask = boost::bind(&LeafManager::doSyncAllBuffers2, _1, _2); break;
-            default: mTask = boost::bind(&LeafManager::doSyncAllBuffersN, _1, _2); break;
+            case 1: mTask = std::bind(&LeafManager::doSyncAllBuffers1, ph::_1, ph::_2); break;
+            case 2: mTask = std::bind(&LeafManager::doSyncAllBuffers2, ph::_1, ph::_2); break;
+            default: mTask = std::bind(&LeafManager::doSyncAllBuffersN, ph::_1, ph::_2); break;
         }
         this->cook(serial ? 0 : 64);
         return true;//success
     }
 
     /// @brief   Threaded method that applies a user-supplied functor
-    ///          to each leaf node in the LeafManager
+    ///          to each leaf node in the LeafManager.
+    ///
+    /// @details The user-supplied functor needs to define the methods
+    ///          required for tbb::parallel_for.
     ///
     /// @param op        user-supplied functor, see examples for interface details.
     /// @param threaded  optional toggle to disable threading, on by default.
@@ -427,6 +470,8 @@ public:
     ///                  for threading, one by default.
     ///
     /// @warning The functor object is deep-copied to create TBB tasks.
+    ///          This allows the function to use non-thread-safe members
+    ///          like a ValueAccessor.
     ///
     /// @par Example:
     /// @code
@@ -434,14 +479,14 @@ public:
     /// template<typename TreeType>
     /// struct OffsetOp
     /// {
-    ///     typedef tree::ValueAccessor<const TreeType> Accessor;
+    ///     using Accessor = tree::ValueAccessor<const TreeType>;
     ///
     ///     OffsetOp(const TreeType& tree): mRhsTreeAcc(tree) {}
     ///
     ///     template <typename LeafNodeType>
     ///     void operator()(LeafNodeType &lhsLeaf, size_t) const
     ///     {
-    ///         const LeafNodeType * rhsLeaf = mRhsTreeAcc.probeConstLeaf(lhsLeaf.origin());
+    ///         const LeafNodeType *rhsLeaf = mRhsTreeAcc.probeConstLeaf(lhsLeaf.origin());
     ///         if (rhsLeaf) {
     ///             typename LeafNodeType::ValueOnIter iter = lhsLeaf.beginValueOn();
     ///             for (; iter; ++iter) {
@@ -449,7 +494,6 @@ public:
     ///             }
     ///         }
     ///     }
-    /// private:
     ///     Accessor mRhsTreeAcc;
     /// };
     ///
@@ -461,7 +505,7 @@ public:
     /// template<typename LeafManagerType>
     /// struct MinOp
     /// {
-    ///     typedef typename LeafManagerType::BufferType BufferType;
+    ///     using BufferType = typename LeafManagerType::BufferType;
     ///
     ///     MinOp(LeafManagerType& leafNodes): mLeafs(leafNodes) {}
     ///
@@ -473,7 +517,6 @@ public:
     ///
     ///         // min ...
     ///     }
-    /// private:
     ///     LeafManagerType& mLeafs;
     /// };
     /// @endcode
@@ -484,34 +527,92 @@ public:
         transform.run(this->leafRange(grainSize), threaded);
     }
 
+    /// @brief   Threaded method that applies a user-supplied functor
+    ///          to each leaf node in the LeafManager. Unlike foreach
+    ///          (defined above) this method performs a reduction on
+    ///          all the leaf nodes.
+    ///
+    /// @details The user-supplied functor needs to define the methods
+    ///          required for tbb::parallel_reduce.
+    ///
+    /// @param op        user-supplied functor, see examples for interface details.
+    /// @param threaded  optional toggle to disable threading, on by default.
+    /// @param grainSize optional parameter to specify the grainsize
+    ///                  for threading, one by default.
+    ///
+    /// @warning The functor object is deep-copied to create TBB tasks.
+    ///          This allows the function to use non-thread-safe members
+    ///          like a ValueAccessor.
+    ///
+    /// @par Example:
+    /// @code
+    /// // Functor to count the number of negative (active) leaf values
+    /// struct CountOp
+    /// {
+    ///     CountOp() : mCounter(0) {}
+    ///     CountOp(const CountOp &other) : mCounter(other.mCounter) {}
+    ///     CountOp(const CountOp &other, tbb::split) : mCounter(0) {}
+    ///     template <typename LeafNodeType>
+    ///     void operator()(LeafNodeType &leaf, size_t)
+    ///     {
+    ///       typename LeafNodeType::ValueOnIter iter = leaf.beginValueOn();
+    ///       for (; iter; ++iter) if (*iter < 0.0f) ++mCounter;
+    ///     }
+    ///     void join(const CountOp &other) {mCounter += other.mCounter;}
+    ///     size_t mCounter;
+    /// };
+    ///
+    /// // usage:
+    /// tree::LeafManager<FloatTree> leafNodes(tree);
+    /// MinValueOp min;
+    /// leafNodes.reduce(min);
+    /// std::cerr << "Number of negative active voxels = " << min.mCounter << std::endl;
+    ///
+    /// @endcode
+    template<typename LeafOp>
+    void reduce(LeafOp& op, bool threaded = true, size_t grainSize=1)
+    {
+        LeafReducer<LeafOp> transform(op);
+        transform.run(this->leafRange(grainSize), threaded);
+    }
 
+
+    /// @brief Insert pointers to nodes of the specified type into the array.
+    /// @details The type of node pointer is defined by the type
+    /// ArrayT::value_type. If the node type is a LeafNode the nodes
+    /// are inserted from this LeafManager, else of the corresponding tree.
     template<typename ArrayT>
     void getNodes(ArrayT& array)
     {
-        typedef typename ArrayT::value_type T;
-        BOOST_STATIC_ASSERT(boost::is_pointer<T>::value);
-        typedef typename boost::mpl::if_<boost::is_const<typename boost::remove_pointer<T>::type>,
-            const LeafType, LeafType>::type LeafT;
+        using T = typename ArrayT::value_type;
+        static_assert(std::is_pointer<T>::value, "argument to getNodes() must be a pointer array");
+        using LeafT = typename std::conditional<std::is_const<
+            typename std::remove_pointer<T>::type>::value, const LeafType, LeafType>::type;
 
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_BEGIN
-        if (boost::is_same<T, LeafT*>::value) {
+        if (std::is_same<T, LeafT*>::value) {
             array.resize(mLeafCount);
             for (size_t i=0; i<mLeafCount; ++i) array[i] = reinterpret_cast<T>(mLeafs[i]);
         } else {
             mTree->getNodes(array);
         }
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_END
-            }
+    }
 
+    /// @brief Insert node pointers of the specified type into the array.
+    /// @details The type of node pointer is defined by the type
+    /// ArrayT::value_type. If the node type is a LeafNode the nodes
+    /// are inserted from this LeafManager, else of the corresponding tree.
     template<typename ArrayT>
     void getNodes(ArrayT& array) const
     {
-        typedef typename ArrayT::value_type T;
-        BOOST_STATIC_ASSERT(boost::is_pointer<T>::value);
-        BOOST_STATIC_ASSERT(boost::is_const<typename boost::remove_pointer<T>::type>::value);
+        using T = typename ArrayT::value_type;
+        static_assert(std::is_pointer<T>::value, "argument to getNodes() must be a pointer array");
+        static_assert(std::is_const<typename std::remove_pointer<T>::type>::value,
+            "argument to getNodes() must be an array of const node pointers");
 
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_BEGIN
-        if (boost::is_same<T, const LeafType*>::value) {
+        if (std::is_same<T, const LeafType*>::value) {
             array.resize(mLeafCount);
             for (size_t i=0; i<mLeafCount; ++i) array[i] = reinterpret_cast<T>(mLeafs[i]);
         } else {
@@ -520,6 +621,36 @@ public:
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_END
     }
 
+    /// @brief Generate a linear array of prefix sums of offsets into the
+    /// active voxels in the leafs. So @a offsets[n]+m is the offset to the
+    /// mth active voxel in the nth leaf node (useful for
+    /// user-managed value buffers, e.g. in tools/LevelSetAdvect.h).
+    /// @return The total number of active values in the leaf nodes
+    /// @param offsets    array of prefix sums of offsets to active voxels
+    /// @param size       on input, the size of @a offsets; on output, its new size
+    /// @param grainSize  optional grain size for threading
+    /// @details If @a offsets is @c nullptr or @a size is smaller than the
+    /// total number of active voxels (the return value) then @a offsets
+    /// is reallocated and @a size equals the total number of active voxels.
+    size_t getPrefixSum(size_t*& offsets, size_t& size, size_t grainSize=1) const
+    {
+        if (offsets == nullptr || size < mLeafCount) {
+            delete [] offsets;
+            offsets = new size_t[mLeafCount];
+            size = mLeafCount;
+        }
+        size_t prefix = 0;
+        if ( grainSize > 0 ) {
+            PrefixSum tmp(this->leafRange( grainSize ), offsets, prefix);
+        } else {// serial
+            for (size_t i=0; i<mLeafCount; ++i) {
+                offsets[i] = prefix;
+                prefix += mLeafs[i]->onVoxelCount();
+            }
+        }
+        return prefix;
+    }
+
     ////////////////////////////////////////////////////////////////////////////////////
     // All methods below are for internal use only and should never be called directly
 
@@ -530,15 +661,13 @@ public:
         else OPENVDB_THROW(ValueError, "task is undefined");
     }
 
-
-
-  private:
+private:
 
     // This a simple wrapper for a c-style array so it mimics the api
     // of a std container, e.g. std::vector or std::deque, and can be
     // passed to Tree::getNodes().
     struct MyArray {
-        typedef LeafType* value_type;//required by Tree::getNodes
+        using value_type = LeafType*;//required by Tree::getNodes
         value_type* ptr;
         MyArray(value_type* array) : ptr(array) {}
         void push_back(value_type leaf) { *ptr++ = leaf; }//required by Tree::getNodes
@@ -549,7 +678,7 @@ public:
         const size_t leafCount = mTree->leafCount();
         if (leafCount != mLeafCount) {
             delete [] mLeafs;
-            mLeafs = (leafCount == 0) ? NULL : new LeafType*[leafCount];
+            mLeafs = (leafCount == 0) ? nullptr : new LeafType*[leafCount];
             mLeafCount = leafCount;
         }
         MyArray a(mLeafs);
@@ -561,7 +690,7 @@ public:
         const size_t auxBufferCount = mLeafCount * mAuxBuffersPerLeaf;
         if (auxBufferCount != mAuxBufferCount) {
             delete [] mAuxBuffers;
-            mAuxBuffers = (auxBufferCount == 0) ? NULL : new NonConstBufferType[auxBufferCount];
+            mAuxBuffers = (auxBufferCount == 0) ? nullptr : new NonConstBufferType[auxBufferCount];
             mAuxBufferCount = auxBufferCount;
         }
         this->syncAllBuffers(serial);
@@ -621,30 +750,80 @@ public:
     }
 
     /// @brief Private member class that applies a user-defined
-    /// functor to all the leaf nodes.
+    /// functor to perform parallel_for on all the leaf nodes.
     template<typename LeafOp>
     struct LeafTransformer
     {
-        LeafTransformer(const LeafOp& leafOp) : mLeafOp(leafOp) {}
-        void run(const LeafRange& range, bool threaded = true)
+        LeafTransformer(const LeafOp &leafOp) : mLeafOp(leafOp)
+        {
+        }
+        void run(const LeafRange &range, bool threaded) const
         {
             threaded ? tbb::parallel_for(range, *this) : (*this)(range);
         }
-        void operator()(const LeafRange& range) const
+        void operator()(const LeafRange &range) const
         {
             for (typename LeafRange::Iterator it = range.begin(); it; ++it) mLeafOp(*it, it.pos());
         }
         const LeafOp mLeafOp;
-    };
+    };// LeafTransformer
 
-    typedef typename boost::function<void (LeafManager*, const RangeType&)> FuncType;
+    /// @brief Private member class that applies a user-defined
+    /// functor to perform parallel_reduce on all the leaf nodes.
+    template<typename LeafOp>
+    struct LeafReducer
+    {
+        LeafReducer(LeafOp &leafOp) : mLeafOp(&leafOp), mOwnsOp(false)
+        {
+        }
+        LeafReducer(const LeafReducer &other, tbb::split)
+            : mLeafOp(new LeafOp(*(other.mLeafOp), tbb::split())), mOwnsOp(true)
+        {
+        }
+        ~LeafReducer() { if (mOwnsOp) delete mLeafOp; }
+        void run(const LeafRange& range, bool threaded)
+        {
+            threaded ? tbb::parallel_reduce(range, *this) : (*this)(range);
+        }
+        void operator()(const LeafRange& range)
+        {
+            LeafOp &op = *mLeafOp;//local registry
+            for (typename LeafRange::Iterator it = range.begin(); it; ++it) op(*it, it.pos());
+        }
+        void join(const LeafReducer& other) { mLeafOp->join(*(other.mLeafOp)); }
+        LeafOp *mLeafOp;
+        const bool mOwnsOp;
+    };// LeafReducer
 
-    TreeType*           mTree;
-    size_t              mLeafCount, mAuxBufferCount, mAuxBuffersPerLeaf;
-    LeafType**          mLeafs;//array of LeafNode pointers
-    NonConstBufferType* mAuxBuffers;//array of auxiliary buffers
-    FuncType            mTask;
-    const bool          mIsMaster;
+    // Helper class to compute a prefix sum of offsets to active voxels
+    struct PrefixSum
+    {
+        PrefixSum(const LeafRange& r, size_t* offsets, size_t& prefix)
+            : mOffsets(offsets)
+        {
+            tbb::parallel_for( r, *this);
+            for (size_t i=0, leafCount = r.size(); i<leafCount; ++i) {
+                size_t tmp = offsets[i];
+                offsets[i] = prefix;
+                prefix += tmp;
+            }
+        }
+        inline void operator()(const LeafRange& r) const {
+            for (typename LeafRange::Iterator i = r.begin(); i; ++i) {
+                mOffsets[i.pos()] = i->onVoxelCount();
+            }
+        }
+        size_t* mOffsets;
+    };// PrefixSum
+
+    using FuncType = typename std::function<void (LeafManager*, const RangeType&)>;
+
+    TreeType*            mTree;
+    size_t               mLeafCount, mAuxBufferCount, mAuxBuffersPerLeaf;
+    LeafType**           mLeafs;//array of LeafNode pointers
+    NonConstBufferType*  mAuxBuffers;//array of auxiliary buffers
+    FuncType             mTask;
+    const bool           mIsMaster;
 };//end of LeafManager class
 
 
@@ -652,13 +831,13 @@ public:
 template<typename TreeT>
 struct LeafManagerImpl<LeafManager<const TreeT> >
 {
-    typedef LeafManager<const TreeT> ManagerT;
-    typedef typename ManagerT::RangeType      RangeT;
-    typedef typename ManagerT::LeafType       LeafT;
-    typedef typename ManagerT::BufferType     BufT;
+    using ManagerT = LeafManager<const TreeT>;
+    using RangeT = typename ManagerT::RangeType;
+    using LeafT = typename ManagerT::LeafType;
+    using BufT = typename ManagerT::BufferType;
 
     static inline void doSwapLeafBuffer(const RangeT&, size_t /*auxBufferIdx*/,
-        LeafT**, BufT*, size_t /*bufsPerLeaf*/)
+                                        LeafT**, BufT*, size_t /*bufsPerLeaf*/)
     {
         // Buffers can't be swapped into const trees.
     }
@@ -670,6 +849,6 @@ struct LeafManagerImpl<LeafManager<const TreeT> >
 
 #endif // OPENVDB_TREE_LEAFMANAGER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
