@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -31,22 +31,24 @@
 #ifndef OPENVDB_TREE_LEAF_NODE_MASK_HAS_BEEN_INCLUDED
 #define OPENVDB_TREE_LEAF_NODE_MASK_HAS_BEEN_INCLUDED
 
-#include <iostream>
-#include <boost/shared_ptr.hpp>
-#include <boost/shared_array.hpp>
-#include <boost/static_assert.hpp>
+#include <openvdb/version.h>
 #include <openvdb/Types.h>
 #include <openvdb/io/Compression.h> // for io::readData(), etc.
 #include <openvdb/math/Math.h> // for math::isZero()
 #include <openvdb/util/NodeMasks.h>
 #include "LeafNode.h"
 #include "Iterator.h"
+#include <iostream>
+#include <sstream>
+#include <string>
+#include <type_traits>
+#include <vector>
 
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
-namespace tree {        
+namespace tree {
 
 /// @brief LeafNode specialization for values of type ValueMask that encodes both
 /// the active states and the boolean values of (2^Log2Dim)^3 voxels
@@ -55,11 +57,12 @@ template<Index Log2Dim>
 class LeafNode<ValueMask, Log2Dim>
 {
 public:
-    typedef LeafNode<ValueMask, Log2Dim>    LeafNodeType;
-    typedef boost::shared_ptr<LeafNodeType> Ptr;
-    typedef ValueMask                       BuildType;// this is a rare case where 
-    typedef bool                            ValueType;// value type != build type
-    typedef util::NodeMask<Log2Dim>         NodeMaskType;
+    using LeafNodeType = LeafNode<ValueMask, Log2Dim>;
+    using BuildType = ValueMask;// this is a rare case where
+    using ValueType = bool;// value type != build type
+    using Buffer = LeafBuffer<ValueType, Log2Dim>;// buffer uses the bool specialization
+    using NodeMaskType = util::NodeMask<Log2Dim>;
+    using Ptr = SharedPtr<LeafNodeType>;
 
     // These static declarations must be on separate lines to avoid VC9 compiler errors.
     static const Index LOG2DIM    = Log2Dim;    // needed by parent nodes
@@ -73,9 +76,7 @@ public:
     /// @brief ValueConverter<T>::Type is the type of a LeafNode having the same
     /// dimensions as this node but a different value type, T.
     template<typename OtherValueType>
-    struct ValueConverter {
-        typedef LeafNode<OtherValueType, Log2Dim> Type;
-    };
+    struct ValueConverter { using Type = LeafNode<OtherValueType, Log2Dim>; };
 
     /// @brief SameConfiguration<OtherNodeType>::value is @c true if and only if
     /// OtherNodeType is the type of a LeafNode with the same dimensions as this node.
@@ -84,78 +85,16 @@ public:
         static const bool value = SameLeafConfig<LOG2DIM, OtherNodeType>::value;
     };
 
-    class Buffer
-    {
-    public:
-        typedef typename NodeMaskType::Word WordType;
-        static const Index WORD_COUNT = NodeMaskType::WORD_COUNT;
-        Buffer() {}
-        explicit Buffer(bool on) : mData(on) {}
-        Buffer(const NodeMaskType& other): mData(other) {}
-        Buffer(const Buffer& other): mData(other.mData) {}
-        ~Buffer() {}
-        void fill(bool val) { mData.set(val); }
-        Buffer& operator=(const Buffer& b)
-        {
-            if (&b != this) mData = b.mData;
-            return *this;
-        }
-
-        const bool& getValue(Index i) const
-        {
-            assert(i < SIZE);
-            // We can't use the ternary operator here, otherwise Visual C++ returns
-            // a reference to a temporary.
-            if (mData.isOn(i)) return LeafNode::sOn;
-            return LeafNode::sOff;
-        }
-        const bool& operator[](Index i) const { return this->getValue(i); }
-
-        bool operator==(const Buffer& other) const { return mData == other.mData; }
-        bool operator!=(const Buffer& other) const { return mData != other.mData; }
-
-        void setValue(Index i, bool val) { assert(i < SIZE); mData.set(i, val); }
-
-        void swap(Buffer& other) { if (&other != this) std::swap(mData, other.mData); }
-
-        Index memUsage() const { return mData.memUsage(); }
-        static Index size() { return SIZE; }
-
-        /// Return a point to the c-style array of words encoding the bits.
-        /// @warning This method should only be used by experts that
-        /// seek low-level optimizations.
-        WordType* data()
-        {
-            return &(mData.template getWord<WordType>(0));
-        }
-        /// Return a const point to the c-style array of words
-        /// encoding the bits.
-        /// @warning This method should only be used by experts that
-        /// seek low-level optimizations.
-        const WordType* data() const
-        {
-            return const_cast<Buffer*>(this)->data();
-        }
-
-    private:
-        friend class ::TestLeaf;
-        // Allow the parent LeafNode to access this Buffer's bit mask.
-        friend class LeafNode;
-
-        NodeMaskType mData;
-    }; // class Buffer
-
-
     /// Default constructor
     LeafNode();
-    
+
     /// Constructor
     /// @param xyz     the coordinates of a voxel that lies within the node
     /// @param value   the initial value = state for all of this node's voxels
     /// @param dummy   dummy value
     explicit LeafNode(const Coord& xyz, bool value = false, bool dummy = false);
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
     /// "Partial creation" constructor used during file input
     LeafNode(PartialCreate, const Coord& xyz, bool value = false, bool dummy = false);
 #endif
@@ -219,7 +158,7 @@ public:
     /// Return @c true if this node only contains active voxels.
     bool isDense() const { return mBuffer.mData.isOn(); }
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
     /// @brief Return @c true if memory for this node's buffer has been allocated.
     /// @details Currently, boolean leaf nodes don't support partial creation,
     /// so this always returns @c true.
@@ -371,9 +310,11 @@ public:
     /// Set all voxels that lie outside the given axis-aligned box to the background.
     void clip(const CoordBBox&, bool background);
 
-    /// Set all voxels within an axis-aligned box to the specified value and active state.
-    void fill(const CoordBBox& bbox, bool value, bool dummy = false);
-    
+    /// Set all voxels within an axis-aligned box to the specified value.
+    void fill(const CoordBBox& bbox, bool value, bool = false);
+    /// Set all voxels within an axis-aligned box to the specified value.
+    void denseFill(const CoordBBox& bbox, bool value, bool = false) { this->fill(bbox, value); }
+
     /// Set the state of all voxels to the specified active state.
     void fill(const bool& value, bool dummy = false);
 
@@ -482,16 +423,43 @@ public:
     /// @brief Return a const reference to the first entry in the buffer.
     /// @note Since it's actually a reference to a static data member
     /// it should not be converted to a non-const pointer!
-    const bool& getFirstValue() const { if (mBuffer.mData.isOn(0)) return sOn; else return sOff; }
+    const bool& getFirstValue() const { if (mBuffer.mData.isOn(0)) return Buffer::sOn; else return Buffer::sOff; }
     /// @brief Return a const reference to the last entry in the buffer.
     /// @note Since it's actually a reference to a static data member
     /// it should not be converted to a non-const pointer!
-    const bool& getLastValue() const { if (mBuffer.mData.isOn(SIZE-1)) return sOn; else return sOff; }
+    const bool& getLastValue() const { if (mBuffer.mData.isOn(SIZE-1)) return Buffer::sOn; else return Buffer::sOff; }
 
     /// Return @c true if all of this node's voxels have the same active state
     /// and are equal to within the given tolerance, and return the value in
     /// @a constValue and the active state in @a state.
     bool isConstant(bool& constValue, bool& state, bool tolerance = 0) const;
+
+    /// @brief Computes the median value of all the active and inactive voxels in this node.
+    /// @return The median value.
+    ///
+    /// @details The median for boolean values is defined as the mode
+    /// of the values, i.e. the value that occurs most often.
+    bool medianAll() const;
+
+    /// @brief Computes the median value of all the active voxels in this node.
+    /// @return The number of active voxels.
+    ///
+    /// @param value Updated with the median value of all the active voxels.
+    ///
+    /// @note Since the value and state are shared for this
+    ///       specialization of the LeafNode the @a value will always be true!
+    Index medianOn(ValueType &value) const;
+
+    /// @brief Computes the median value of all the inactive voxels in this node.
+    /// @return The number of inactive voxels.
+    ///
+    /// @param value Updated with the median value of all the inactive
+    /// voxels.
+    ///
+    /// @note Since the value and state are shared for this
+    ///       specialization of the LeafNode the @a value will always be false!
+    Index medianOff(ValueType &value) const;
+
     /// Return @c true if all of this node's values are inactive.
     bool isInactive() const { return mBuffer.mData.isOff(); }
 
@@ -584,11 +552,11 @@ public:
     template<typename AccessorT>
     void addLeafAndCache(LeafNode*, AccessorT&) {}
     template<typename NodeT>
-    NodeT* stealNode(const Coord&, const ValueType&, bool) { return NULL; }
+    NodeT* stealNode(const Coord&, const ValueType&, bool) { return nullptr; }
     template<typename NodeT>
-    NodeT* probeNode(const Coord&) { return NULL; }
+    NodeT* probeNode(const Coord&) { return nullptr; }
     template<typename NodeT>
-    const NodeT* probeConstNode(const Coord&) const { return NULL; }
+    const NodeT* probeConstNode(const Coord&) const { return nullptr; }
     template<typename ArrayT> void getNodes(ArrayT&) const {}
     template<typename ArrayT> void stealNodes(ArrayT&, const ValueType&, bool) {}
     //@}
@@ -610,7 +578,7 @@ public:
     NodeT* probeNodeAndCache(const Coord&, AccessorT&)
     {
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_BEGIN
-        if (!(boost::is_same<NodeT,LeafNode>::value)) return NULL;
+        if (!(std::is_same<NodeT, LeafNode>::value)) return nullptr;
         return reinterpret_cast<NodeT*>(this);
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_END
     }
@@ -627,7 +595,7 @@ public:
     const NodeT* probeConstNodeAndCache(const Coord&, AccessorT&) const
     {
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_BEGIN
-        if (!(boost::is_same<NodeT,LeafNode>::value)) return NULL;
+        if (!(std::is_same<NodeT, LeafNode>::value)) return nullptr;
         return reinterpret_cast<const NodeT*>(this);
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_END
     }
@@ -637,9 +605,9 @@ public:
     // Iterators
     //
 protected:
-    typedef typename NodeMaskType::OnIterator    MaskOnIter;
-    typedef typename NodeMaskType::OffIterator   MaskOffIter;
-    typedef typename NodeMaskType::DenseIterator MaskDenseIter;
+    using MaskOnIter = typename NodeMaskType::OnIterator;
+    using MaskOffIter = typename NodeMaskType::OffIterator;
+    using MaskDenseIter = typename NodeMaskType::DenseIterator;
 
     template<typename MaskIterT, typename NodeT, typename ValueT>
     struct ValueIter:
@@ -647,7 +615,7 @@ protected:
         // if MaskIterT is a dense mask iterator type.
         public SparseIteratorBase<MaskIterT, ValueIter<MaskIterT, NodeT, ValueT>, NodeT, ValueT>
     {
-        typedef SparseIteratorBase<MaskIterT, ValueIter, NodeT, ValueT> BaseT;
+        using BaseT = SparseIteratorBase<MaskIterT, ValueIter, NodeT, ValueT>;
 
         ValueIter() {}
         ValueIter(const MaskIterT& iter, NodeT* parent): BaseT(iter, parent) {}
@@ -682,8 +650,8 @@ protected:
     struct DenseIter: public DenseIteratorBase<
         MaskDenseIter, DenseIter<NodeT, ValueT>, NodeT, /*ChildT=*/void, ValueT>
     {
-        typedef DenseIteratorBase<MaskDenseIter, DenseIter, NodeT, void, ValueT> BaseT;
-        typedef typename BaseT::NonConstValueType NonConstValueT;
+        using BaseT = DenseIteratorBase<MaskDenseIter, DenseIter, NodeT, void, ValueT>;
+        using NonConstValueT = typename BaseT::NonConstValueType;
 
         DenseIter() {}
         DenseIter(const MaskDenseIter& iter, NodeT* parent): BaseT(iter, parent) {}
@@ -691,7 +659,7 @@ protected:
         bool getItem(Index pos, void*& child, NonConstValueT& value) const
         {
             value = this->parent().getValue(pos);
-            child = NULL;
+            child = nullptr;
             return false; // no child
         }
 
@@ -703,18 +671,18 @@ protected:
     };
 
 public:
-    typedef ValueIter<MaskOnIter, LeafNode, const bool>           ValueOnIter;
-    typedef ValueIter<MaskOnIter, const LeafNode, const bool>     ValueOnCIter;
-    typedef ValueIter<MaskOffIter, LeafNode, const bool>          ValueOffIter;
-    typedef ValueIter<MaskOffIter, const LeafNode, const bool>    ValueOffCIter;
-    typedef ValueIter<MaskDenseIter, LeafNode, const bool>        ValueAllIter;
-    typedef ValueIter<MaskDenseIter, const LeafNode, const bool>  ValueAllCIter;
-    typedef ChildIter<MaskOnIter, LeafNode>                       ChildOnIter;
-    typedef ChildIter<MaskOnIter, const LeafNode>                 ChildOnCIter;
-    typedef ChildIter<MaskOffIter, LeafNode>                      ChildOffIter;
-    typedef ChildIter<MaskOffIter, const LeafNode>                ChildOffCIter;
-    typedef DenseIter<LeafNode, bool>                             ChildAllIter;
-    typedef DenseIter<const LeafNode, const bool>                 ChildAllCIter;
+    using ValueOnIter = ValueIter<MaskOnIter, LeafNode, const bool>;
+    using ValueOnCIter = ValueIter<MaskOnIter, const LeafNode, const bool>;
+    using ValueOffIter = ValueIter<MaskOffIter, LeafNode, const bool>;
+    using ValueOffCIter = ValueIter<MaskOffIter, const LeafNode, const bool>;
+    using ValueAllIter = ValueIter<MaskDenseIter, LeafNode, const bool>;
+    using ValueAllCIter = ValueIter<MaskDenseIter, const LeafNode, const bool>;
+    using ChildOnIter = ChildIter<MaskOnIter, LeafNode>;
+    using ChildOnCIter = ChildIter<MaskOnIter, const LeafNode>;
+    using ChildOffIter = ChildIter<MaskOffIter, LeafNode>;
+    using ChildOffCIter = ChildIter<MaskOffIter, const LeafNode>;
+    using ChildAllIter = DenseIter<LeafNode, bool>;
+    using ChildAllCIter = DenseIter<const LeafNode, const bool>;
 
     ValueOnCIter  cbeginValueOn() const { return ValueOnCIter(mBuffer.mData.beginOn(), this); }
     ValueOnCIter   beginValueOn() const { return ValueOnCIter(mBuffer.mData.beginOn(), this); }
@@ -790,17 +758,13 @@ protected:
     template<typename NodeT, typename VisitorOp,
         typename ChildAllIterT, typename OtherChildAllIterT>
     static inline void doVisit2(NodeT& self, OtherChildAllIterT&, VisitorOp&, bool otherIsLHS);
-    
+
     /// Bitmask representing the values AND state of voxels
     Buffer mBuffer;
 
     /// Global grid index coordinates (x,y,z) of the local origin of this node
     Coord mOrigin;
 
-    // These static declarations must be on separate lines to avoid VC9 compiler errors.
-    static const bool sOn;
-    static const bool sOff;
-
 private:
     /// @brief During topology-only construction, access is needed
     /// to protected/private members of other template instances.
@@ -821,17 +785,11 @@ private:
     friend class IteratorBase<MaskDenseIter, LeafNode>;
     //@}
 
+    template<typename, Index> friend class LeafBuffer;
+
 }; // class LeafNode<ValueMask>
 
 
-/// @internal For consistency with other nodes and with iterators, methods like
-/// LeafNode::getValue() return a reference to a value.  Since it's not possible
-/// to return a reference to a bit in a node mask, we return a reference to one
-/// of the following static values instead.
-template<Index Log2Dim> const bool LeafNode<ValueMask, Log2Dim>::sOn = true;
-template<Index Log2Dim> const bool LeafNode<ValueMask, Log2Dim>::sOff = false;
-
-
 ////////////////////////////////////////
 
 
@@ -844,18 +802,18 @@ LeafNode<ValueMask, Log2Dim>::LeafNode()
 
 template<Index Log2Dim>
 inline
-LeafNode<ValueMask, Log2Dim>::LeafNode(const Coord& xyz, bool value, bool)
-    : mBuffer(value)
+LeafNode<ValueMask, Log2Dim>::LeafNode(const Coord& xyz, bool value, bool active)
+    : mBuffer(value || active)
     , mOrigin(xyz & (~(DIM - 1)))
 {
 }
 
 
-#ifndef OPENVDB_2_ABI_COMPATIBLE
+#if OPENVDB_ABI_VERSION_NUMBER >= 3
 template<Index Log2Dim>
 inline
-LeafNode<ValueMask, Log2Dim>::LeafNode(PartialCreate, const Coord& xyz, bool value, bool)
-    : mBuffer(value)
+LeafNode<ValueMask, Log2Dim>::LeafNode(PartialCreate, const Coord& xyz, bool value, bool active)
+    : mBuffer(value || active)
     , mOrigin(xyz & (~(DIM - 1)))
 {
 }
@@ -1100,12 +1058,41 @@ inline bool
 LeafNode<ValueMask, Log2Dim>::isConstant(bool& constValue, bool& state, bool) const
 {
     if (!mBuffer.mData.isConstant(state)) return false;
-    
+
     constValue = state;
     return true;
 }
 
 
+////////////////////////////////////////
+
+template<Index Log2Dim>
+inline bool
+LeafNode<ValueMask, Log2Dim>::medianAll() const
+{
+    const Index countTrue = mBuffer.mData.countOn();
+    return countTrue > (NUM_VALUES >> 1);
+}
+
+template<Index Log2Dim>
+inline Index
+LeafNode<ValueMask, Log2Dim>::medianOn(bool& state) const
+{
+    const Index countTrueOn = mBuffer.mData.countOn();
+    state = true;//since value and state are the same for this specialization of the leaf node
+    return countTrueOn;
+}
+
+template<Index Log2Dim>
+inline Index
+LeafNode<ValueMask, Log2Dim>::medianOff(bool& state) const
+{
+    const Index countFalseOff = mBuffer.mData.countOff();
+    state = false;//since value and state are the same for this specialization of the leaf node
+    return countFalseOff;
+}
+
+
 ////////////////////////////////////////
 
 
@@ -1143,7 +1130,7 @@ inline const bool&
 LeafNode<ValueMask, Log2Dim>::getValue(const Coord& xyz) const
 {
     // This *CANNOT* use operator ? because Visual C++
-    if (mBuffer.mData.isOn(this->coordToOffset(xyz))) return sOn; else return sOff;
+    if (mBuffer.mData.isOn(this->coordToOffset(xyz))) return Buffer::sOn; else return Buffer::sOff;
 }
 
 
@@ -1153,7 +1140,7 @@ LeafNode<ValueMask, Log2Dim>::getValue(Index offset) const
 {
     assert(offset < SIZE);
     // This *CANNOT* use operator ? for Windows
-    if (mBuffer.mData.isOn(offset)) return sOn; else return sOff;
+    if (mBuffer.mData.isOn(offset)) return Buffer::sOn; else return Buffer::sOff;
 }
 
 
@@ -1354,11 +1341,15 @@ template<Index Log2Dim>
 inline void
 LeafNode<ValueMask, Log2Dim>::fill(const CoordBBox& bbox, bool value, bool)
 {
-    for (Int32 x = bbox.min().x(); x <= bbox.max().x(); ++x) {
+    auto clippedBBox = this->getNodeBoundingBox();
+    clippedBBox.intersect(bbox);
+    if (!clippedBBox) return;
+
+    for (Int32 x = clippedBBox.min().x(); x <= clippedBBox.max().x(); ++x) {
         const Index offsetX = (x & (DIM-1u))<<2*Log2Dim;
-        for (Int32 y = bbox.min().y(); y <= bbox.max().y(); ++y) {
+        for (Int32 y = clippedBBox.min().y(); y <= clippedBBox.max().y(); ++y) {
             const Index offsetXY = offsetX + ((y & (DIM-1u))<<  Log2Dim);
-            for (Int32 z = bbox.min().z(); z <= bbox.max().z(); ++z) {
+            for (Int32 z = clippedBBox.min().z(); z <= clippedBBox.max().z(); ++z) {
                 const Index offset = offsetXY + (z & (DIM-1u));
                 mBuffer.mData.set(offset, value);
             }
@@ -1382,7 +1373,7 @@ template<typename DenseT>
 inline void
 LeafNode<ValueMask, Log2Dim>::copyToDense(const CoordBBox& bbox, DenseT& dense) const
 {
-    typedef typename DenseT::ValueType DenseValueType;
+    using DenseValueType = typename DenseT::ValueType;
 
     const size_t xStride = dense.xStride(), yStride = dense.yStride(), zStride = dense.zStride();
     const Coord& min = dense.bbox().min();
@@ -1408,7 +1399,7 @@ inline void
 LeafNode<ValueMask, Log2Dim>::copyFromDense(const CoordBBox& bbox, const DenseT& dense,
                                        bool background, bool tolerance)
 {
-    typedef typename DenseT::ValueType DenseValueType;
+    using DenseValueType = typename DenseT::ValueType;
     struct Local {
         inline static bool toBool(const DenseValueType& v) { return !math::isZero(v); }
     };
@@ -1619,8 +1610,10 @@ inline void
 LeafNode<ValueMask, Log2Dim>::doVisit2Node(NodeT& self, OtherNodeT& other, VisitorOp& op)
 {
     // Allow the two nodes to have different ValueTypes, but not different dimensions.
-    BOOST_STATIC_ASSERT(OtherNodeT::SIZE == NodeT::SIZE);
-    BOOST_STATIC_ASSERT(OtherNodeT::LEVEL == NodeT::LEVEL);
+    static_assert(OtherNodeT::SIZE == NodeT::SIZE,
+        "can't visit nodes of different sizes simultaneously");
+    static_assert(OtherNodeT::LEVEL == NodeT::LEVEL,
+        "can't visit nodes at different tree levels simultaneously");
 
     ChildAllIterT iter = self.beginChildAll();
     OtherChildAllIterT otherIter = other.beginChildAll();
@@ -1681,6 +1674,6 @@ LeafNode<ValueMask, Log2Dim>::doVisit2(NodeT& self, OtherChildAllIterT& otherIte
 
 #endif // OPENVDB_TREE_LEAF_NODE_MASK_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
