@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -27,8 +27,8 @@
 // LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
 //
 ///////////////////////////////////////////////////////////////////////////
-//
-/// @file NodeManager.h
+
+/// @file tree/NodeManager.h
 ///
 /// @author Ken Museth
 ///
@@ -41,11 +41,12 @@
 #ifndef OPENVDB_TREE_NODEMANAGER_HAS_BEEN_INCLUDED
 #define OPENVDB_TREE_NODEMANAGER_HAS_BEEN_INCLUDED
 
+#include <openvdb/Types.h>
 #include <tbb/parallel_for.h>
 #include <tbb/parallel_reduce.h>
-#include <openvdb/Types.h>
 #include <deque>
 
+
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
@@ -67,8 +68,8 @@ template<typename NodeT>
 class NodeList
 {
 public:
-    typedef NodeT* value_type;
-    typedef std::deque<value_type> ListT;
+    using value_type = NodeT*;
+    using ListT = std::deque<value_type>;
 
     NodeList() {}
 
@@ -112,10 +113,8 @@ public:
             {
                 assert(this->isValid());
             }
-            Iterator& operator=(const Iterator& other)
-            {
-                mRange = other.mRange; mPos = other.mPos; return *this;
-            }
+            Iterator(const Iterator&) = default;
+            Iterator& operator=(const Iterator&) = default;
             /// Advance to the next node.
             Iterator& operator++() { ++mPos; return *this; }
             /// Return a reference to the node to which this iterator is pointing.
@@ -199,7 +198,7 @@ private:
         }
         const NodeOp mNodeOp;
     };// NodeList::NodeTransformer
-    
+
     // Private struct of NodeList that performs parallel_reduce
     template<typename NodeOp>
     struct NodeReducer
@@ -207,7 +206,7 @@ private:
         NodeReducer(NodeOp& nodeOp) : mNodeOp(&nodeOp), mOwnsOp(false)
         {
         }
-        NodeReducer(const NodeReducer& other, tbb::split) : 
+        NodeReducer(const NodeReducer& other, tbb::split) :
             mNodeOp(new NodeOp(*(other.mNodeOp), tbb::split())), mOwnsOp(true)
         {
         }
@@ -228,7 +227,7 @@ private:
         NodeOp *mNodeOp;
         const bool mOwnsOp;
     };// NodeList::NodeReducer
-    
+
 
 protected:
     ListT mList;
@@ -288,18 +287,6 @@ public:
         mNext.foreachTopDown(op, threaded, grainSize);
     }
 
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processBottomUp(const NodeOp& op, bool threaded, size_t grainSize)
-    {
-        this->foreachBottomUp<NodeOp>(op, threaded, grainSize);
-    }
-
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processTopDown(const NodeOp& op, bool threaded, size_t grainSize)
-    {
-        this->foreachTopDown<NodeOp>(op, threaded, grainSize);
-    }
-    
     template<typename NodeOp>
     void reduceBottomUp(NodeOp& op, bool threaded, size_t grainSize)
     {
@@ -323,8 +310,8 @@ protected:
 ////////////////////////////////////////
 
 
+/// @private
 /// @brief Specialization that terminates the chain of cached tree nodes
-///
 /// @note It is for internal use and should rarely be used directly.
 template<typename NodeT>
 class NodeManagerLink<NodeT, 0>
@@ -356,18 +343,6 @@ public:
         mList.foreach(op, threaded, grainSize);
     }
 
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processBottomUp(const NodeOp& op, bool threaded, size_t grainSize)
-    {
-        this->foreachBottomUp<NodeOp>(op, threaded, grainSize);
-    }
-
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processTopDown(const NodeOp& op, bool threaded, size_t grainSize)
-    {
-        this->foreachTopDown<NodeOp>(op, threaded, grainSize);
-    }
-
     template<typename NodeOp>
     void reduceBottomUp(NodeOp& op, bool threaded, size_t grainSize)
     {
@@ -409,9 +384,10 @@ class NodeManager
 {
 public:
     static const Index LEVELS = _LEVELS;
-    BOOST_STATIC_ASSERT(LEVELS > 0);//special implementation below
-    typedef typename TreeOrLeafManagerT::RootNodeType RootNodeType;
-    BOOST_STATIC_ASSERT(RootNodeType::LEVEL >= LEVELS);
+    static_assert(LEVELS > 0,
+        "expected instantiation of template specialization"); // see specialization below
+    using RootNodeType = typename TreeOrLeafManagerT::RootNodeType;
+    static_assert(RootNodeType::LEVEL >= LEVELS, "number of levels exceeds root node height");
 
     NodeManager(TreeOrLeafManagerT& tree) : mRoot(tree.root()) { mChain.init(mRoot, tree); }
 
@@ -453,9 +429,9 @@ public:
     /// template<typename TreeType>
     /// struct OffsetOp
     /// {
-    ///     typedef typename TreeT::ValueType    ValueT;
-    ///     typedef typename TreeT::RootNodeType RootT;
-    ///     typedef typename TreeT::LeafNodeType LeafT;
+    ///     using ValueT = typename TreeT::ValueType;
+    ///     using RootT = typename TreeT::RootNodeType;
+    ///     using LeafT = typename TreeT::LeafNodeType;
     ///     OffsetOp(const ValueT& v) : mOffset(v) {}
     ///
     ///     // Processes the root node. Required by the NodeManager
@@ -484,7 +460,7 @@ public:
     /// nodes.foreachBottomUp(op);
     ///
     /// // or if a LeafManager already exists
-    /// typedef tree::LeafManager<FloatTree> T;
+    /// using T = tree::LeafManager<FloatTree>;
     /// OffsetOp<T> op(3.0f);
     /// tree::NodeManager<T> nodes(leafManager);
     /// nodes.foreachBottomUp(op);
@@ -496,22 +472,14 @@ public:
         mChain.foreachBottomUp(op, threaded, grainSize);
         op(mRoot);
     }
+
     template<typename NodeOp>
     void foreachTopDown(const NodeOp& op, bool threaded = true, size_t grainSize=1)
     {
         op(mRoot);
         mChain.foreachTopDown(op, threaded, grainSize);
     }
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processBottomUp(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachBottomUp<NodeOp>(op, threaded, grainSize);
-    }
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processTopDown(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachTopDown<NodeOp>(op, threaded, grainSize);
-    }
+
     //@}
 
     //@{
@@ -530,7 +498,7 @@ public:
     ///  template<typename TreeType>
     ///  struct NodeCountOp
     ///  {
-    ///      NodeCountOp() : nodeCount(TreeType::DEPTH, 0), totalCount(0) 
+    ///      NodeCountOp() : nodeCount(TreeType::DEPTH, 0), totalCount(0)
     ///      {
     ///      }
     ///      NodeCountOp(const NodeCountOp& other, tbb::split) :
@@ -547,10 +515,10 @@ public:
     ///      // do nothing for the root node
     ///      void operator()(const typename TreeT::RootNodeType& node)
     ///      {
-    ///      }  
+    ///      }
     ///      // count the internal and leaf nodes
     ///      template<typename NodeT>
-    ///      void operator()(const NodeT& node) 
+    ///      void operator()(const NodeT& node)
     ///      {
     ///          ++(nodeCount[NodeT::LEVEL]);
     ///          ++totalCount;
@@ -566,7 +534,7 @@ public:
     ///
     /// // or if a LeafManager already exists
     /// NodeCountOp<FloatTree> op;
-    /// typedef tree::LeafManager<FloatTree> T;
+    /// using T = tree::LeafManager<FloatTree>;
     /// T leafManager(tree);
     /// tree::NodeManager<T> nodes(leafManager);
     /// nodes.reduceBottomUp(op);
@@ -599,12 +567,13 @@ private:
 ////////////////////////////////////////////
 
 
+/// @private
 /// Template specialization of the NodeManager with no caching of nodes
 template<typename TreeOrLeafManagerT>
 class NodeManager<TreeOrLeafManagerT, 0>
 {
 public:
-    typedef typename TreeOrLeafManagerT::RootNodeType RootNodeType;
+    using RootNodeType = typename TreeOrLeafManagerT::RootNodeType;
     static const Index LEVELS = 0;
 
     NodeManager(TreeOrLeafManagerT& tree) : mRoot(tree.root()) {}
@@ -626,12 +595,6 @@ public:
 
     Index64 nodeCount(Index) const { return 0; }
 
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processBottomUp(const NodeOp& op, bool, size_t) { op(mRoot); }
-
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processTopDown(const NodeOp& op, bool, size_t) { op(mRoot); }
-
     template<typename NodeOp>
     void foreachBottomUp(const NodeOp& op, bool, size_t) { op(mRoot); }
 
@@ -655,13 +618,14 @@ private:
 ////////////////////////////////////////////
 
 
+/// @private
 /// Template specialization of the NodeManager with one level of nodes
 template<typename TreeOrLeafManagerT>
 class NodeManager<TreeOrLeafManagerT, 1>
 {
 public:
-    typedef typename TreeOrLeafManagerT::RootNodeType RootNodeType;
-    BOOST_STATIC_ASSERT(RootNodeType::LEVEL > 0);
+    using RootNodeType = typename TreeOrLeafManagerT::RootNodeType;
+    static_assert(RootNodeType::LEVEL > 0, "expected instantiation of template specialization");
     static const Index LEVELS = 1;
 
     NodeManager(TreeOrLeafManagerT& tree) : mRoot(tree.root())
@@ -707,17 +671,6 @@ public:
         op(mRoot);
         mList0.foreach(op, threaded, grainSize);
     }
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processBottomUp(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachBottomUp<NodeOp>(op, threaded, grainSize);
-    }
-
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processTopDown(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachTopDown<NodeOp>(op, threaded, grainSize);
-    }
 
     template<typename NodeOp>
     void reduceBottomUp(NodeOp& op, bool threaded = true, size_t grainSize=1)
@@ -734,9 +687,9 @@ public:
     }
 
 protected:
-    typedef RootNodeType                   NodeT1;
-    typedef typename NodeT1::ChildNodeType NodeT0;
-    typedef NodeList<NodeT0>               ListT0;
+    using NodeT1 = RootNodeType;
+    using NodeT0 = typename NodeT1::ChildNodeType;
+    using ListT0 = NodeList<NodeT0>;
 
     NodeT1& mRoot;
     ListT0 mList0;
@@ -749,13 +702,14 @@ private:
 ////////////////////////////////////////////
 
 
+/// @private
 /// Template specialization of the NodeManager with two levels of nodes
 template<typename TreeOrLeafManagerT>
 class NodeManager<TreeOrLeafManagerT, 2>
 {
 public:
-    typedef typename TreeOrLeafManagerT::RootNodeType RootNodeType;
-    BOOST_STATIC_ASSERT(RootNodeType::LEVEL > 1);
+    using RootNodeType = typename TreeOrLeafManagerT::RootNodeType;
+    static_assert(RootNodeType::LEVEL > 1, "expected instantiation of template specialization");
     static const Index LEVELS = 2;
 
     NodeManager(TreeOrLeafManagerT& tree) : mRoot(tree.root())
@@ -814,18 +768,6 @@ public:
         mList0.foreach(op, threaded, grainSize);
     }
 
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processBottomUp(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachBottomUp<NodeOp>(op, threaded, grainSize);
-    }
-
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processTopDown(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachTopDown<NodeOp>(op, threaded, grainSize);
-    }
-
     template<typename NodeOp>
     void reduceBottomUp(NodeOp& op, bool threaded = true, size_t grainSize=1)
     {
@@ -843,12 +785,12 @@ public:
     }
 
 protected:
-    typedef RootNodeType                   NodeT2;
-    typedef typename NodeT2::ChildNodeType NodeT1;//upper level
-    typedef typename NodeT1::ChildNodeType NodeT0;//lower level
+    using NodeT2 = RootNodeType;
+    using NodeT1 = typename NodeT2::ChildNodeType; // upper level
+    using NodeT0 = typename NodeT1::ChildNodeType; // lower level
 
-    typedef NodeList<NodeT1>               ListT1;//upper level
-    typedef NodeList<NodeT0>               ListT0;//lower level
+    using ListT1 = NodeList<NodeT1>; // upper level
+    using ListT0 = NodeList<NodeT0>; // lower level
 
     NodeT2& mRoot;
     ListT1 mList1;
@@ -862,13 +804,14 @@ private:
 ////////////////////////////////////////////
 
 
+/// @private
 /// Template specialization of the NodeManager with three levels of nodes
 template<typename TreeOrLeafManagerT>
 class NodeManager<TreeOrLeafManagerT, 3>
 {
 public:
-    typedef typename TreeOrLeafManagerT::RootNodeType RootNodeType;
-    BOOST_STATIC_ASSERT(RootNodeType::LEVEL > 2);
+    using RootNodeType = typename TreeOrLeafManagerT::RootNodeType;
+    static_assert(RootNodeType::LEVEL > 2, "expected instantiation of template specialization");
     static const Index LEVELS = 3;
 
     NodeManager(TreeOrLeafManagerT& tree) : mRoot(tree.root())
@@ -932,18 +875,6 @@ public:
         mList0.foreach(op, threaded, grainSize);
     }
 
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processBottomUp(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachBottomUp<NodeOp>(op, threaded, grainSize);
-    }
-
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processTopDown(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachTopDown<NodeOp>(op, threaded, grainSize);
-    }
-
     template<typename NodeOp>
     void reduceBottomUp(NodeOp& op, bool threaded = true, size_t grainSize=1)
     {
@@ -963,14 +894,14 @@ public:
     }
 
 protected:
-    typedef RootNodeType                   NodeT3;
-    typedef typename NodeT3::ChildNodeType NodeT2;//upper level
-    typedef typename NodeT2::ChildNodeType NodeT1;//mid level
-    typedef typename NodeT1::ChildNodeType NodeT0;//lower level
+    using NodeT3 = RootNodeType;
+    using NodeT2 = typename NodeT3::ChildNodeType; // upper level
+    using NodeT1 = typename NodeT2::ChildNodeType; // mid level
+    using NodeT0 = typename NodeT1::ChildNodeType; // lower level
 
-    typedef NodeList<NodeT2>               ListT2;//upper level of internal nodes
-    typedef NodeList<NodeT1>               ListT1;//lower level of internal nodes
-    typedef NodeList<NodeT0>               ListT0;//lower level of internal nodes or leafs
+    using ListT2 = NodeList<NodeT2>; // upper level of internal nodes
+    using ListT1 = NodeList<NodeT1>; // lower level of internal nodes
+    using ListT0 = NodeList<NodeT0>; // lower level of internal nodes or leafs
 
     NodeT3& mRoot;
     ListT2 mList2;
@@ -985,13 +916,14 @@ private:
 ////////////////////////////////////////////
 
 
+/// @private
 /// Template specialization of the NodeManager with four levels of nodes
 template<typename TreeOrLeafManagerT>
 class NodeManager<TreeOrLeafManagerT, 4>
 {
 public:
-    typedef typename TreeOrLeafManagerT::RootNodeType RootNodeType;
-    BOOST_STATIC_ASSERT(RootNodeType::LEVEL > 3);
+    using RootNodeType = typename TreeOrLeafManagerT::RootNodeType;
+    static_assert(RootNodeType::LEVEL > 3, "expected instantiation of template specialization");
     static const Index LEVELS = 4;
 
     NodeManager(TreeOrLeafManagerT& tree) : mRoot(tree.root())
@@ -1063,18 +995,6 @@ public:
         mList0.foreach(op, threaded, grainSize);
     }
 
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processBottomUp(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachBottomUp<NodeOp>(op, threaded, grainSize);
-    }
-
-    template<typename NodeOp>
-    OPENVDB_DEPRECATED void processTopDown(const NodeOp& op, bool threaded = true, size_t grainSize=1)
-    {
-        this->foreachTopDown<NodeOp>(op, threaded, grainSize);
-    }
-
     template<typename NodeOp>
     void reduceBottomUp(NodeOp& op, bool threaded = true, size_t grainSize=1)
     {
@@ -1096,16 +1016,16 @@ public:
     }
 
 protected:
-    typedef RootNodeType                   NodeT4;
-    typedef typename NodeT4::ChildNodeType NodeT3;//upper level
-    typedef typename NodeT3::ChildNodeType NodeT2;//upper mid level
-    typedef typename NodeT2::ChildNodeType NodeT1;//lower mid level
-    typedef typename NodeT1::ChildNodeType NodeT0;//lower level
+    using NodeT4 = RootNodeType;
+    using NodeT3 = typename NodeT4::ChildNodeType; // upper level
+    using NodeT2 = typename NodeT3::ChildNodeType; // upper mid level
+    using NodeT1 = typename NodeT2::ChildNodeType; // lower mid level
+    using NodeT0 = typename NodeT1::ChildNodeType; // lower level
 
-    typedef NodeList<NodeT3>               ListT3;//upper level of internal nodes
-    typedef NodeList<NodeT2>               ListT2;//upper mid level of internal nodes
-    typedef NodeList<NodeT1>               ListT1;//lower mid level of internal nodes
-    typedef NodeList<NodeT0>               ListT0;//lower level of internal nodes or leafs
+    using ListT3 = NodeList<NodeT3>; // upper level of internal nodes
+    using ListT2 = NodeList<NodeT2>; // upper mid level of internal nodes
+    using ListT1 = NodeList<NodeT1>; // lower mid level of internal nodes
+    using ListT0 = NodeList<NodeT0>; // lower level of internal nodes or leafs
 
     NodeT4& mRoot;
     ListT3  mList3;
@@ -1123,6 +1043,6 @@ private:
 
 #endif // OPENVDB_TREE_NODEMANAGER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
