@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -32,23 +32,27 @@
 #include <openvdb/Exceptions.h>
 #include <openvdb/Metadata.h>
 
-class TestMetadata : public CppUnit::TestCase
+
+class TestMetadata: public CppUnit::TestCase
 {
 public:
-    virtual void setUp() { openvdb::Metadata::clearRegistry(); }
-    virtual void tearDown() { openvdb::Metadata::clearRegistry(); }
+    void setUp() override { openvdb::Metadata::clearRegistry(); }
+    void tearDown() override { openvdb::Metadata::clearRegistry(); }
 
     CPPUNIT_TEST_SUITE(TestMetadata);
     CPPUNIT_TEST(testMetadataRegistry);
     CPPUNIT_TEST(testMetadataAsBool);
+    CPPUNIT_TEST(testCustomMetadata);
     CPPUNIT_TEST_SUITE_END();
 
     void testMetadataRegistry();
     void testMetadataAsBool();
+    void testCustomMetadata();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestMetadata);
 
+
 void
 TestMetadata::testMetadataRegistry()
 {
@@ -63,18 +67,15 @@ TestMetadata::testMetadataRegistry()
     StringMetadata::registerType();
 
     CPPUNIT_ASSERT(Metadata::isRegisteredType(strMetadata.typeName()));
-    CPPUNIT_ASSERT(
-            Metadata::isRegisteredType(Int32Metadata::staticTypeName()));
+    CPPUNIT_ASSERT(Metadata::isRegisteredType(Int32Metadata::staticTypeName()));
 
-    Metadata::Ptr stringMetadata =
-        Metadata::createMetadata(strMetadata.typeName());
+    Metadata::Ptr stringMetadata = Metadata::createMetadata(strMetadata.typeName());
 
     CPPUNIT_ASSERT(stringMetadata->typeName() == strMetadata.typeName());
 
     StringMetadata::unregisterType();
 
-    CPPUNIT_ASSERT_THROW(Metadata::createMetadata(strMetadata.typeName()),
-                         openvdb::LookupError);
+    CPPUNIT_ASSERT_THROW(Metadata::createMetadata(strMetadata.typeName()), openvdb::LookupError);
 }
 
 void
@@ -122,6 +123,65 @@ TestMetadata::testMetadataAsBool()
     }
 }
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+
+void
+TestMetadata::testCustomMetadata()
+{
+    using namespace openvdb;
+
+    const Vec3i expected(1, 2, 3);
+    std::ostringstream ostr(std::ios_base::binary);
+    {
+        Vec3IMetadata::registerType();
+        Vec3IMetadata meta(expected);
+
+        // Write Vec3I metadata to a byte string.
+        meta.write(ostr);
+    }
+
+    // Unregister Vec3I metadata.
+    Metadata::clearRegistry();
+
+#if OPENVDB_ABI_VERSION_NUMBER >= 5
+    {
+        std::istringstream istr(ostr.str(), std::ios_base::binary);
+
+        UnknownMetadata meta;
+        // Verify that metadata of an unregistered type can be read successfully.
+        CPPUNIT_ASSERT_NO_THROW(meta.read(istr));
+
+        // Verify that the metadata matches the original vector value.
+        CPPUNIT_ASSERT_EQUAL(sizeof(Vec3i), size_t(meta.size()));
+        CPPUNIT_ASSERT(meta.value().size() == size_t(meta.size()));
+        CPPUNIT_ASSERT_EQUAL(expected, *reinterpret_cast<const Vec3i*>(&meta.value()[0]));
+
+        ostr.str("");
+        meta.write(ostr);
+
+        // Verify that UnknownMetadata can be copied.
+        auto metaPtr = meta.copy();
+        CPPUNIT_ASSERT(metaPtr.get() != nullptr);
+        CPPUNIT_ASSERT(meta == *metaPtr);
+
+        // Verify that typed metadata can be copied into UnknownMetadata.
+        meta.copy(Vec3IMetadata(expected));
+        CPPUNIT_ASSERT_EQUAL(sizeof(expected), size_t(meta.size()));
+        const auto* ptr = reinterpret_cast<const uint8_t*>(&expected);
+        CPPUNIT_ASSERT(UnknownMetadata::ByteVec(ptr, ptr + sizeof(expected)) == meta.value());
+    }
+#endif
+
+    Vec3IMetadata::registerType();
+
+    {
+        std::istringstream istr(ostr.str(), std::ios_base::binary);
+        Vec3IMetadata meta;
+        meta.read(istr);
+
+        CPPUNIT_ASSERT_EQUAL(expected, meta.value());
+    }
+}
+
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
