@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -40,13 +40,15 @@
 #include <openvdb/util/CpuTimer.h>
 #include <openvdb/util/PagedArray.h>
 
+#include <iostream>
+
 //#define BENCHMARK_PAGED_ARRAY
 
 // For benchmark comparisons
 #ifdef BENCHMARK_PAGED_ARRAY
-#include <vector>// for std::vector
-#include <deque>// for std::deque
-#include <tbb/tbb.h>// for tbb::concurrent_vector
+#include <deque> // for std::deque
+#include <vector> // for std::vector
+#include <tbb/tbb.h> // for tbb::concurrent_vector
 #endif
 
 class TestUtil: public CppUnit::TestCase
@@ -59,67 +61,66 @@ public:
 
     void testCpuTimer();
     void testPagedArray();
-    
-    // Multi-threading T::push_back
-    template<typename T>
-    struct Functor {
-        Functor(size_t begin, size_t end, size_t grainSize,
-               T& _d, bool threaded = true) : d(&_d) {
-            if (threaded) {
-                tbb::parallel_for(tbb::blocked_range<size_t>(begin, end, grainSize), *this);
-            } else {
-                (*this)(tbb::blocked_range<size_t>(begin, end));
-            }
+
+    using RangeT = tbb::blocked_range<size_t>;
+
+    // Multi-threading ArrayT::push_back
+    template<typename ArrayT>
+    struct ArrayPushBack {
+        ArrayPushBack(ArrayT& array) : mArray(&array) {}
+        void parallel(size_t size) {tbb::parallel_for(RangeT(size_t(0), size, mArray->pageSize()), *this);}
+        void serial(size_t size) { (*this)(RangeT(size_t(0), size)); }
+        void unsafe(size_t size) { for (size_t i=0; i!=size; ++i) mArray->push_back_unsafe(i); }
+        void operator()(const RangeT& r) const {
+            for (size_t i=r.begin(), n=r.end(); i!=n; ++i) mArray->push_back(i);
         }
-        void operator()(const tbb::blocked_range<size_t>& r) const {
-            for (size_t i=r.begin(), n=r.end(); i!=n; ++i) d->push_back(i);
-        }
-        T* d;
+        ArrayT* mArray;
     };
-    // Multi-threading T::ValueBuffer::push_back
-    template<typename T>
-    struct Functor2 {
-        Functor2(size_t begin, size_t end, size_t grainSize,
-                T& d, bool threaded = true) : buffer(d) {
-            if (threaded) {
-                tbb::parallel_for(tbb::blocked_range<size_t>(begin, end, grainSize), *this);
-            } else {
-                (*this)(tbb::blocked_range<size_t>(begin, end));
-            }
+
+    // Multi-threading ArrayT::ValueBuffer::push_back
+    template<typename ArrayT>
+    struct BufferPushBack {
+        BufferPushBack(ArrayT& array) : mBuffer(array) {}
+        void parallel(size_t size) {
+            tbb::parallel_for(RangeT(size_t(0), size, mBuffer.parent().pageSize()), *this);
         }
-        void operator()(const tbb::blocked_range<size_t>& r) const {
+        void serial(size_t size) { (*this)(RangeT(size_t(0), size)); }
+        void operator()(const RangeT& r) const {
+            for (size_t i=r.begin(), n=r.end(); i!=n; ++i) mBuffer.push_back(i);
+        }
+        mutable typename ArrayT::ValueBuffer mBuffer;//local instance
+    };
+
+    // Thread Local Storage version of BufferPushBack
+    template<typename ArrayT>
+    struct TLS_BufferPushBack {
+        using PoolT = tbb::enumerable_thread_specific<typename ArrayT::ValueBuffer>;
+        TLS_BufferPushBack(ArrayT &array) : mArray(&array), mPool(nullptr) {}
+        void parallel(size_t size) {
+            typename ArrayT::ValueBuffer exemplar(*mArray);//dummy used for initialization
+            mPool = new PoolT(exemplar);//thread local storage pool of ValueBuffers
+            tbb::parallel_for(RangeT(size_t(0), size, mArray->pageSize()), *this);
+            for (auto i=mPool->begin(); i!=mPool->end(); ++i) i->flush();
+            delete mPool;
+        }
+        void operator()(const RangeT& r) const {
+            typename PoolT::reference buffer = mPool->local();
             for (size_t i=r.begin(), n=r.end(); i!=n; ++i) buffer.push_back(i);
         }
-        mutable typename T::ValueBuffer buffer;
+        ArrayT *mArray;
+        PoolT  *mPool;
     };
-    // Thread Local Storage version
-    template<typename T>
-    struct Functor3 {
-        Functor3(size_t begin, size_t end, size_t grainSize,
-                T& _pool, bool threaded = true) : pool(&_pool) {
-            if (threaded) {
-                tbb::parallel_for(tbb::blocked_range<size_t>(begin, end, grainSize), *this);
-            } else {
-                (*this)(tbb::blocked_range<size_t>(begin, end));
-            }
-        }
-        void operator()(const tbb::blocked_range<size_t>& r) const {
-            typename T::reference buffer = pool->local();
-            for (size_t i=r.begin(), n=r.end(); i!=n; ++i) buffer.push_back(i);
-        }
-        T* pool;
-    };
-    
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestUtil);
 
+
 void
 TestUtil::testCpuTimer()
 {
-    const int expected = 259, tolerance = 10;//milliseconds
+    const int expected = 259, tolerance = 20;//milliseconds
     const tbb::tick_count::interval_t sec(expected/1000.0);
-    
+
     openvdb::util::CpuTimer timer;
     tbb::this_tbb_thread::sleep(sec);
     const int actual1 = static_cast<int>(timer.delta());
@@ -129,10 +130,10 @@ TestUtil::testCpuTimer()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(2*expected, actual2, tolerance);
 }
 
+
 void
 TestUtil::testPagedArray()
 {
-    
 #ifdef BENCHMARK_PAGED_ARRAY
     const size_t problemSize = 2560000;
     openvdb::util::CpuTimer timer;
@@ -140,8 +141,8 @@ TestUtil::testPagedArray()
 #else
     const size_t problemSize = 256000;
 #endif
-    
-    {//serial PagedArray::push_back (manual)
+
+    {//serial PagedArray::push_back (check return value)
         openvdb::util::PagedArray<int, size_t(8)> d;
         CPPUNIT_ASSERT(d.isEmpty());
         CPPUNIT_ASSERT_EQUAL(size_t(0), d.size());
@@ -149,7 +150,7 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL(size_t(1)<<d.log2PageSize(), d.pageSize());
         CPPUNIT_ASSERT_EQUAL(size_t(0), d.pageCount());
         CPPUNIT_ASSERT_EQUAL(size_t(0), d.capacity());
-        
+
         CPPUNIT_ASSERT_EQUAL(size_t(0), d.push_back(10));
         CPPUNIT_ASSERT_EQUAL(10, d[0]);
         CPPUNIT_ASSERT(!d.isEmpty());
@@ -157,13 +158,6 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL(size_t(1), d.pageCount());
         CPPUNIT_ASSERT_EQUAL(d.pageSize(), d.capacity());
 
-        CPPUNIT_ASSERT_EQUAL(10, d.pop_back());
-        CPPUNIT_ASSERT(d.isEmpty());
-        CPPUNIT_ASSERT_EQUAL(size_t(0), d.size());
-        CPPUNIT_ASSERT_EQUAL(size_t(1), d.pageCount());
-        CPPUNIT_ASSERT_EQUAL(d.pageSize(), d.capacity());
-        CPPUNIT_ASSERT_EQUAL(size_t(0), d.push_back(10));
-        
         CPPUNIT_ASSERT_EQUAL(size_t(1), d.push_back(1));
         CPPUNIT_ASSERT_EQUAL(size_t(2), d.size());
         CPPUNIT_ASSERT_EQUAL(size_t(1), d.pageCount());
@@ -181,13 +175,17 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL(size_t(2), d.pageCount());
         CPPUNIT_ASSERT_EQUAL(2*d.pageSize(), d.capacity());
     }
-    {//serial PagedArray::push_back (using Functor)
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
+    {//serial PagedArray::push_back
+        using ArrayT = openvdb::util::PagedArray<size_t>;
         ArrayT d;
 #ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("Serial test 1: Default page size");
+        timer.start("1: Serial PagedArray::push_back with default page size");
 #endif
-        Functor<ArrayT> t(0, problemSize, ArrayT::pageSize(), d, false);
+        {// for some reason this:
+            ArrayPushBack<ArrayT> tmp(d);
+            tmp.serial(problemSize);
+        }// is faster than:
+        //for (size_t i=0; i<problemSize; ++i) d.push_back(i);
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
@@ -199,13 +197,37 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
         for (size_t i=0, n=d.size(); i<n; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
     }
-    {//parallel PagedArray::push_back
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
+    {//serial PagedArray::push_back_unsafe
+        using ArrayT = openvdb::util::PagedArray<size_t>;
         ArrayT d;
 #ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel test 1: Default page size");
+        timer.start("2: Serial PagedArray::push_back_unsafe with default page size");
 #endif
-        Functor<ArrayT> t(0, problemSize, ArrayT::pageSize(), d);
+        {// for some reason this:
+            ArrayPushBack<ArrayT> tmp(d);
+            tmp.unsafe(problemSize);
+        }// is faster than:
+        //openvdb::util::PagedArray<size_t> d;
+        //for (size_t i=0; i<problemSize; ++i) d.push_back_unsafe(i);
+#ifdef BENCHMARK_PAGED_ARRAY
+        timer.stop();
+#endif
+        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
+        for (size_t i=0; i<problemSize; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
+    }
+    {//parallel PagedArray::push_back
+        using ArrayT = openvdb::util::PagedArray<size_t>;
+        ArrayT d;
+#ifdef BENCHMARK_PAGED_ARRAY
+        timer.start("3: Parallel PagedArray::push_back with default page size");
+#endif
+        //{// for some reason this:
+        //    ArrayPushBack<ArrayT> tmp(d);
+        //    tmp.parallel(problemSize);
+        //}// is faster than:
+        tbb::parallel_for(tbb::blocked_range<size_t>(0, problemSize, d.pageSize()),
+                          [&d](const tbb::blocked_range<size_t> &range){
+                          for (size_t i=range.begin(); i!=range.end(); ++i) d.push_back(i);});
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
@@ -234,13 +256,19 @@ TestUtil::testPagedArray()
 
 
     }
-    {//parallel PagedArray::push_back
-        typedef openvdb::util::PagedArray<size_t, size_t(3)> ArrayT;
+    {//parallel PagedArray::push_back with page size of only 8
+        using ArrayT = openvdb::util::PagedArray<size_t, size_t(3)>;
         ArrayT d;
 #ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel test 1: Page size of only 8");
+        timer.start("4: Parallel PagedArray::push_back with page size of only 8");
 #endif
-        Functor<ArrayT> t(0, problemSize, ArrayT::pageSize(), d);
+        {// for some reason this:
+            ArrayPushBack<ArrayT> tmp(d);
+            tmp.parallel(problemSize);
+        }// is faster than:
+        //tbb::parallel_for(tbb::blocked_range<size_t>(0, problemSize, d.pageSize()),
+        //                  [&d](const tbb::blocked_range<size_t> &range){
+        //                  for (size_t i=range.begin(); i!=range.end(); ++i) d.push_back(i);});
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
@@ -250,7 +278,7 @@ TestUtil::testPagedArray()
         // pageCount - 1 = max index >> log2PageSize
         CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
         CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
-        
+
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.start("parallel sort with a page size of only 8");
 #endif
@@ -269,7 +297,7 @@ TestUtil::testPagedArray()
     }
 #ifdef BENCHMARK_PAGED_ARRAY
     {//benchmark against a std::vector
-        timer.start("std::vector");
+        timer.start("5: Serial std::vector::push_back");
         std::vector<size_t> v;
         for (size_t i=0; i<problemSize; ++i) v.push_back(i);
         timer.stop();
@@ -277,7 +305,7 @@ TestUtil::testPagedArray()
         for (size_t i=0; i<problemSize; ++i) CPPUNIT_ASSERT_EQUAL(i, v[i]);
     }
     {//benchmark against a std::deque
-        timer.start("std::deque");
+        timer.start("6: Serial std::deque::push_back");
         std::deque<size_t> d;
         for (size_t i=0; i<problemSize; ++i) d.push_back(i);
         timer.stop();
@@ -291,7 +319,7 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL(size_t(1234), d2.size());
     }
     {//benchmark against a tbb::concurrent_vector::push_back
-        timer.start("serial tbb::concurrent_vector::push_back");
+        timer.start("7: Serial tbb::concurrent_vector::push_back");
         tbb::concurrent_vector<size_t> v;
         for (size_t i=0; i<problemSize; ++i) v.push_back(i);
         timer.stop();
@@ -299,38 +327,19 @@ TestUtil::testPagedArray()
         for (size_t i=0; i<problemSize; ++i) CPPUNIT_ASSERT_EQUAL(i, v[i]);
 
         v.clear();
-        timer.start("parallel tbb::concurrent_vector::push_back");
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
-        Functor<tbb::concurrent_vector<size_t> > t(0, problemSize, ArrayT::pageSize(), v);
+        timer.start("8: Parallel tbb::concurrent_vector::push_back");
+        using ArrayT = openvdb::util::PagedArray<size_t>;
+        tbb::parallel_for(tbb::blocked_range<size_t>(0, problemSize, ArrayT::pageSize()),
+                          [&v](const tbb::blocked_range<size_t> &range){
+                          for (size_t i=range.begin(); i!=range.end(); ++i) v.push_back(i);});
         timer.stop();
+        tbb::parallel_sort(v.begin(), v.end());
+        for (size_t i=0; i<problemSize; ++i) CPPUNIT_ASSERT_EQUAL(i, v[i]);
     }
 #endif
-    {//benchmark against a PagedArray::push_back
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("serial PagedArray::push_back");
-#endif
-        openvdb::util::PagedArray<size_t> d;
-        for (size_t i=0; i<problemSize; ++i) d.push_back(i);
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
-        for (size_t i=0; i<problemSize; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
-    }
-    {//benchmark against a PagedArray::push_back_unsafe
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("serial PagedArray::push_back_unsafe");
-#endif
-        openvdb::util::PagedArray<size_t, 10> d;
-        for (size_t i=0; i<problemSize; ++i) d.push_back_unsafe(i);
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
-        for (size_t i=0; i<problemSize; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
-    }
-    {//parallel push_back with resize and fill
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
+
+    {//serial PagedArray::ValueBuffer::push_back
+        using ArrayT = openvdb::util::PagedArray<size_t>;
         ArrayT d;
 
         CPPUNIT_ASSERT_EQUAL(size_t(0), d.size());
@@ -342,24 +351,59 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
 
         d.clear();
+        CPPUNIT_ASSERT_EQUAL(size_t(0), d.size());
 #ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("serial PagedArray::push_back");
+        timer.start("9: Serial PagedArray::ValueBuffer::push_back");
 #endif
-        Functor<ArrayT> t(0, problemSize, ArrayT::pageSize(), d, false);
+        {// for some reason this:
+            BufferPushBack<ArrayT> tmp(d);
+            tmp.serial(problemSize);
+            // is faster than:
+            //ArrayT::ValueBuffer buffer(d);
+            //for (size_t i=0, n=problemSize; i<n; ++i) buffer.push_back(i);
+        }
+
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
+        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
+        for (size_t i=0; i<problemSize; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
+
+        size_t unsorted = 0;
+        for (size_t i=0, n=d.size(); i<n; ++i) unsorted += i != d[i];
+        CPPUNIT_ASSERT_EQUAL(size_t(0), unsorted);
+
+#ifdef BENCHMARK_PAGED_ARRAY
+        timer.start("parallel sort");
+#endif
+        d.sort();
+#ifdef BENCHMARK_PAGED_ARRAY
+        timer.stop();
+#endif
+        for (size_t i=0, n=d.size(); i<n; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
+
 
         CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
         CPPUNIT_ASSERT_EQUAL(size_t(1)<<d.log2PageSize(), d.pageSize());
         CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
         CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
-        
-        d.clear();
+
+
+    }
+    {//parallel PagedArray::ValueBuffer::push_back
+        using ArrayT = openvdb::util::PagedArray<size_t>;
+        ArrayT d;
 #ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel PagedArray::push_back");
+        timer.start("10: Parallel PagedArray::ValueBuffer::push_back");
 #endif
-        Functor<ArrayT> tmp1(0, problemSize, ArrayT::pageSize(), d, true);
+        {// for some reason this:
+            BufferPushBack<ArrayT> tmp(d);
+            tmp.parallel(problemSize);
+        }// is faster than:
+        //tbb::parallel_for(tbb::blocked_range<size_t>(0, problemSize, d.pageSize()),
+        //                  [&d](const tbb::blocked_range<size_t> &r){
+        //                  typename ArrayT::ValueBuffer buffer(d);
+        //                  for (size_t i=r.begin(), n=r.end(); i!=n; ++i) buffer.push_back(i);});
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
@@ -369,33 +413,24 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
         CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
 
-        d.clear();
+        // Test sorting
 #ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("serial PagedArray::ValueBuffer::push_back");
+        timer.start("parallel sort");
 #endif
-        Functor2<ArrayT> tmp2(0, problemSize, ArrayT::pageSize(), d, false);
+        d.sort();
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
+        for (size_t i=0; i<d.size(); ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
 
-        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
-        CPPUNIT_ASSERT_EQUAL(size_t(1)<<d.log2PageSize(), d.pageSize());
-        CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
-        CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
-
-        d.clear();
 #ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel PagedArray::ValueBuffer::push_back");
+        timer.start("parallel inverse sort");
 #endif
-        Functor2<ArrayT> tmp3(0, problemSize, ArrayT::pageSize(), d, true);
+        d.invSort();
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
-        
-        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
-        CPPUNIT_ASSERT_EQUAL(size_t(1)<<d.log2PageSize(), d.pageSize());
-        CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
-        CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
+        for (size_t i=0, n=d.size()-1; i<=n; ++i) CPPUNIT_ASSERT_EQUAL(n-i, d[i]);
 
         CPPUNIT_ASSERT_EQUAL(problemSize, d.push_back(1));
         CPPUNIT_ASSERT_EQUAL(problemSize+1, d.size());
@@ -404,12 +439,13 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL(size_t(1)+(problemSize>>d.log2PageSize()), d.pageCount());
         CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
 
+        // test PagedArray::fill
         const size_t v = 13;
         d.fill(v);
         for (size_t i=0, n=d.capacity(); i<n; ++i) CPPUNIT_ASSERT_EQUAL(v, d[i]);
     }
-    {//serial PagedArray::ValueBuffer::push_back
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
+    {//test PagedArray::ValueBuffer::flush
+        using ArrayT = openvdb::util::PagedArray<size_t>;
         ArrayT d;
         CPPUNIT_ASSERT_EQUAL(size_t(0), d.size());
         {
@@ -426,115 +462,24 @@ TestUtil::testPagedArray()
         CPPUNIT_ASSERT_EQUAL(size_t(1), d[0]);
         CPPUNIT_ASSERT_EQUAL(size_t(2), d[1]);
     }
-    {//parallel PagedArray::push_back followed by parallel sort
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
-        ArrayT d;
-
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel PagedArray::push_back");
-#endif
-        Functor<ArrayT> t(0, problemSize, ArrayT::pageSize(), d);
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
-        CPPUNIT_ASSERT_EQUAL(size_t(1)<<d.log2PageSize(), d.pageSize());
-        CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
-        CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
-
-        // Not guaranteed to pass
-        //size_t unsorted = 0;
-        //for (size_t i=0, n=d.size(); i<n; ++i) unsorted += i != d[i];
-        //CPPUNIT_ASSERT( unsorted > 0 );
-
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel sort");
-#endif
-        d.sort();
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        for (size_t i=0, n=d.size(); i<n; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
-    }
-    {//serial PagedArray::ValueBuffer::push_back followed by parallel sort
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
-        ArrayT d;
-
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("serial PagedArray::ValueBuffer::push_back");
-#endif
-        Functor2<ArrayT> t(0, problemSize, ArrayT::pageSize(), d, false);
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
-        CPPUNIT_ASSERT_EQUAL(size_t(1)<<d.log2PageSize(), d.pageSize());
-        CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
-        CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
-        size_t unsorted = 0;
-        for (size_t i=0, n=d.size(); i<n; ++i) unsorted += i != d[i];
-        CPPUNIT_ASSERT_EQUAL(size_t(0), unsorted);
-
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel sort");
-#endif
-        d.sort();
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        for (size_t i=0, n=d.size(); i<n; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
-    }
-    {//parallel PagedArray::ValueBuffer::push_back followed by parallel sort
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
-        ArrayT d;
-
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel PagedArray::ValueBuffer::push_back");
-#endif
-        Functor2<ArrayT> t(0, problemSize, ArrayT::pageSize(), d, true);
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
-        CPPUNIT_ASSERT_EQUAL(size_t(1)<<d.log2PageSize(), d.pageSize());
-        CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
-        CPPUNIT_ASSERT_EQUAL(d.pageCount()*d.pageSize(), d.capacity());
-
-        // Not guaranteed to pass
-        //size_t unsorted = 0;
-        //for (size_t i=0, n=d.size(); i<n; ++i) unsorted += i != d[i];
-        //CPPUNIT_ASSERT( unsorted > 0 );
-
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel sort");
-#endif
-        d.sort();
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        for (size_t i=0, n=d.size(); i<n; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
-
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel inverse sort");
-#endif
-        d.invSort();
-#ifdef BENCHMARK_PAGED_ARRAY
-        timer.stop();
-#endif
-        for (size_t i=0, n=d.size()-1; i<=n; ++i) CPPUNIT_ASSERT_EQUAL(n-i, d[i]);
-    }
     {//thread-local-storage PagedArray::ValueBuffer::push_back followed by parallel sort
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
-        ArrayT d;        
+        using ArrayT = openvdb::util::PagedArray<size_t>;
+        ArrayT d;
 
 #ifdef BENCHMARK_PAGED_ARRAY
-        timer.start("parallel TLS PagedArray::ValueBuffer::push_back");
+        timer.start("11: Parallel TLS PagedArray::ValueBuffer::push_back");
 #endif
-        ArrayT::ValueBuffer tmp(d);
-        typedef tbb::enumerable_thread_specific<ArrayT::ValueBuffer> BufferT;
-        BufferT buffer(tmp);
-        Functor3<BufferT> t(0, problemSize, ArrayT::pageSize(), buffer, true);
-        for (BufferT::iterator i = buffer.begin(); i != buffer.end(); ++i) i->flush();
+        {// for some reason this:
+            TLS_BufferPushBack<ArrayT> tmp(d);
+            tmp.parallel(problemSize);
+        }// is faster than:
+        //ArrayT::ValueBuffer exemplar(d);//dummy used for initialization
+        ///tbb::enumerable_thread_specific<ArrayT::ValueBuffer> pool(exemplar);//thread local storage pool of ValueBuffers
+        //tbb::parallel_for(tbb::blocked_range<size_t>(0, problemSize, d.pageSize()),
+        //                  [&pool](const tbb::blocked_range<size_t> &range){
+        //                  ArrayT::ValueBuffer &buffer = pool.local();
+        //                  for (size_t i=range.begin(); i!=range.end(); ++i) buffer.push_back(i);});
+        //for (auto i=pool.begin(); i!=pool.end(); ++i) i->flush();
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
@@ -557,13 +502,16 @@ TestUtil::testPagedArray()
 #ifdef BENCHMARK_PAGED_ARRAY
         timer.stop();
 #endif
-        for (size_t i=0, n=d.size(); i<n; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);        
+        for (size_t i=0, n=d.size(); i<n; ++i) CPPUNIT_ASSERT_EQUAL(i, d[i]);
     }
     {//parallel PagedArray::merge followed by parallel sort
-        typedef openvdb::util::PagedArray<size_t> ArrayT;
+        using ArrayT = openvdb::util::PagedArray<size_t>;
         ArrayT d, d2;
-        
-        Functor2<ArrayT> t1(0, problemSize, ArrayT::pageSize(), d, true);
+
+        tbb::parallel_for(tbb::blocked_range<size_t>(0, problemSize, d.pageSize()),
+                          [&d](const tbb::blocked_range<size_t> &range){
+                          ArrayT::ValueBuffer buffer(d);
+                          for (size_t i=range.begin(); i!=range.end(); ++i) buffer.push_back(i);});
         CPPUNIT_ASSERT_EQUAL(problemSize, d.size());
         CPPUNIT_ASSERT_EQUAL(size_t(1)<<d.log2PageSize(), d.pageSize());
         CPPUNIT_ASSERT_EQUAL((d.size()-1)>>d.log2PageSize(), d.pageCount()-1);
@@ -572,7 +520,10 @@ TestUtil::testPagedArray()
         d.push_back(problemSize);
         CPPUNIT_ASSERT(d.isPartiallyFull());
 
-        Functor2<ArrayT> t2(problemSize+1, 2*problemSize+1, ArrayT::pageSize(), d2, true);
+        tbb::parallel_for(tbb::blocked_range<size_t>(problemSize+1, 2*problemSize+1, d2.pageSize()),
+                          [&d2](const tbb::blocked_range<size_t> &range){
+                          ArrayT::ValueBuffer buffer(d2);
+                          for (size_t i=range.begin(); i!=range.end(); ++i) buffer.push_back(i);});
         //for (size_t i=d.size(), n=i+problemSize; i<n; ++i) d2.push_back(i);
         CPPUNIT_ASSERT(!d2.isPartiallyFull());
         CPPUNIT_ASSERT_EQUAL(problemSize, d2.size());
@@ -590,7 +541,7 @@ TestUtil::testPagedArray()
         timer.stop();
 #endif
         CPPUNIT_ASSERT(d.isPartiallyFull());
-        
+
         //d.print();
         //d2.print();
         CPPUNIT_ASSERT_EQUAL(2*problemSize+1, d.size());
@@ -614,17 +565,17 @@ TestUtil::testPagedArray()
             for (int i=0; i<100000; ++i) CPPUNIT_ASSERT_EQUAL(i, array[i]);
         }
         {//2A
-            openvdb::util::PagedArray<int> array;    
+            openvdb::util::PagedArray<int> array;
             openvdb::util::PagedArray<int>::ValueBuffer buffer(array);
-            for (int i=0; i<100000; ++i) buffer.push_back(i);    
+            for (int i=0; i<100000; ++i) buffer.push_back(i);
             buffer.flush();
             for (int i=0; i<100000; ++i) CPPUNIT_ASSERT_EQUAL(i, array[i]);
         }
         {//2B
             openvdb::util::PagedArray<int> array;
-            {//local scope of a single thread   
+            {//local scope of a single thread
                 openvdb::util::PagedArray<int>::ValueBuffer buffer(array);
-                for (int i=0; i<100000; ++i) buffer.push_back(i);    
+                for (int i=0; i<100000; ++i) buffer.push_back(i);
             }
             for (int i=0; i<100000; ++i) CPPUNIT_ASSERT_EQUAL(i, array[i]);
         }
@@ -635,7 +586,7 @@ TestUtil::testPagedArray()
             for (int i=0; i<100000; ++i) CPPUNIT_ASSERT_EQUAL(i, array[i]);
         }
         {//3B
-            typedef openvdb::util::PagedArray<int> ArrayT;
+            using ArrayT = openvdb::util::PagedArray<int>;
             ArrayT array;
             array.resize(100000);
             for (ArrayT::Iterator i=array.begin(); i!=array.end(); ++i) *i = int(i.pos());
@@ -644,6 +595,6 @@ TestUtil::testPagedArray()
     }
 }
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
