@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -35,6 +35,7 @@
 #ifndef OPENVDB_UTIL_NODEMASKS_HAS_BEEN_INCLUDED
 #define OPENVDB_UTIL_NODEMASKS_HAS_BEEN_INCLUDED
 
+#include <algorithm> // for std::min()
 #include <cassert>
 #include <cstring>
 #include <iostream>// for cout
@@ -43,6 +44,7 @@
 //#include <boost/mpl/if.hpp>
 //#include <strings.h> // for ffs
 
+
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
@@ -56,13 +58,17 @@ CountOn(Byte v)
 #ifndef _MSC_VER // Visual C++ doesn't guarantee thread-safe initialization of local statics
     static
 #endif
+    /// @todo Move this table and others into, say, Util.cc
     const Byte numBits[256] = {
-#   define B2(n)  n,     n+1,     n+1,     n+2
-#   define B4(n)  B2(n), B2(n+1), B2(n+1), B2(n+2)
-#   define B6(n)  B4(n), B4(n+1), B4(n+1), B4(n+2)
-           B6(0), B6(1), B6(1),   B6(2)
+#define COUNTONB2(n)  n,            n+1,            n+1,            n+2
+#define COUNTONB4(n)  COUNTONB2(n), COUNTONB2(n+1), COUNTONB2(n+1), COUNTONB2(n+2)
+#define COUNTONB6(n)  COUNTONB4(n), COUNTONB4(n+1), COUNTONB4(n+1), COUNTONB4(n+2)
+        COUNTONB6(0), COUNTONB6(1), COUNTONB6(1),   COUNTONB6(2)
     };
     return numBits[v];
+#undef COUNTONB6
+#undef COUNTONB4
+#undef COUNTONB2
 
     // Sequentially clear least significant bits
     //Index32 c;
@@ -72,6 +78,7 @@ CountOn(Byte v)
     // This version is only fast on CPUs with fast "%" and "*" operations
     //return (v * UINT64_C(0x200040008001) & UINT64_C(0x111111111111111)) % 0xF;
 }
+
 /// Return the number of off bits in the given 8-bit value.
 inline Index32 CountOff(Byte v) { return CountOn(static_cast<Byte>(~v)); }
 
@@ -170,17 +177,19 @@ FindHighestOn(Index32 v)
 
 
 /// Base class for the bit mask iterators
-template <typename NodeMask>
+template<typename NodeMask>
 class BaseMaskIterator
 {
 protected:
-    Index32          mPos;//bit position
-    const NodeMask*  mParent;//this iterator can't change the parent_mask!
+    Index32 mPos; // bit position
+    const NodeMask* mParent; // this iterator can't change the parent_mask!
+
 public:
-    BaseMaskIterator() : mPos(NodeMask::SIZE), mParent(NULL) {}
-    BaseMaskIterator(Index32 pos,const NodeMask *parent) : mPos(pos), mParent(parent)
+    BaseMaskIterator(): mPos(NodeMask::SIZE), mParent(nullptr) {}
+    BaseMaskIterator(const BaseMaskIterator&) = default;
+    BaseMaskIterator(Index32 pos, const NodeMask* parent): mPos(pos), mParent(parent)
     {
-        assert( (parent==NULL && pos==0 ) ||  (parent!=NULL && pos<=NodeMask::SIZE) );
+        assert((parent == nullptr && pos == 0) || (parent != nullptr && pos <= NodeMask::SIZE));
     }
     bool operator==(const BaseMaskIterator &iter) const {return mPos == iter.mPos;}
     bool operator!=(const BaseMaskIterator &iter) const {return mPos != iter.mPos;}
@@ -189,14 +198,10 @@ public:
     {
         mPos = iter.mPos; mParent = iter.mParent; return *this;
     }
-    Index32 offset() const {return mPos;}
-    Index32 pos() const {return mPos;}
-    bool test() const
-    {
-        assert(mPos  <= NodeMask::SIZE);
-        return (mPos != NodeMask::SIZE);
-    }
-    operator bool() const {return this->test();}
+    Index32 offset() const { return mPos; }
+    Index32 pos() const { return mPos; }
+    bool test() const { assert(mPos <= NodeMask::SIZE); return (mPos != NodeMask::SIZE); }
+    operator bool() const { return this->test(); }
 }; // class BaseMaskIterator
 
 
@@ -205,7 +210,7 @@ template <typename NodeMask>
 class OnMaskIterator: public BaseMaskIterator<NodeMask>
 {
 private:
-    typedef BaseMaskIterator<NodeMask> BaseType;
+    using BaseType = BaseMaskIterator<NodeMask>;
     using BaseType::mPos;//bit position;
     using BaseType::mParent;//this iterator can't change the parent_mask!
 public:
@@ -213,7 +218,7 @@ public:
     OnMaskIterator(Index32 pos,const NodeMask *parent) : BaseType(pos,parent) {}
     void increment()
     {
-        assert(mParent != NULL);
+        assert(mParent != nullptr);
         mPos = mParent->findNextOn(mPos+1);
         assert(mPos <= NodeMask::SIZE);
     }
@@ -236,7 +241,7 @@ template <typename NodeMask>
 class OffMaskIterator: public BaseMaskIterator<NodeMask>
 {
 private:
-    typedef BaseMaskIterator<NodeMask> BaseType;
+    using BaseType = BaseMaskIterator<NodeMask>;
     using BaseType::mPos;//bit position;
     using BaseType::mParent;//this iterator can't change the parent_mask!
 public:
@@ -244,7 +249,7 @@ public:
     OffMaskIterator(Index32 pos,const NodeMask *parent) : BaseType(pos,parent) {}
     void increment()
     {
-        assert(mParent != NULL);
+        assert(mParent != nullptr);
         mPos=mParent->findNextOff(mPos+1);
         assert(mPos <= NodeMask::SIZE);
     }
@@ -267,7 +272,7 @@ template <typename NodeMask>
 class DenseMaskIterator: public BaseMaskIterator<NodeMask>
 {
 private:
-    typedef BaseMaskIterator<NodeMask> BaseType;
+    using BaseType = BaseMaskIterator<NodeMask>;
     using BaseType::mPos;//bit position;
     using BaseType::mParent;//this iterator can't change the parent_mask!
 
@@ -276,7 +281,7 @@ public:
     DenseMaskIterator(Index32 pos,const NodeMask *parent) : BaseType(pos,parent) {}
     void increment()
     {
-        assert(mParent != NULL);
+        assert(mParent != nullptr);
         mPos += 1;//careful - the increment might go beyond the end
         assert(mPos<= NodeMask::SIZE);
     }
@@ -304,13 +309,13 @@ template<Index Log2Dim>
 class NodeMask
 {
 public:
-    BOOST_STATIC_ASSERT( Log2Dim>2 );
+    static_assert(Log2Dim > 2, "expected NodeMask template specialization, got base template");
 
     static const Index32 LOG2DIM    = Log2Dim;
     static const Index32 DIM        = 1<<Log2Dim;
     static const Index32 SIZE       = 1<<3*Log2Dim;
     static const Index32 WORD_COUNT = SIZE >> 6;// 2^6=64
-    typedef Index64 Word;
+    using Word = Index64;
 
 private:
 
@@ -320,7 +325,7 @@ private:
     //static const Index32 BIT_MASK   = sizeof(void*) == 8 ? 63 : 31;
     //static const Index32 LOG2WORD   = BIT_MASK == 63 ? 6 : 5;
     //static const Index32 WORD_COUNT = SIZE >> LOG2WORD;
-    //typedef boost::mpl::if_c<BIT_MASK == 63, Index64, Index32>::type Word;
+    //using Word = boost::mpl::if_c<BIT_MASK == 63, Index64, Index32>::type;
 
     Word mWords[WORD_COUNT];//only member data!
 
@@ -342,9 +347,9 @@ public:
         return *this;
     }
 
-    typedef OnMaskIterator<NodeMask>    OnIterator;
-    typedef OffMaskIterator<NodeMask>   OffIterator;
-    typedef DenseMaskIterator<NodeMask> DenseIterator;
+    using OnIterator = OnMaskIterator<NodeMask>;
+    using OffIterator = OffMaskIterator<NodeMask>;
+    using DenseIterator = DenseMaskIterator<NodeMask>;
 
     OnIterator beginOn() const       { return OnIterator(this->findFirstOn(),this); }
     OnIterator endOn() const         { return OnIterator(SIZE,this); }
@@ -365,7 +370,7 @@ public:
     //
     // Bitwise logical operations
     //
-    
+
     /// @brief Apply a functor to the words of the this and the other mask.
     ///
     /// @details An example that implements the "operator&=" method:
@@ -405,7 +410,7 @@ public:
         for (Index32 n = WORD_COUNT; n--;  ++w1, ++w2) *w1 &= *w2;
         return *this;
     }
-    /// @brief Bitwise union 
+    /// @brief Bitwise union
     const NodeMask& operator|=(const NodeMask& other)
     {
         Word *w1 = mWords;
@@ -433,7 +438,7 @@ public:
     NodeMask operator&(const NodeMask& other) const { NodeMask m(*this); m &= other; return m; }
     NodeMask operator|(const NodeMask& other) const { NodeMask m(*this); m |= other; return m; }
     NodeMask operator^(const NodeMask& other) const { NodeMask m(*this); m ^= other; return m; }
-   
+
     /// Return the byte size of this NodeMask
     static Index32 memUsage() { return static_cast<Index32>(WORD_COUNT*sizeof(Word)); }
     /// Return the total number of on bits
@@ -517,6 +522,17 @@ public:
         for (const Word *w = mWords; n-- && *w++ == Word(0);) ;
         return n == -1;
     }
+    /// Return @c true if bits are either all off OR all on.
+    /// @param isOn Takes on the values of all bits if the method
+    /// returns true - else it is undefined.
+    bool isConstant(bool &isOn) const
+    {
+        isOn = (mWords[0] == ~Word(0));//first word has all bits on
+        if ( !isOn && mWords[0] != Word(0)) return false;//early out
+        const Word *w = mWords + 1, *n = mWords + WORD_COUNT;
+        while( w<n && *w == mWords[0] ) ++w;
+        return w == n;
+    }
     Index32 findFirstOn() const
     {
         Index32 n = 0;
@@ -552,9 +568,8 @@ public:
     {
         os.write(reinterpret_cast<const char*>(mWords), this->memUsage());
     }
-    void load(std::istream& is) {
-        is.read(reinterpret_cast<char*>(mWords), this->memUsage());
-    }
+    void load(std::istream& is) { is.read(reinterpret_cast<char*>(mWords), this->memUsage()); }
+    void seek(std::istream& is) const { is.seekg(this->memUsage(), std::ios_base::cur); }
     /// @brief simple print method for debugging
     void printInfo(std::ostream& os=std::cout) const
     {
@@ -615,7 +630,7 @@ public:
     static const Index32 DIM        = 2;
     static const Index32 SIZE       = 8;
     static const Index32 WORD_COUNT = 1;
-    typedef Byte Word;
+    using Word = Byte;
 
 private:
 
@@ -633,9 +648,9 @@ public:
     /// Assignment operator
     void operator = (const NodeMask &other) { mByte = other.mByte; }
 
-    typedef OnMaskIterator<NodeMask>    OnIterator;
-    typedef OffMaskIterator<NodeMask>   OffIterator;
-    typedef DenseMaskIterator<NodeMask> DenseIterator;
+    using OnIterator = OnMaskIterator<NodeMask>;
+    using OffIterator = OffMaskIterator<NodeMask>;
+    using DenseIterator = DenseMaskIterator<NodeMask>;
 
     OnIterator beginOn() const       { return OnIterator(this->findFirstOn(),this); }
     OnIterator endOn() const         { return OnIterator(SIZE,this); }
@@ -651,7 +666,7 @@ public:
     //
     // Bitwise logical operations
     //
-    
+
     /// @brief Apply a functor to the words of the this and the other mask.
     ///
     /// @details An example that implements the "operator&=" method:
@@ -683,13 +698,13 @@ public:
         mByte &= other.mByte;
         return *this;
     }
-    /// @brief Bitwise union 
+    /// @brief Bitwise union
     const NodeMask& operator|=(const NodeMask& other)
     {
         mByte |= other.mByte;
         return *this;
     }
-    /// @brief Bitwise difference 
+    /// @brief Bitwise difference
     const NodeMask& operator-=(const NodeMask& other)
     {
         mByte &= static_cast<Byte>(~other.mByte);
@@ -756,6 +771,14 @@ public:
     bool isOn() const { return mByte == 0xFFU; }
     /// Return true if all the bits are off
     bool isOff() const { return mByte == 0; }
+    /// Return @c true if bits are either all off OR all on.
+    /// @param isOn Takes on the values of all bits if the method
+    /// returns true - else it is undefined.
+    bool isConstant(bool &isOn) const
+    {
+        isOn = this->isOn();
+        return isOn || this->isOff();
+    }
     Index32 findFirstOn() const { return mByte ? FindLowestOn(mByte) : 8; }
     Index32 findFirstOff() const
     {
@@ -769,24 +792,22 @@ public:
     template<typename WordT>
     WordT getWord(Index n) const
     {
-        BOOST_STATIC_ASSERT(sizeof(WordT) == sizeof(Byte));
+        static_assert(sizeof(WordT) == sizeof(Byte), "expected word size to be one byte");
         assert(n == 0);
         return reinterpret_cast<WordT>(mByte);
     }
     template<typename WordT>
     WordT& getWord(Index n)
     {
-        BOOST_STATIC_ASSERT(sizeof(WordT) == sizeof(Byte));
+        static_assert(sizeof(WordT) == sizeof(Byte), "expected word size to be one byte");
         assert(n == 0);
         return reinterpret_cast<WordT&>(mByte);
     }
     //@}
     */
-    void save(std::ostream& os) const
-    {
-        os.write(reinterpret_cast<const char*>(&mByte), 1);
-    }
+    void save(std::ostream& os) const { os.write(reinterpret_cast<const char*>(&mByte), 1); }
     void load(std::istream& is) { is.read(reinterpret_cast<char*>(&mByte), 1); }
+    void seek(std::istream& is) const { is.seekg(1, std::ios_base::cur); }
     /// @brief simple print method for debugging
     void printInfo(std::ostream& os=std::cout) const
     {
@@ -831,7 +852,7 @@ public:
     static const Index32 DIM        =  4;
     static const Index32 SIZE       = 64;
     static const Index32 WORD_COUNT = 1;
-    typedef Index64 Word;
+    using Word = Index64;
 
 private:
 
@@ -849,9 +870,9 @@ public:
     /// Assignment operator
     void operator = (const NodeMask &other) { mWord = other.mWord; }
 
-    typedef OnMaskIterator<NodeMask>    OnIterator;
-    typedef OffMaskIterator<NodeMask>   OffIterator;
-    typedef DenseMaskIterator<NodeMask> DenseIterator;
+    using OnIterator = OnMaskIterator<NodeMask>;
+    using OffIterator = OffMaskIterator<NodeMask>;
+    using DenseIterator = DenseMaskIterator<NodeMask>;
 
     OnIterator beginOn() const       { return OnIterator(this->findFirstOn(),this); }
     OnIterator endOn() const         { return OnIterator(SIZE,this); }
@@ -905,7 +926,7 @@ public:
         mWord |= other.mWord;
         return *this;
     }
-    /// @brief Bitwise difference 
+    /// @brief Bitwise difference
     const NodeMask& operator-=(const NodeMask& other)
     {
         mWord &= ~other.mWord;
@@ -972,6 +993,13 @@ public:
     bool isOn() const { return mWord == UINT64_C(0xFFFFFFFFFFFFFFFF); }
     /// Return true if all the bits are off
     bool isOff() const { return mWord == 0; }
+    /// Return @c true if bits are either all off OR all on.
+    /// @param isOn Takes on the values of all bits if the method
+    /// returns true - else it is undefined.
+    bool isConstant(bool &isOn) const
+    {   isOn = this->isOn();
+        return isOn || this->isOff();
+    }
     Index32 findFirstOn() const { return mWord ? FindLowestOn(mWord) : 64; }
     Index32 findFirstOff() const
     {
@@ -993,11 +1021,9 @@ public:
         return reinterpret_cast<WordT*>(mWord)[n];
     }
     //@}
-    void save(std::ostream& os) const
-    {
-        os.write(reinterpret_cast<const char*>(&mWord), 8);
-    }
+    void save(std::ostream& os) const { os.write(reinterpret_cast<const char*>(&mWord), 8); }
     void load(std::istream& is) { is.read(reinterpret_cast<char*>(&mWord), 8); }
+    void seek(std::istream& is) const { is.seekg(8, std::ios_base::cur); }
     /// @brief simple print method for debugging
     void printInfo(std::ostream& os=std::cout) const
     {
@@ -1046,7 +1072,7 @@ protected:
     Index32  *mBits;
 
 public:
-    RootNodeMask(): mBitSize(0), mIntSize(0), mBits(NULL) {}
+    RootNodeMask(): mBitSize(0), mIntSize(0), mBits(nullptr) {}
     RootNodeMask(Index32 bit_size):
         mBitSize(bit_size), mIntSize(((bit_size-1)>>5)+1), mBits(new Index32[mIntSize])
     {
@@ -1089,11 +1115,10 @@ public:
         Index32             mBitSize;
         const RootNodeMask* mParent;//this iterator can't change the parent_mask!
     public:
-        BaseIterator() : mPos(0), mBitSize(0), mParent(NULL) {}
-        BaseIterator(Index32 pos,const RootNodeMask *parent)
-            : mPos(pos), mBitSize(parent->getBitSize()), mParent(parent) {
-            assert( pos<=mBitSize );
-        }
+        BaseIterator() : mPos(0), mBitSize(0), mParent(nullptr) {}
+        BaseIterator(const BaseIterator&) = default;
+        BaseIterator(Index32 pos, const RootNodeMask* parent):
+            mPos(pos), mBitSize(parent->getBitSize()), mParent(parent) { assert(pos <= mBitSize); }
         bool operator==(const BaseIterator &iter) const {return mPos == iter.mPos;}
         bool operator!=(const BaseIterator &iter) const {return mPos != iter.mPos;}
         bool operator< (const BaseIterator &iter) const {return mPos <  iter.mPos;}
@@ -1127,7 +1152,7 @@ public:
         OnIterator() : BaseIterator() {}
         OnIterator(Index32 pos,const RootNodeMask *parent) : BaseIterator(pos,parent) {}
         void increment() {
-            assert(mParent!=NULL);
+            assert(mParent != nullptr);
             mPos=mParent->findNextOn(mPos+1);
             assert(mPos <= mBitSize);
         }
@@ -1155,7 +1180,7 @@ public:
         OffIterator() : BaseIterator()  {}
         OffIterator(Index32 pos,const RootNodeMask *parent) : BaseIterator(pos,parent) {}
         void increment() {
-            assert(mParent!=NULL);
+            assert(mParent != nullptr);
             mPos=mParent->findNextOff(mPos+1);
             assert(mPos <= mBitSize);
         }
@@ -1183,7 +1208,7 @@ public:
         DenseIterator() : BaseIterator() {}
         DenseIterator(Index32 pos,const RootNodeMask *parent) : BaseIterator(pos,parent) {}
         void increment() {
-            assert(mParent!=NULL);
+            assert(mParent != nullptr);
             mPos += 1;//carefull - the increament might go beyond the end
             assert(mPos<= mBitSize);
         }
@@ -1344,11 +1369,15 @@ public:
 
     void save(std::ostream& os) const {
         assert(mBits);
-        os.write((const char *)mBits,mIntSize*sizeof(Index32));
+        os.write(reinterpret_cast<const char*>(mBits), mIntSize * sizeof(Index32));
     }
     void load(std::istream& is) {
         assert(mBits);
-        is.read((char *)mBits,mIntSize*sizeof(Index32));
+        is.read(reinterpret_cast<char*>(mBits), mIntSize * sizeof(Index32));
+    }
+    void seek(std::istream& is) const {
+        assert(mBits);
+        is.seekg(mIntSize * sizeof(Index32), std::ios_base::cur);
     }
     /// @brief simple print method for debugging
     void printInfo(std::ostream& os=std::cout) const {
@@ -1406,6 +1435,6 @@ public:
 
 #endif // OPENVDB_UTIL_NODEMASKS_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
