@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -30,12 +30,15 @@
 
 #include "RenderModules.h"
 
+#include <openvdb/points/PointDataGrid.h>
+#include <openvdb/points/PointCount.h>
+#include <openvdb/points/PointConversion.h>
 #include <openvdb/tools/Morphology.h>
 #include <openvdb/tools/Prune.h>
 #include <openvdb/tree/LeafManager.h>
 #include <openvdb/util/logging.h>
-
-#include <math.h>
+#include <cmath> // for std::abs(), std::fabs(), std::floor()
+#include <type_traits> // for std::is_const
 
 
 namespace openvdb_viewer {
@@ -73,7 +76,7 @@ inline void
 doProcessTypedGrid(GridPtrType grid, OpType& op)
 {
     GridProcessor<GridType, OpType,
-        boost::is_const<typename GridPtrType::element_type>::value>::call(op, grid);
+        std::is_const<typename GridPtrType::element_type>::value>::call(op, grid);
 }
 
 
@@ -107,6 +110,9 @@ processTypedGrid(GridPtrType grid, OpType& op)
     else if (grid->template isType<Vec3IGrid>())  doProcessTypedGrid<Vec3IGrid>(grid, op);
     else if (grid->template isType<Vec3SGrid>())  doProcessTypedGrid<Vec3SGrid>(grid, op);
     else if (grid->template isType<Vec3DGrid>())  doProcessTypedGrid<Vec3DGrid>(grid, op);
+    else if (grid->template isType<points::PointDataGrid>()) {
+        doProcessTypedGrid<points::PointDataGrid>(grid, op);
+    }
     else return false;
     return true;
 }
@@ -115,18 +121,6 @@ processTypedGrid(GridPtrType grid, OpType& op)
 /// @brief Utility function that, given a generic grid pointer, calls
 /// a functor on the fully-resolved grid, provided that the grid's
 /// voxel values are scalars
-///
-/// Usage:
-/// @code
-/// struct PruneOp {
-///     template<typename GridT>
-///     void operator()(typename GridT::Ptr grid) const { grid->tree()->prune(); }
-/// };
-///
-/// processTypedScalarGrid(myGridPtr, PruneOp());
-/// @endcode
-///
-/// @return @c false if the grid type is unknown or non-scalar.
 template<typename GridPtrType, typename OpType>
 bool
 processTypedScalarGrid(GridPtrType grid, OpType& op)
@@ -141,6 +135,23 @@ processTypedScalarGrid(GridPtrType grid, OpType& op)
 }
 
 
+/// @brief Utility function that, given a generic grid pointer, calls
+/// a functor on the fully-resolved grid, provided that the grid's
+/// voxel values are scalars or PointIndex objects
+template<typename GridPtrType, typename OpType>
+bool
+processTypedScalarOrPointDataGrid(GridPtrType grid, OpType& op)
+{
+    using namespace openvdb;
+    if (processTypedScalarGrid(grid, op)) return true;
+    if (grid->template isType<points::PointDataGrid>()) {
+        doProcessTypedGrid<points::PointDataGrid>(grid, op);
+        return true;
+    }
+    return false;
+}
+
+
 /// @brief Utility function that, given a generic grid pointer, calls
 /// a functor on the fully-resolved grid, provided that the grid's
 /// voxel values are vectors
@@ -150,7 +161,7 @@ processTypedVectorGrid(GridPtrType grid, OpType& op)
 {
     using namespace openvdb;
     if (grid->template isType<Vec3IGrid>())       doProcessTypedGrid<Vec3IGrid>(grid, op);
-    else if (grid->template isType<Vec3SGrid>()) doProcessTypedGrid<Vec3SGrid>(grid, op);
+    else if (grid->template isType<Vec3SGrid>())  doProcessTypedGrid<Vec3SGrid>(grid, op);
     else if (grid->template isType<Vec3DGrid>())  doProcessTypedGrid<Vec3DGrid>(grid, op);
     else return false;
     return true;
@@ -160,8 +171,8 @@ template<class TreeType>
 class MinMaxVoxel
 {
 public:
-    typedef openvdb::tree::LeafManager<TreeType> LeafArray;
-    typedef typename TreeType::ValueType ValueType;
+    using LeafArray = openvdb::tree::LeafManager<TreeType>;
+    using ValueType = typename TreeType::ValueType;
 
     // LeafArray = openvdb::tree::LeafManager<TreeType> leafs(myTree)
     MinMaxVoxel(LeafArray&);
@@ -531,13 +542,13 @@ ViewportModule::render()
     float step = 0.125;
     for (float x = -1; x < 1.125; x+=step) {
 
-        if (fabs(x) == 0.5 || fabs(x) == 0.0) {
+        if (std::fabs(x) == 0.5 || std::fabs(x) == 0.0) {
             glLineWidth(1.5);
         } else {
             glLineWidth(1.0);
         }
 
-        glBegin( GL_LINES );
+        glBegin(GL_LINES);
         glVertex3f(x, 0, 1);
         glVertex3f(x, 0, -1);
         glVertex3f(1, 0, x);
@@ -821,7 +832,7 @@ template<typename TreeType>
 class PointGenerator
 {
 public:
-    typedef openvdb::tree::LeafManager<TreeType> LeafManagerType;
+    using LeafManagerType = openvdb::tree::LeafManager<TreeType>;
 
     PointGenerator(
         std::vector<GLfloat>& points,
@@ -849,7 +860,7 @@ public:
     {
         using openvdb::Index64;
 
-        typedef typename TreeType::LeafNodeType::ValueOnCIter ValueOnCIter;
+        using ValueOnCIter = typename TreeType::LeafNodeType::ValueOnCIter;
 
         openvdb::Vec3d pos;
         size_t index = 0;
@@ -918,11 +929,49 @@ private:
 }; // PointGenerator
 
 
+template<typename GridType>
+class NormalGenerator
+{
+public:
+    using AccessorType = typename GridType::ConstAccessor;
+    using Grad = openvdb::math::ISGradient<openvdb::math::CD_2ND>;
+
+    NormalGenerator(const AccessorType& acc): mAccessor(acc) {}
+
+    NormalGenerator(const NormalGenerator&) = delete;
+    NormalGenerator& operator=(const NormalGenerator&) = delete;
+
+    void operator()(const openvdb::Coord& ijk, openvdb::Vec3d& normal)
+    {
+        openvdb::Vec3d v{Grad::result(mAccessor, ijk)};
+        const double length = v.length();
+        if (length > 1.0e-7) {
+            v *= 1.0 / length;
+            normal = v;
+        }
+    }
+
+private:
+    const AccessorType& mAccessor;
+}; // class NormalGenerator
+
+// Specialization for PointDataGrids, for which normals are not generated
+template<>
+class NormalGenerator<openvdb::points::PointDataGrid>
+{
+public:
+    NormalGenerator(const openvdb::points::PointDataGrid::ConstAccessor&) {}
+    NormalGenerator(const NormalGenerator&) = delete;
+    NormalGenerator& operator=(const NormalGenerator&) = delete;
+    void operator()(const openvdb::Coord&, openvdb::Vec3d&) {}
+};
+
+
 template<typename GridType>
 class PointAttributeGenerator
 {
 public:
-    typedef typename GridType::ValueType ValueType;
+    using ValueType = typename GridType::ValueType;
 
     PointAttributeGenerator(
         std::vector<GLfloat>& points,
@@ -934,7 +983,7 @@ public:
         bool isLevelSet = false)
         : mPoints(points)
         , mColors(colors)
-        , mNormals(NULL)
+        , mNormals(nullptr)
         , mGrid(grid)
         , mAccessor(grid.tree())
         , mMinValue(minValue)
@@ -977,9 +1026,10 @@ public:
     inline void operator()(const tbb::blocked_range<size_t>& range) const
     {
         openvdb::Coord ijk;
-        openvdb::Vec3d pos, tmpNormal, normal(0.0, -1.0, 0.0);
+        openvdb::Vec3d pos, normal(0.0, -1.0, 0.0);
         openvdb::Vec3s color(0.9f, 0.3f, 0.3f);
         float w = 0.0;
+        NormalGenerator<GridType> computeNormal{mAccessor};
 
         size_t e1, e2, e3, voxelNum = 0;
         for (size_t n = range.begin(); n < range.end(); ++n) {
@@ -1019,19 +1069,8 @@ public:
             mColors[e3] = color[2];
 
             if (mNormals) {
-
-                if ((voxelNum % 2) == 0) {
-                    tmpNormal = openvdb::Vec3d(openvdb::math::ISGradient<
-                        openvdb::math::CD_2ND>::result(mAccessor, ijk));
-
-                    double length = tmpNormal.length();
-                    if (length > 1.0e-7) {
-                        tmpNormal *= 1.0 / length;
-                        normal = tmpNormal;
-                    }
-                }
+                if ((voxelNum % 2) == 0) { computeNormal(ijk, normal); }
                 ++voxelNum;
-
                 (*mNormals)[e1] = static_cast<GLfloat>(normal[0]);
                 (*mNormals)[e2] = static_cast<GLfloat>(normal[1]);
                 (*mNormals)[e3] = static_cast<GLfloat>(normal[2]);
@@ -1095,9 +1134,9 @@ public:
 
         //////////
 
-        typedef typename GridType::ValueType ValueType;
-        typedef typename GridType::TreeType TreeType;
-        typedef typename TreeType::template ValueConverter<bool>::Type BoolTreeT;
+        using ValueType = typename GridType::ValueType;
+        using TreeType = typename GridType::TreeType;
+        using BoolTreeT = typename TreeType::template ValueConverter<bool>::Type;
 
         const TreeType& tree = grid->tree();
         const bool isLevelSetGrid = grid->getGridClass() == openvdb::GRID_LEVEL_SET;
@@ -1253,9 +1292,9 @@ public:
     {
         using openvdb::Index64;
 
-        typedef typename GridType::ValueType ValueType;
-        typedef typename GridType::TreeType TreeType;
-        typedef typename TreeType::template ValueConverter<bool>::Type BoolTreeT;
+        using ValueType = typename GridType::ValueType;
+        using TreeType = typename GridType::TreeType;
+        using BoolTreeT = typename TreeType::template ValueConverter<bool>::Type;
 
 
         const TreeType& tree = grid->tree();
@@ -1336,11 +1375,93 @@ private:
 }; // ActiveVectorValuesOp
 
 
+class PointDataOp
+{
+public:
+    using GLfloatVec = std::vector<GLfloat>;
+    using GLuintVec = std::vector<GLuint>;
+
+private:
+    struct VectorAttributeWrapper
+    {
+        using ValueType = openvdb::Vec3f;
+
+        struct Handle
+        {
+            explicit Handle(VectorAttributeWrapper& attribute):
+                mValues(attribute.mValues), mIndices(attribute.mIndices) {}
+
+            void set(openvdb::Index offset, openvdb::Index/*unused*/, const ValueType& value)
+            {
+                if (mIndices) (*mIndices)[offset] = static_cast<GLuint>(offset);
+                offset *= 3;
+                for (int i = 0; i < 3; ++i, ++offset) { mValues[offset] = value[i]; }
+            }
+        private:
+            GLfloatVec& mValues;
+            GLuintVec* mIndices;
+        }; // struct Handle
+
+        explicit VectorAttributeWrapper(GLfloatVec& values, GLuintVec* indices = nullptr):
+            mValues(values), mIndices(indices) {}
+
+        void expand() {}
+        void compact() {}
+    private:
+        GLfloatVec& mValues;
+        GLuintVec* mIndices;
+    }; // struct VectorAttributeWrapper
+
+public:
+    explicit PointDataOp(BufferObject& buffer) : mBuffer(&buffer) {}
+
+    template<typename GridType>
+    void operator()(typename GridType::ConstPtr grid)
+    {
+        const typename GridType::TreeType& tree = grid->tree();
+
+        // obtain cumulative point offsets and total points
+        std::vector<openvdb::Index64> pointOffsets;
+        const openvdb::Index64 total = openvdb::points::getPointOffsets(pointOffsets, tree);
+
+        // @todo use glDrawArrays with GL_POINTS to avoid generating indices
+        GLfloatVec values(total * 3);
+        GLuintVec indices(total);
+
+        VectorAttributeWrapper positionWrapper{values, &indices};
+        openvdb::points::convertPointDataGridPosition(positionWrapper, *grid, pointOffsets, 0);
+
+        // gen buffers and upload data to GPU
+        mBuffer->genVertexBuffer(values);
+        mBuffer->genIndexBuffer(indices, GL_POINTS);
+
+        const auto leafIter = tree.cbeginLeaf();
+        if (!leafIter) return;
+
+        const size_t colorIdx = leafIter->attributeSet().find("Cd");
+        if (colorIdx == openvdb::points::AttributeSet::INVALID_POS) return;
+
+        const auto& colorArray = leafIter->constAttributeArray(colorIdx);
+        if (colorArray.template hasValueType<openvdb::Vec3f>()) {
+            VectorAttributeWrapper colorWrapper{values};
+            openvdb::points::convertPointDataGridAttribute(colorWrapper, tree, pointOffsets,
+                /*startOffset=*/0, static_cast<unsigned>(colorIdx));
+
+            // gen color buffer
+            mBuffer->genColorBuffer(values);
+        }
+    }
+
+private:
+    BufferObject* mBuffer;
+}; // PointDataOp
+
+
 ////////////////////////////////////////
 
 // Active value render module
 
-ActiveValueModule::ActiveValueModule(const openvdb::GridBase::ConstPtr& grid):
+VoxelModule::VoxelModule(const openvdb::GridBase::ConstPtr& grid):
     mGrid(grid),
     mIsInitialized(false)
 {
@@ -1385,26 +1506,29 @@ ActiveValueModule::ActiveValueModule(const openvdb::GridBase::ConstPtr& grid):
 
 
 void
-ActiveValueModule::init()
+VoxelModule::init()
 {
     mIsInitialized = true;
 
-    ActiveScalarValuesOp drawScalars(mInteriorBuffer, mSurfaceBuffer);
-
-    if (!util::processTypedScalarGrid(mGrid, drawScalars)) {
-
-        ActiveVectorValuesOp drawVectors(mVectorBuffer);
-
-        if(!util::processTypedVectorGrid(mGrid, drawVectors)) {
-            OPENVDB_LOG_INFO("Ignoring unrecognized grid type"
-                " during active value module initialization.");
+    if (mGrid->isType<openvdb::points::PointDataGrid>()) {
+        mSurfaceBuffer.clear();
+        PointDataOp drawPoints(mInteriorBuffer);
+        util::doProcessTypedGrid<openvdb::points::PointDataGrid>(mGrid, drawPoints);
+    } else {
+        ActiveScalarValuesOp drawScalars(mInteriorBuffer, mSurfaceBuffer);
+        if (!util::processTypedScalarOrPointDataGrid(mGrid, drawScalars)) {
+            ActiveVectorValuesOp drawVectors(mVectorBuffer);
+            if (!util::processTypedVectorGrid(mGrid, drawVectors)) {
+                OPENVDB_LOG_INFO("Ignoring unrecognized grid type " << mGrid->type()
+                    << " during active value module initialization.");
+            }
         }
     }
 }
 
 
 void
-ActiveValueModule::render()
+VoxelModule::render()
 {
     if (!mIsVisible) return;
     if (!mIsInitialized) init();
@@ -1565,6 +1689,6 @@ MeshModule::render()
 
 } // namespace openvdb_viewer
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
