@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -36,17 +36,20 @@
 #include "RenderModules.h"
 #include <openvdb/util/Formats.h> // for formattedInt()
 #include <openvdb/util/logging.h>
+#include <openvdb/points/PointDataGrid.h>
+#include <openvdb/points/PointCount.h>
 #include <openvdb/version.h> // for OPENVDB_LIBRARY_MAJOR_VERSION, etc.
 #include <tbb/atomic.h>
 #include <tbb/mutex.h>
 #include <cmath> // for fabs()
 #include <iomanip> // for std::setprecision()
 #include <iostream>
+#include <memory>
 #include <sstream>
 #include <vector>
 #include <limits>
+#include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/thread/thread.hpp>
-#include <time.h> // for nanosleep()
 
 #ifdef OPENVDB_USE_GLFW_3
 //#define GLFW_INCLUDE_GLU
@@ -68,9 +71,9 @@ namespace openvdb_viewer {
 class ViewerImpl
 {
 public:
-    typedef boost::shared_ptr<Camera> CameraPtr;
-    typedef boost::shared_ptr<ClipBox> ClipBoxPtr;
-    typedef boost::shared_ptr<RenderModule> RenderModulePtr;
+    using CameraPtr = std::shared_ptr<Camera>;
+    using ClipBoxPtr = std::shared_ptr<ClipBox>;
+    using RenderModulePtr = std::shared_ptr<RenderModule>;
 
     ViewerImpl();
 
@@ -157,8 +160,8 @@ private:
 
 namespace {
 
-ViewerImpl* sViewer = NULL;
-ThreadManager* sThreadMgr = NULL;
+ViewerImpl* sViewer = nullptr;
+ThreadManager* sThreadMgr = nullptr;
 tbb::mutex sLock;
 
 
@@ -248,9 +251,9 @@ windowRefreshCB()
 Viewer
 init(const std::string& progName, bool background)
 {
-    if (sViewer == NULL) {
+    if (sViewer == nullptr) {
         tbb::mutex::scoped_lock lock(sLock);
-        if (sViewer == NULL) {
+        if (sViewer == nullptr) {
             OPENVDB_START_THREADSAFE_STATIC_WRITE
             sViewer = new ViewerImpl;
             OPENVDB_FINISH_THREADSAFE_STATIC_WRITE
@@ -259,20 +262,20 @@ init(const std::string& progName, bool background)
     sViewer->init(progName);
 
     if (background) {
-        if (sThreadMgr == NULL) {
+        if (sThreadMgr == nullptr) {
             tbb::mutex::scoped_lock lock(sLock);
-            if (sThreadMgr == NULL) {
+            if (sThreadMgr == nullptr) {
                 OPENVDB_START_THREADSAFE_STATIC_WRITE
                 sThreadMgr = new ThreadManager;
                 OPENVDB_FINISH_THREADSAFE_STATIC_WRITE
             }
         }
     } else {
-        if (sThreadMgr != NULL) {
+        if (sThreadMgr != nullptr) {
             tbb::mutex::scoped_lock lock(sLock);
             delete sThreadMgr;
             OPENVDB_START_THREADSAFE_STATIC_WRITE
-            sThreadMgr = NULL;
+            sThreadMgr = nullptr;
             OPENVDB_FINISH_THREADSAFE_STATIC_WRITE
         }
     }
@@ -420,7 +423,7 @@ ThreadManager::doViewTask(void* arg)
     if (ThreadManager* self = static_cast<ThreadManager*>(arg)) {
         self->doView();
     }
-    return NULL;
+    return nullptr;
 }
 
 
@@ -439,7 +442,7 @@ ViewerImpl::ViewerImpl()
     , mShowInfo(true)
     , mInterrupt(false)
 #if GLFW_VERSION_MAJOR >= 3
-    , mWindow(NULL)
+    , mWindow(nullptr)
 #endif
 {
 }
@@ -488,10 +491,10 @@ ViewerImpl::getVersionString() const
     if (mDidInit) {
         ostr << ", " << "OpenGL: ";
 #if GLFW_VERSION_MAJOR >= 3
-        boost::shared_ptr<GLFWwindow> wPtr;
+        std::shared_ptr<GLFWwindow> wPtr;
         GLFWwindow* w = mWindow;
         if (!w) {
-            wPtr.reset(glfwCreateWindow(100, 100, "", NULL, NULL), &glfwDestroyWindow);
+            wPtr.reset(glfwCreateWindow(100, 100, "", nullptr, nullptr), &glfwDestroyWindow);
             w = wPtr.get();
         }
         if (w) {
@@ -518,7 +521,7 @@ ViewerImpl::getVersionString() const
 bool
 ViewerImpl::open(int width, int height)
 {
-    if (mWindow == NULL) {
+    if (mWindow == nullptr) {
         glfwWindowHint(GLFW_RED_BITS, 8);
         glfwWindowHint(GLFW_GREEN_BITS, 8);
         glfwWindowHint(GLFW_BLUE_BITS, 8);
@@ -527,14 +530,14 @@ ViewerImpl::open(int width, int height)
         glfwWindowHint(GLFW_STENCIL_BITS, 0);
 
         mWindow = glfwCreateWindow(
-            width, height, mProgName.c_str(), /*monitor=*/NULL, /*share=*/NULL);
+            width, height, mProgName.c_str(), /*monitor=*/nullptr, /*share=*/nullptr);
 
         OPENVDB_LOG_DEBUG_RUNTIME("created window " << std::hex << mWindow << std::dec
             << " from thread " << boost::this_thread::get_id());
 
-        if (mWindow != NULL) {
+        if (mWindow != nullptr) {
             // Temporarily make the new window the current context, then create a font.
-            boost::shared_ptr<GLFWwindow> curWindow(
+            std::shared_ptr<GLFWwindow> curWindow(
                 glfwGetCurrentContext(), glfwMakeContextCurrent);
             glfwMakeContextCurrent(mWindow);
             BitmapFont13::initialize();
@@ -542,7 +545,7 @@ ViewerImpl::open(int width, int height)
     }
     mCamera->setWindow(mWindow);
 
-    if (mWindow != NULL) {
+    if (mWindow != nullptr) {
         glfwSetKeyCallback(mWindow, keyCB);
         glfwSetMouseButtonCallback(mWindow, mouseButtonCB);
         glfwSetCursorPosCallback(mWindow, mousePosCB);
@@ -550,7 +553,7 @@ ViewerImpl::open(int width, int height)
         glfwSetWindowSizeCallback(mWindow, windowSizeCB);
         glfwSetWindowRefreshCallback(mWindow, windowRefreshCB);
     }
-    return (mWindow != NULL);
+    return (mWindow != nullptr);
 }
 #else // if GLFW_VERSION_MAJOR <= 2
 bool
@@ -585,7 +588,7 @@ bool
 ViewerImpl::isOpen() const
 {
 #if GLFW_VERSION_MAJOR >= 3
-    return (mWindow != NULL);
+    return (mWindow != nullptr);
 #else
     return glfwGetWindowParam(GLFW_OPENED);
 #endif
@@ -627,9 +630,9 @@ ViewerImpl::close()
     mViewportModule.reset();
     mRenderModules.clear();
 #if GLFW_VERSION_MAJOR >= 3
-    mCamera->setWindow(NULL);
+    mCamera->setWindow(nullptr);
     GLFWwindow* win = mWindow;
-    mWindow = NULL;
+    mWindow = nullptr;
     glfwDestroyWindow(win);
     OPENVDB_LOG_DEBUG_RUNTIME("destroyed window " << std::hex << win << std::dec
         << " from thread " << boost::this_thread::get_id());
@@ -747,7 +750,7 @@ ViewerImpl::view(const openvdb::GridCPtrVec& gridList)
 #if GLFW_VERSION_MAJOR >= 3
     if (glfwGetCurrentContext() == mWindow) { ///< @todo not thread-safe
         // Detach this viewer's GL context.
-        glfwMakeContextCurrent(NULL);
+        glfwMakeContextCurrent(nullptr);
         OPENVDB_LOG_DEBUG_RUNTIME("detached window " << std::hex << mWindow << std::dec
             << " from thread " << boost::this_thread::get_id());
     }
@@ -783,7 +786,7 @@ void
 ViewerImpl::render()
 {
 #if GLFW_VERSION_MAJOR >= 3
-    if (mWindow == NULL) return;
+    if (mWindow == nullptr) return;
 
     // Prepare window for rendering.
     glfwMakeContextCurrent(mWindow);
@@ -821,6 +824,13 @@ ViewerImpl::render()
         BitmapFont13::print(10, height - 13 - 30, mTransformInfo);
         BitmapFont13::print(10, height - 13 - 50, mTreeInfo);
 
+        // Indicate via their hotkeys which render modules are enabled.
+        std::string keys = "123";
+        for (auto n: {0, 1, 2}) { if (!mRenderModules[n]->visible()) keys[n] = ' '; }
+        BitmapFont13::print(width - 10 - 30, 10, keys);
+        glColor3d(0.75, 0.75, 0.75);
+        BitmapFont13::print(width - 10 - 30, 10, "123");
+
         BitmapFont13::disableFontRendering();
     }
 }
@@ -835,8 +845,7 @@ ViewerImpl::sleep(double secs)
 {
     secs = fabs(secs);
     int isecs = int(secs);
-    struct timespec sleepTime = { isecs /*sec*/, int(1.0e9 * (secs - isecs)) /*nsec*/ };
-    nanosleep(&sleepTime, /*remainingTime=*/NULL);
+    boost::this_thread::sleep(boost::posix_time::seconds(isecs));
 }
 
 
@@ -1021,7 +1030,7 @@ ViewerImpl::showNthGrid(size_t n)
     mRenderModules.clear();
     mRenderModules.push_back(RenderModulePtr(new TreeTopologyModule(mGrids[n])));
     mRenderModules.push_back(RenderModulePtr(new MeshModule(mGrids[n])));
-    mRenderModules.push_back(RenderModulePtr(new ActiveValueModule(mGrids[n])));
+    mRenderModules.push_back(RenderModulePtr(new VoxelModule(mGrids[n])));
 
     if (active.empty()) {
         for (size_t i = 1, I = mRenderModules.size(); i < I; ++i) {
@@ -1059,6 +1068,17 @@ ViewerImpl::showNthGrid(size_t n)
             << " active voxel" << (count == 1 ? "" : "s");
         mTreeInfo = ostrm.str();
     }
+    {
+        if (mGrids[n]->isType<openvdb::points::PointDataGrid>()) {
+            const openvdb::points::PointDataGrid::ConstPtr points =
+                openvdb::gridConstPtrCast<openvdb::points::PointDataGrid>(mGrids[n]);
+            const openvdb::Index64 count = openvdb::points::pointCount(points->tree());
+            std::ostringstream ostrm;
+            ostrm << " / " << openvdb::util::formattedInt(count)
+                 << " point" << (count == 1 ? "" : "s");
+            mTreeInfo.append(ostrm.str());
+        }
+    }
 
     setWindowTitle();
 }
@@ -1073,7 +1093,7 @@ ViewerImpl::keyCallback(int key, int action)
     mCamera->keyCallback(key, action);
 
 #if GLFW_VERSION_MAJOR >= 3
-    if (mWindow == NULL) return;
+    if (mWindow == nullptr) return;
     const bool keyPress = (glfwGetKey(mWindow, key) == GLFW_PRESS);
     /// @todo Should use "modifiers" argument to keyCB().
     mShiftIsDown = glfwGetKey(mWindow, GLFW_KEY_LEFT_SHIFT);
@@ -1225,6 +1245,6 @@ ViewerImpl::toggleInfoText()
 
 } // namespace openvdb_viewer
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
